@@ -27,7 +27,9 @@ static string TychoDatabaseFile("tyc_main.dat");
 static string ComponentDatabaseFile("h_dm_com.dat");
 static string OrbitalDatabase("hip_dm_o.dat");
 #else
+#ifndef MACOSX_PB
 #include <config.h>
+#endif /*MACOSX_PB*/
 static string MainDatabaseFile(HIP_DATA_DIR "/hip_main.dat");
 static string TychoDatabaseFile(HIP_DATA_DIR "/tyc_main.dat");
 static string ComponentDatabaseFile(HIP_DATA_DIR "/h_dm_com.dat");
@@ -921,7 +923,7 @@ void BuildMultistarSystemCatalog()
             {
                 MultistarSystem* multiSystem = new MultistarSystem();
                 multiSystem->nStars = 1;
-                multiSystem->stars[0] = iter;
+                multiSystem->stars[0] = &*iter;
                 starSystems.insert(MultistarSystemCatalog::value_type(iter->CCDMIdentifier, multiSystem));
             }
             else
@@ -933,7 +935,7 @@ void BuildMultistarSystemCatalog()
                 }
                 else
                 {
-                    multiSystem->stars[multiSystem->nStars] = iter;
+                    multiSystem->stars[multiSystem->nStars] = &*iter;
                     multiSystem->nStars++;
                 }
             }
@@ -1162,7 +1164,7 @@ int main(int argc, char* argv[])
         for (vector<HipparcosStar>::iterator iter = stars.begin();
              iter != stars.end(); iter++)
         {
-            starIndex.insert(starIndex.end(), iter);
+            starIndex.insert(starIndex.end(), &*iter);
         }
 
         HIPCatalogComparePredicate pred;
@@ -1250,7 +1252,7 @@ int main(int argc, char* argv[])
     if (verbose>0)
         ShowStarsWithComponents();
 
-    char* outputFile = "stars.dat";
+    const char* outputFile = "stars.dat";
     if (argv[optind])
         outputFile = argv[optind];
 
