@@ -115,7 +115,7 @@ void indent()
 
 void logChunk(uint16 chunkType, int chunkSize)
 {
-    char* name = NULL;
+    const char* name = NULL;
 
     switch (chunkType)
     {
@@ -330,7 +330,7 @@ void readTextureCoordArray(ifstream& in, M3DTriangleMesh* triMesh)
     {
         float u = readFloat(in);
         float v = readFloat(in);
-        triMesh->addTexCoord(Point2f(u, v));
+        triMesh->addTexCoord(Point2f(u, -v));
     }
 }
 
