@@ -116,6 +116,11 @@ float astro::AUtoKilometers(float au)
     return au * (float) KM_PER_AU;
 }
 
+double astro::AUtoKilometers(double au)
+{
+    return au * (double) KM_PER_AU;
+}
+
 float astro::kilometersToAU(float km)
 {
     return km / (float) KM_PER_AU;
@@ -140,13 +145,13 @@ void astro::decimalToDegMinSec(double angle, int& hours, int& minutes, double& s
 {
     double A, B, C;
 
-    hours = (int)angle;
+    hours = (int) angle;
 
-    A = angle - (double)hours;
+    A = angle - (double) hours;
     B = A * 60.0;
-    minutes = (int)B;
-    C = B - (double)minutes;
-    seconds = (int)(C * 60.0);
+    minutes = (int) B;
+    C = B - (double) minutes;
+    seconds = C * 60.0;
 }
 
 double astro::degMinSecToDecimal(int hours, int minutes, double seconds)
@@ -185,7 +190,7 @@ double astro::kilometersToMicroLightYears(double km)
 
 float astro::microLightYearsToAU(float ly)
 {
-    return ly * AU_PER_LY * 1e-6f;
+    return ly * (float) AU_PER_LY * 1e-6f;
 }
 
 double astro::microLightYearsToAU(double ly)
@@ -408,7 +413,7 @@ bool astro::parseDate(const string& s, astro::Date& date)
             else
                 maxDay = 28;
         }
-        if (day > maxDay || day < 1)
+        if (day > (unsigned int)maxDay || day < 1)
             return false;
 
         date.year = year;
