@@ -13,6 +13,9 @@
 #include <cassert>
 #include <celmath/mathlib.h>
 #include <celutil/util.h>
+#include <celutil/utf8.h>
+#include "mesh.h"
+#include "meshmanager.h"
 #include "body.h"
 
 using namespace std;
@@ -31,8 +34,11 @@ RotationElements::RotationElements() :
 
 Body::Body(PlanetarySystem* _system) :
     orbit(NULL),
-    orbitFrame(astro::Equatorial),
+    orbitRefPlane(astro::BodyEquator),
+    radius(10000.0f),
+    mass(0.0f),
     oblateness(0),
+    albedo(0.5),
     orientation(1.0f),
     // Ugh.  Numeric_limits class is missing from g++
     // protos(-numeric_limits<double>::infinity()),
@@ -40,12 +46,15 @@ Body::Body(PlanetarySystem* _system) :
     // Do it the ugly way instead:
     protos(-1.0e+50),
     eschatos(1.0e+50),
-    mesh(InvalidResource),
+    model(InvalidResource),
     surface(Color(1.0f, 1.0f, 1.0f)),
     atmosphere(NULL),
     rings(NULL),
     satellites(NULL),
-    classification(Unknown)
+    classification(Unknown),
+    altSurfaces(NULL),
+    locations(NULL),
+    locationsComputed(false)
 {
     system = _system;
 }
@@ -91,26 +100,18 @@ void Body::setOrbit(Orbit* _orbit)
 }
 
 
-astro::CoordinateSystem Body::getOrbitFrame() const
+astro::ReferencePlane Body::getOrbitReferencePlane() const
 {
-    return orbitFrame;
+    return orbitRefPlane;
 }
 
 
-void Body::setOrbitFrame(astro::CoordinateSystem _orbitFrame)
+void Body::setOrbitReferencePlane(astro::ReferencePlane refPlane)
 {
-    switch (_orbitFrame)
-    {
-    case astro::Equatorial:
-    case astro::Ecliptical:
-    case astro::Geographic:
-        orbitFrame = _orbitFrame;
-        break;
-    default:
-        assert(0);
-    }
+    orbitRefPlane = refPlane;
 }
 
+
 float Body::getRadius() const
 {
     return radius;
@@ -123,6 +124,18 @@ void Body::setRadius(float _radius)
 }
 
 
+// For an irregular object, the radius is defined to be the largest semi-axis
+// of the axis-aligned bounding box.  The radius of the smallest sphere containing
+// the object is potentially larger by a factor of sqrt(3)
+float Body::getBoundingRadius() const
+{
+    if (model == InvalidResource)
+        return radius;
+    else
+        return radius * 1.7320508f; // sqrt(3)
+}
+
+
 float Body::getMass() const
 {
     return mass;
@@ -201,14 +214,14 @@ void Body::setSurface(const Surface& surf)
 }
 
 
-ResourceHandle Body::getMesh() const
+ResourceHandle Body::getModel() const
 {
-    return mesh;
+    return model;
 }
 
-void Body::setMesh(ResourceHandle _mesh)
+void Body::setModel(ResourceHandle _model)
 {
-    mesh = _mesh;
+    model = _model;
 }
 
 
@@ -262,10 +275,24 @@ Mat4d Body::getLocalToHeliocentric(double when) const
         rotationElements.precessionRate * (when - astro::J2000);
 
     Point3d pos = orbit->positionAtTime(when);
-    Mat4d frame = 
-        Mat4d::xrotation(-rotationElements.obliquity) *
-        Mat4d::yrotation(-ascendingNode) *
-        Mat4d::translation(pos);
+    Mat4d frame;
+
+    switch (orbitRefPlane)
+    {
+    case astro::BodyEquator:
+        frame = (Mat4d::xrotation(-rotationElements.obliquity) *
+                 Mat4d::yrotation(-ascendingNode) *
+                 Mat4d::translation(pos));
+        break;
+    case astro::Ecliptic_J2000:
+        frame = Mat4d::translation(pos);
+        break;
+    case astro::Equator_J2000:
+        frame = Mat4d::translation(pos);
+        break;
+    default:
+        assert(0);
+    }
  
     // Recurse up the hierarchy . . .
     if (system != NULL && system->getPrimaryBody() != NULL)
@@ -299,7 +326,7 @@ Quatd Body::getEclipticalToEquatorial(double when) const
 }
 
 
-Quatd Body::getEclipticalToGeographic(double when)
+Quatd Body::getEclipticalToGeographic(double when) const
 {
     return getEquatorialToGeographic(when) * getEclipticalToEquatorial(when);
 }
@@ -310,7 +337,7 @@ Quatd Body::getEclipticalToGeographic(double when)
 // meridian, and z-axis at a right angle the xy plane.  An object with
 // constant geographic coordinates will thus remain fixed with respect
 // to a point on the surface of the body.
-Quatd Body::getEquatorialToGeographic(double when)
+Quatd Body::getEquatorialToGeographic(double when) const
 {
     double t = when - rotationElements.epoch;
     double rotations = t / (double) rotationElements.period;
@@ -328,13 +355,44 @@ Quatd Body::getEquatorialToGeographic(double when)
 }
 
 
-Mat4d Body::getGeographicToHeliocentric(double when)
+Mat4d Body::getGeographicToHeliocentric(double when) const
 {
     return getEquatorialToGeographic(when).toMatrix4() *
         getLocalToHeliocentric(when);
 }
 
 
+Vec3f Body::planetocentricToCartesian(float lon, float lat, float alt) const
+{
+    float phi = -degToRad(lat) + (float) PI / 2;
+    float theta = degToRad(lon) - (float) PI;
+
+    Vec3f pos((float) (cos(theta) * sin(phi)),
+              (float) (cos(phi)),
+              (float) (-sin(theta) * sin(phi)));
+
+    return pos * (getRadius() + alt);
+}
+
+
+Vec3f Body::planetocentricToCartesian(const Vec3f& lonLatAlt) const
+{
+    return planetocentricToCartesian(lonLatAlt.x, lonLatAlt.y, lonLatAlt.z);
+}
+
+
+Vec3f Body::cartesianToPlanetocentric(const Vec3f& v) const
+{
+    Vec3f w = v;
+    w.normalize();
+
+    double lat = (float) PI / 2.0f - acos(w.y);
+    double lon = atan2(w.z, -w.x);
+
+    return Vec3f((float) lon, (float) lat, v.length() - getRadius());
+}
+
+
 bool Body::extant(double t) const
 {
     return t >= protos && t < eschatos;
@@ -348,6 +406,13 @@ void Body::setLifespan(double begin, double end)
 }
 
 
+void Body::getLifespan(double& begin, double& end) const
+{
+    begin = protos;
+    end = eschatos;
+}
+
+
 #define SOLAR_IRRADIANCE   1367.6
 #define SOLAR_POWER           3.8462e26
 
@@ -418,6 +483,123 @@ void Body::setInfoURL(const string& _infoURL)
 }
 
 
+Surface* Body::getAlternateSurface(const string& name) const
+{
+    if (altSurfaces == NULL)
+        return NULL;
+
+    AltSurfaceTable::iterator iter = altSurfaces->find(name);
+    if (iter == altSurfaces->end())
+        return NULL;
+    else
+        return iter->second;
+}
+
+
+void Body::addAlternateSurface(const string& name, Surface* surface)
+{
+    if (altSurfaces == NULL)
+        altSurfaces = new AltSurfaceTable();
+
+    //altSurfaces->insert(AltSurfaceTable::value_type(name, surface));
+    (*altSurfaces)[name] = surface;
+}
+
+
+vector<string>* Body::getAlternateSurfaceNames() const
+{
+    vector<string>* names = new vector<string>();
+    if (altSurfaces != NULL)
+    {
+        for (AltSurfaceTable::const_iterator iter = altSurfaces->begin();
+             iter != altSurfaces->end(); iter++)
+        {
+            names->insert(names->end(), iter->first);
+        }
+    }
+
+    return names;
+}
+
+
+void Body::addLocation(Location* loc)
+{
+    assert(loc != NULL);
+    if (loc == NULL)
+        return;
+
+    if (locations == NULL)
+        locations = new vector<Location*>();
+    locations->insert(locations->end(), loc);
+    loc->setParentBody(this);
+}
+
+
+vector<Location*>* Body::getLocations() const
+{
+    return locations;
+}
+
+
+Location* Body::findLocation(const string& name) const
+{
+    if (locations == NULL)
+        return NULL;
+
+    for (vector<Location*>::const_iterator iter = locations->begin();
+         iter != locations->end(); iter++)
+    {
+        if (!UTF8StringCompare(name, (*iter)->getName()))
+            return *iter;
+    }
+
+    return NULL;
+}
+
+
+// Compute the positions of locations on an irregular object using ray-mesh
+// intersections.  This is not automatically done when a location is added
+// because it would force the loading of all meshes for objects with 
+// defined locations; on-demand (i.e. when the object becomes visible to
+// a user) loading of meshes is preferred.
+void Body::computeLocations()
+{
+    if (locationsComputed)
+        return;
+
+    locationsComputed = true;
+
+    // No work to do if there's no mesh, or if the mesh cannot be loaded
+    if (model == InvalidResource)
+        return;
+    Model* m = GetModelManager()->find(model);
+    if (m == NULL)
+        return;
+
+    // TODO: Implement separate radius and bounding radius so that this hack is
+    // not necessary.
+    double boundingRadius = 2.0;
+
+    for (vector<Location*>::const_iterator iter = locations->begin();
+         iter != locations->end(); iter++)
+    {
+        Vec3f v = (*iter)->getPosition();
+        float alt = v.length() - radius;
+        if (alt != -radius)
+            v.normalize();
+        v *= (float) boundingRadius;
+
+        Ray3d ray(Point3d(v.x, v.y, v.z), Vec3d(-v.x, -v.y, -v.z));
+        double t = 0.0;
+        if (m->pick(ray, t))
+        {
+            v *= (float) ((1.0 - t) * radius + alt);
+            (*iter)->setPosition(v);
+        }
+    }
+}
+
+
 /**** Implementation of PlanetarySystem ****/
 
 PlanetarySystem::PlanetarySystem(Body* _primary) : primary(_primary)
@@ -454,12 +636,26 @@ void PlanetarySystem::removeBody(Body* body)
 }
 
 
+void PlanetarySystem::replaceBody(Body* oldBody, Body* newBody)
+{
+    for (vector<Body*>::iterator iter = satellites.begin();
+         iter != satellites.end(); iter++)
+    {
+        if (*iter == oldBody)
+        {
+            *iter = newBody;
+            break;
+        }
+    }
+}
+
+
 Body* PlanetarySystem::find(string _name, bool deepSearch) const
 {
     for (vector<Body*>::const_iterator iter = satellites.begin();
          iter != satellites.end(); iter++)
     {
-        if (compareIgnoringCase((*iter)->getName(), _name) == 0)
+        if (UTF8StringCompare((*iter)->getName(), _name) == 0)
         {
             return *iter;
         }
@@ -493,3 +689,23 @@ bool PlanetarySystem::traverse(TraversalFunc func, void* info) const
     return true;
 }
 
+std::vector<std::string> PlanetarySystem::getCompletion(const std::string& _name, bool rec) const
+{
+    std::vector<std::string> completion;
+      
+    for (vector<Body*>::const_iterator iter = satellites.begin();
+         iter != satellites.end(); iter++)
+    {
+        if (UTF8StringCompare((*iter)->getName(), _name, _name.length()) == 0)
+        {
+            completion.push_back((*iter)->getName());
+        }
+        if (rec && (*iter)->getSatellites() != NULL)
+        {
+            std::vector<std::string> bodies = (*iter)->getSatellites()->getCompletion(_name);
+            completion.insert(completion.end(), bodies.begin(), bodies.end());
+        }
+    }
+
+    return completion;
+}
