@@ -12,11 +12,13 @@
 
 #include <string>
 #include <vector>
+#include <map>
 #include <celmath/quaternion.h>
 #include <celengine/surface.h>
 #include <celengine/atmosphere.h>
 #include <celengine/orbit.h>
 #include <celengine/star.h>
+#include <celengine/location.h>
 #include <celengine/astro.h>
 
 
@@ -34,6 +36,7 @@ class PlanetarySystem
     Body* getBody(int i) const { return satellites[i]; };
     void addBody(Body* body);
     void removeBody(Body* body);
+    void replaceBody(Body* oldBody, Body* newBody);
 
     enum TraversalResult
     {
@@ -45,6 +48,7 @@ class PlanetarySystem
 
     bool traverse(TraversalFunc, void*) const;
     Body* find(std::string, bool deepSearch = false) const;
+    std::vector<std::string> getCompletion(const std::string& _name, bool rec = true) const;
 
  private:
     Star* star;
@@ -109,8 +113,8 @@ class Body
     void setName(const std::string);
     Orbit* getOrbit() const;
     void setOrbit(Orbit*);
-    astro::CoordinateSystem getOrbitFrame() const;
-    void setOrbitFrame(astro::CoordinateSystem);
+    astro::ReferencePlane getOrbitReferencePlane() const;
+    void setOrbitReferencePlane(astro::ReferencePlane);
     RotationElements getRotationElements() const;
     void setRotationElements(const RotationElements&);
     float getRadius() const;
@@ -131,14 +135,16 @@ class Body
     PlanetarySystem* getSatellites() const;
     void setSatellites(PlanetarySystem*);
 
+    float getBoundingRadius() const;
+
     RingSystem* getRings() const;
     void setRings(const RingSystem&);
     const Atmosphere* getAtmosphere() const;
     Atmosphere* getAtmosphere();
     void setAtmosphere(const Atmosphere&);
 
-    void setMesh(ResourceHandle);
-    ResourceHandle getMesh() const;
+    void setModel(ResourceHandle);
+    ResourceHandle getModel() const;
     void setSurface(const Surface&);
     const Surface& getSurface() const;
     Surface& getSurface();
@@ -154,20 +160,34 @@ class Body
 
     Mat4d getLocalToHeliocentric(double) const;
     Point3d getHeliocentricPosition(double) const;
-    Quatd getEquatorialToGeographic(double);
+    Quatd getEquatorialToGeographic(double) const;
     Quatd getEclipticalToEquatorial(double) const;
-    Quatd getEclipticalToGeographic(double);
-    Mat4d getGeographicToHeliocentric(double);
+    Quatd getEclipticalToGeographic(double) const;
+    Mat4d getGeographicToHeliocentric(double) const;
+
+    Vec3f planetocentricToCartesian(float lon, float lat, float alt) const;
+    Vec3f planetocentricToCartesian(const Vec3f& lonLatAlt) const;
+    Vec3f cartesianToPlanetocentric(const Vec3f& v) const;
 
     bool extant(double) const;
     void setLifespan(double, double);
+    void getLifespan(double&, double&) const;
+
+    Surface* getAlternateSurface(const std::string&) const;
+    void addAlternateSurface(const std::string&, Surface*);
+    std::vector<std::string>* getAlternateSurfaceNames() const;
+
+    std::vector<Location*>* getLocations() const;
+    void addLocation(Location*);
+    Location* findLocation(const std::string&) const;
+    void computeLocations();
 
  private:
     std::string name;
 
     PlanetarySystem* system;
     Orbit* orbit;
-    astro::CoordinateSystem orbitFrame;
+    astro::ReferencePlane orbitRefPlane;
     RotationElements rotationElements;
 
     float radius;
@@ -179,7 +199,7 @@ class Body
     double protos;
     double eschatos;
 
-    ResourceHandle mesh;
+    ResourceHandle model;
     Surface surface;
 
     Atmosphere* atmosphere;
@@ -190,6 +210,12 @@ class Body
     int classification;
 
     std::string infoURL;
+
+    typedef std::map<std::string, Surface*> AltSurfaceTable;
+    AltSurfaceTable *altSurfaces;
+
+    std::vector<Location*>* locations;
+    mutable bool locationsComputed;
 };
 
 #endif // _BODY_H_
