@@ -10,6 +10,7 @@
 // of the License, or (at your option) any later version.
 
 #include <algorithm>
+#include <cstdio>
 
 #include "celestia.h"
 // Ugh . . . the C++ standard says that stringstream should be in
@@ -23,6 +24,7 @@
 #include <celutil/util.h>
 #include <celutil/debug.h>
 #include <celmath/mathlib.h>
+#include <celengine/astro.h>
 #include "astro.h"
 #include "cmdparser.h"
 
@@ -162,7 +164,21 @@ Command* CommandParser::parseCommand()
         double value = 0.0;
         string name;
         paramList->getString("name", name);
-        paramList->getNumber("value", value);
+        if (!paramList->getNumber("value", value))
+        {
+            // Some values may be specified via strings
+            string valstr;
+            if (paramList->getString("value", valstr))
+            {
+                if (compareIgnoringCase(valstr, "fuzzypoints") == 0)
+                    value = (double) Renderer::FuzzyPointStars;
+                else if (compareIgnoringCase(valstr, "points") == 0)
+                    value = (double) Renderer::PointStars;
+                else if (compareIgnoringCase(valstr, "scaleddiscs") == 0)
+                    value = (double) Renderer::ScaledDiscStars;
+            }
+        }
+
         cmd = new CommandSet(name, value);
     }
     else if (commandName == "select")
@@ -184,6 +200,12 @@ Command* CommandParser::parseCommand()
 
         cmd = new CommandSetFrame(coordSys, refName, targetName);
     }
+    else if (commandName == "setsurface")
+    {
+        string name;
+        paramList->getString("name", name);
+        cmd = new CommandSetSurface(name);
+    }
     else if (commandName == "goto")
     {
         double t = 1.0;
@@ -228,20 +250,43 @@ Command* CommandParser::parseCommand()
         double t = 1.0;
         paramList->getNumber("time", t);
         Vec3d pos(0, 1, 0);
-        paramList->getVector("position", pos);
-        pos = pos * astro::kilometersToMicroLightYears(1.0);
-        double xrot = 0.0;
-        paramList->getNumber("xrot", xrot);
-        double yrot = 0.0;
-        paramList->getNumber("yrot", yrot);
-        double zrot = 0.0;
-        paramList->getNumber("zrot", zrot);
-        zrot = degToRad(zrot);
-        Quatf rotation = Quatf::xrotation((float) degToRad(xrot)) *
-            Quatf::yrotation((float) degToRad(yrot)) *
-            Quatf::zrotation((float) degToRad(zrot));
-        cmd = new CommandGotoLocation(t, Point3d(0.0, 0.0, 0.0) + pos,
-                                      rotation);
+        if (paramList->getVector("position", pos))
+        {
+            pos = pos * astro::kilometersToMicroLightYears(1.0);
+            double xrot = 0.0;
+            paramList->getNumber("xrot", xrot);
+            double yrot = 0.0;
+            paramList->getNumber("yrot", yrot);
+            double zrot = 0.0;
+            paramList->getNumber("zrot", zrot);
+            zrot = degToRad(zrot);
+            Quatf rotation = Quatf::xrotation((float) degToRad(xrot)) *
+                Quatf::yrotation((float) degToRad(yrot)) *
+                Quatf::zrotation((float) degToRad(zrot));
+            cmd = new CommandGotoLocation(t, Point3d(0.0, 0.0, 0.0) + pos,
+                                          rotation);
+        }
+        else
+        {
+            std::string x, y, z;
+            paramList->getString("x", x);
+            paramList->getString("y", y);
+            paramList->getString("z", z);
+            double ow, ox, oy, oz;
+            paramList->getNumber("ow", ow);
+            paramList->getNumber("ox", ox);
+            paramList->getNumber("oy", oy);
+            paramList->getNumber("oz", oz);
+            Quatf orientation((float)ow, (float)ox, (float)oy, (float)oz);
+            cmd = new CommandGotoLocation(t, Point3d((double)BigFix(x), (double)BigFix(y), (double)BigFix(z)),
+                                          orientation);
+        }
+    }
+    else if (commandName == "seturl")
+    {
+        std::string url;
+        paramList->getString("url", url);
+        cmd = new CommandSetUrl(url);
     }
     else if (commandName == "center")
     {
@@ -349,7 +394,16 @@ Command* CommandParser::parseCommand()
     else if (commandName == "time")
     {
         double jd = 2451545;
-        paramList->getNumber("jd", jd);
+        if (!paramList->getNumber("jd", jd))
+        {
+            std::string utc;
+            paramList->getString("utc", utc);
+            astro::Date date = astro::Date(0.0);
+            sscanf(utc.c_str(), "%d-%d-%dT%d:%d:%lf",
+                &date.year, &date.month, &date.day,
+                &date.hour, &date.minute, &date.seconds);
+            jd = (double)date;
+        }
         cmd = new CommandSetTime(jd);
     }
     else if (commandName == "timerate")
@@ -401,19 +455,44 @@ Command* CommandParser::parseCommand()
     else if (commandName == "setposition")
     {
         Vec3d base, offset;
-        paramList->getVector("base", base);
-        paramList->getVector("offset", offset);
-        cmd = new CommandSetPosition(astro::universalPosition(Point3d(offset.x, offset.y, offset.z),
-                                                              Point3f((float) base.x, (float) base.y, (float) base.z)));
+        if (paramList->getVector("base", base))
+        {
+            paramList->getVector("offset", offset);
+            cmd = new CommandSetPosition(astro::universalPosition(Point3d(offset.x, offset.y, offset.z),
+                                                                  Point3f((float) base.x, (float) base.y, (float) base.z)));
+        }
+        else
+        {
+            std::string x, y, z;
+            paramList->getString("x", x);
+            paramList->getString("y", y);
+            paramList->getString("z", z);
+            cmd = new CommandSetPosition(UniversalCoord(BigFix(x), BigFix(y), BigFix(z)));
+        }
     }
     else if (commandName == "setorientation")
     {
         Vec3d axis;
         double angle;
-        paramList->getNumber("angle", angle);
-        paramList->getVector("axis", axis);
-        cmd = new CommandSetOrientation(Vec3f((float) axis.x, (float) axis.y, (float) axis.z),
-                                        (float) degToRad(angle));
+        if (paramList->getNumber("angle", angle))
+        {
+            paramList->getVector("axis", axis);
+            cmd = new CommandSetOrientation(Vec3f((float) axis.x, (float) axis.y, (float) axis.z),
+                                            (float) degToRad(angle));
+        }
+        else
+        {
+            double ow, ox, oy, oz;
+            paramList->getNumber("ow", ow);
+            paramList->getNumber("ox", ox);
+            paramList->getNumber("oy", oy);
+            paramList->getNumber("oz", oz);
+            Quatf orientation((float)ow, (float)ox, (float)oy, (float)oz);
+            Vec3f axis;
+            float angle;
+            orientation.getAxisAngle(axis, angle);
+            cmd = new CommandSetOrientation(axis, angle);
+        }
     }
     else if (commandName == "lookback")
     {
@@ -503,6 +582,10 @@ Command* CommandParser::parseCommand()
         paramList->getString("object", object);
         cmd = new CommandUnmark(object);
     }
+    else if (commandName == "unmarkall")
+    {
+        cmd = new CommandUnmarkAll();
+    }
     else
     {
         error("Unknown command name '" + commandName + "'");
@@ -549,14 +632,20 @@ int parseRenderFlags(string s)
                 flags |= Renderer::ShowEclipseShadows;
             else if (compareIgnoringCase(name, "ringshadows") == 0)
                 flags |= Renderer::ShowRingShadows;
-            else if (compareIgnoringCase(name, "pointstars") == 0)
-                flags |= Renderer::ShowStarsAsPoints;
-            else if (compareIgnoringCase(name, "ringshadows") == 0)
-                flags |= Renderer::ShowRingShadows;
             else if (compareIgnoringCase(name, "comettails") == 0)
                 flags |= Renderer::ShowCometTails;
             else if (compareIgnoringCase(name, "boundaries") == 0)
                 flags |= Renderer::ShowBoundaries;
+            else if (compareIgnoringCase(name, "markers") == 0)
+                flags |= Renderer::ShowMarkers;
+            else if (compareIgnoringCase(name, "automag") == 0)
+                flags |= Renderer::ShowAutoMag;
+            else if (compareIgnoringCase(name, "atmospheres") == 0)
+                flags |= Renderer::ShowAtmospheres;
+            else if (compareIgnoringCase(name, "grid") == 0)
+                flags |= Renderer::ShowCelestialSphere;
+            else if (compareIgnoringCase(name, "partialtrajectories") == 0)
+                flags |= Renderer::ShowPartialTrajectories;
 
             ttype = tokenizer.nextToken();
             if (ttype == Tokenizer::TokenBar)
@@ -597,6 +686,8 @@ int parseLabelFlags(string s)
                 flags |= Renderer::SpacecraftLabels;
             else if (compareIgnoringCase(name, "asteroids") == 0)
                 flags |= Renderer::AsteroidLabels;
+	    else if (compareIgnoringCase(name, "comets") == 0)
+		flags |= Renderer::CometLabels;
             else if (compareIgnoringCase(name, "constellations") == 0)
                 flags |= Renderer::ConstellationLabels;
             else if (compareIgnoringCase(name, "stars") == 0)
