@@ -9,6 +9,7 @@
 
 #include <iostream>
 #include <celutil/util.h>
+#include <celestia/celestiacore.h>
 #include "astro.h"
 #include "command.h"
 #include "execution.h"
@@ -127,6 +128,18 @@ void CommandGotoLocation::process(ExecutionEnvironment& env)
     env.getSimulation()->gotoLocation(to, gotoTime);
 }
 
+/////////////////////////////
+// SetUrl
+
+CommandSetUrl::CommandSetUrl(const std::string& _url) :
+    url(_url)
+{
+}
+
+void CommandSetUrl::process(ExecutionEnvironment& env)
+{
+    env.getCelestiaCore()->goToUrl(url);
+}
 
 
 ////////////////
@@ -214,7 +227,7 @@ void CommandLock::process(ExecutionEnvironment& env)
 
 
 ////////////////
-// Select command: select a body
+// Setframe command
 
 CommandSetFrame::CommandSetFrame(astro::CoordinateSystem _coordSys,
                                  const string& refName,
@@ -233,9 +246,23 @@ void CommandSetFrame::process(ExecutionEnvironment& env)
 }
 
 
+////////////////
+// SetSurface command: select an alternate surface to show
+
+CommandSetSurface::CommandSetSurface(const string& _surfaceName) :
+    surfaceName(_surfaceName)
+{
+}
+
+void CommandSetSurface::process(ExecutionEnvironment& env)
+{
+    env.getSimulation()->getActiveObserver()->setDisplayedSurface(surfaceName);
+}
+
 
 ////////////////
-// Cancel command: cancel a follow or goto command
+// Cancel command: stop all motion, set the coordinate system to absolute,
+//                 and cancel any tracking
 
 CommandCancel::CommandCancel()
 {
@@ -244,6 +271,8 @@ CommandCancel::CommandCancel()
 void CommandCancel::process(ExecutionEnvironment& env)
 {
     env.getSimulation()->cancelMotion();
+    env.getSimulation()->setFrame(FrameOfReference());
+    env.getSimulation()->setTrackedObject(Selection());
 }
 
 
@@ -528,6 +557,14 @@ void CommandSet::process(ExecutionEnvironment& env)
         if (env.getRenderer() != NULL)
             env.getRenderer()->setDistanceLimit((float) value);
     }
+    else if (compareIgnoringCase(name, "StarStyle") == 0)
+    {
+        // The cast from double to an enum requires an intermediate cast to int
+        // Probably shouldn't be doing this at all, but other alternatives
+        // are more trouble than they're worth.
+        if (env.getRenderer() != NULL)
+            env.getRenderer()->setStarStyle((Renderer::StarStyle) (int) value);
+    }
 }
 
 
@@ -575,6 +612,20 @@ void CommandUnmark::process(ExecutionEnvironment& env)
 
 
 
+///////////////
+// Unmarkall command - clear all current markers
+
+CommandUnmarkAll::CommandUnmarkAll()
+{
+}
+
+void CommandUnmarkAll::process(ExecutionEnvironment& env)
+{
+    if (env.getSimulation()->getUniverse() != NULL)
+        env.getSimulation()->getUniverse()->unmarkAll();
+}
+
+
 ////////////////
 // Preload textures command
 
@@ -586,11 +637,11 @@ CommandPreloadTextures::CommandPreloadTextures(const string& _name) :
 void CommandPreloadTextures::process(ExecutionEnvironment& env)
 {
     Selection target = env.getSimulation()->findObjectFromPath(name);
-    if (target.body == NULL)
+    if (target.body() == NULL)
         return;
 
     if (env.getRenderer() != NULL)
-        env.getRenderer()->loadTextures(target.body);
+        env.getRenderer()->loadTextures(target.body());
 }
 
 
