@@ -124,6 +124,16 @@ class CommandGotoLocation : public InstantaneousCommand
     Quatf rotation;
 };
 
+class CommandSetUrl : public InstantaneousCommand
+{
+ public:
+    CommandSetUrl(const std::string& _url);
+    void process(ExecutionEnvironment&);
+
+ private:
+    std::string url;
+};
+
 
 class CommandCenter : public InstantaneousCommand
 {
@@ -206,6 +216,17 @@ class CommandSetFrame : public InstantaneousCommand
 };
 
 
+class CommandSetSurface : public InstantaneousCommand
+{
+ public:
+    CommandSetSurface(const std::string&);
+    void process(ExecutionEnvironment&);
+
+ private:
+    std::string surfaceName;
+};
+
+
 class CommandCancel : public InstantaneousCommand
 {
  public:
@@ -447,6 +468,17 @@ class CommandUnmark : public InstantaneousCommand
 };
 
 
+class CommandUnmarkAll : public InstantaneousCommand
+{
+ public:
+    CommandUnmarkAll();
+    void process(ExecutionEnvironment&);
+
+ private:
+    int dummy;   // Keep the class from having zero size
+};
+
+
 class Execution;
 
 class RepeatCommand : public Command
