@@ -9,10 +9,13 @@
 
 #include <cassert>
 #include <vector>
+#include <fstream>
+#include <iomanip>
 #include <celmath/mathlib.h>
 #include "astro.h"
 #include "customorbit.h"
 #include "vsop87.h"
+#include "jpleph.h"
 
 using namespace std;
 
@@ -32,6 +35,10 @@ static const double SaturnRadius = 60330.0;
 // the apocenter distance computed from the mean elements.
 static const double BoundingRadiusSlack = 1.2;
 
+static bool jplephInitialized = false;
+static JPLEphemeris* jpleph = NULL;
+
+
 double gPlanetElements[8][9];
 double gElements[8][23] = {
 	{   /*     mercury... */
@@ -115,6 +122,20 @@ double gElements[8][23] = {
 	}
 };
 
+
+// Useful version of trig functions which operate on values in degrees instead
+// of radians.
+static double sinD(double theta)
+{
+    return sin(degToRad(theta));
+}
+
+static double cosD(double theta)
+{
+    return cos(degToRad(theta));
+}
+
+
 static double Obliquity(double t)
 {
     // Parameter t represents the Julian centuries elapsed since 1900.
@@ -1248,6 +1269,227 @@ class PlutoOrbit : public CachingOrbit
     };
 };
 
+
+// Compute for mean anomaly M the point on the ellipse with
+// semimajor axis a and eccentricity e.  This helper function assumes
+// a low eccentricity; orbit.cpp has functions appropriate for solving
+// Kepler's equation for larger values of e.
+static Point3d ellipsePosition(double a, double e, double M)
+{
+    // Solve Kepler's equation--for a low eccentricity orbit, just a few
+    // iterations is enough.
+    double E = M;
+    for (int k = 0; k < 3; k++)
+        E = M + e * sin(E);
+
+    return Point3d(a * (cos(E) - e),
+                   0.0,
+                   a * sqrt(1 - square(e)) * -sin(E));
+}
+
+
+class PhobosOrbit : public CachingOrbit
+{
+    Point3d computePosition(double jd) const
+    {
+        double epoch = 2433283.0 - 0.5; // 00:00 1 Jan 1950
+        double a     = 9380.0;
+        double e     = 0.0151;
+        double w0    = 150.247;
+        double M0    =  92.474;
+        double i     =   1.075;
+        double node0 = 164.931;
+        double n     = 1128.8444155;
+        double Pw    =  1.131;
+        double Pnode =  2.262;
+
+        double refplane_RA  = 317.724;
+        double refplane_Dec =  52.924;
+        double marspole_RA  = 317.681;
+        double marspole_Dec =  52.886;
+
+        double t = jd - epoch;
+        t += 10.5 / 1440.0;     // light time correction?
+        double T = t / 365.25;
+
+        double dnode = 360.0 / Pnode;
+        double dw    = 360.0 / Pw;
+        double node = degToRad(node0 + T * dnode);
+        double w    = degToRad(w0 + T * dw - T * dnode);
+        double M    = degToRad(M0 + t * n  - T * dw);
+
+        Point3d p = ellipsePosition(a, e, M);
+
+        // Orientation of the orbital plane with respect to the Laplacian plane
+        Mat3d Rorbit     = (Mat3d::yrotation(node) *
+                            Mat3d::xrotation(degToRad(i)) *
+                            Mat3d::yrotation(w));
+
+        // Rotate to the Earth's equatorial plane
+        double N = degToRad(refplane_RA);
+        double J = degToRad(90 - refplane_Dec);
+        Mat3d RLaplacian = (Mat3d::yrotation( N) *
+                            Mat3d::xrotation( J) *
+                            Mat3d::yrotation(-N));
+
+        // Rotate to the Martian equatorial plane
+        N = degToRad(marspole_RA);
+        J = degToRad(90 - marspole_Dec);
+        Mat3d RMars_eq   = (Mat3d::yrotation( N) *
+                            Mat3d::xrotation(-J) *
+                            Mat3d::yrotation(-N));
+
+        return RMars_eq * (RLaplacian * (Rorbit * p));
+    }
+    
+    double getPeriod() const
+    {
+        return 0.319;
+    }
+
+    double getBoundingRadius() const
+    {
+        return 9380 * BoundingRadiusSlack;
+    }
+};
+
+
+class DeimosOrbit : public CachingOrbit
+{
+    Point3d computePosition(double jd) const
+    {
+        double epoch = 2433283.0 - 0.5;
+        double a     = 23460.0;
+        double e     = 0.0002;
+        double w0    = 290.496;
+        double M0    = 296.230;
+        double i     = 1.793;
+        double node0 = 339.600;
+        double n     = 285.1618919;
+        double Pw    = 26.892;
+        double Pnode = 54.536;
+
+        double refplane_RA  = 316.700;
+        double refplane_Dec =  53.564;
+        double marspole_RA  = 317.681;
+        double marspole_Dec =  52.886;
+
+        double t = jd - epoch;
+        t += 10.5 / 1440.0;     // light time correction?
+        double T = t / 365.25;
+
+        double dnode = 360.0 / Pnode;
+        double dw    = 360.0 / Pw;
+        double node = degToRad(node0 + T * dnode);
+        double w    = degToRad(w0 + T * dw - T * dnode);
+        double M    = degToRad(M0 + t * n  - T * dw);
+
+        Point3d p = ellipsePosition(a, e, M);
+
+        // Orientation of the orbital plane with respect to the Laplacian plane
+        Mat3d Rorbit     = (Mat3d::yrotation(node) *
+                            Mat3d::xrotation(degToRad(i)) *
+                            Mat3d::yrotation(w));
+
+        // Rotate to the Earth's equatorial plane
+        double N = degToRad(refplane_RA);
+        double J = degToRad(90 - refplane_Dec);
+        Mat3d RLaplacian = (Mat3d::yrotation( N) *
+                            Mat3d::xrotation( J) *
+                            Mat3d::yrotation(-N));
+
+        // Rotate to the Martian equatorial plane
+        N = degToRad(marspole_RA);
+        J = degToRad(90 - marspole_Dec);
+        Mat3d RMars_eq   = (Mat3d::yrotation( N) *
+                            Mat3d::xrotation(-J) *
+                            Mat3d::yrotation(-N));
+
+        return RMars_eq * (RLaplacian * (Rorbit * p));
+    }
+
+#if 0
+    // More accurate orbit calculation for Mars from _Explanatory
+    // Supplement to the Astronomical Almanac_  There's still a bug in
+    // this routine however.
+    Point3d computePosition(double jd) const
+    {
+        double d = jd - 2441266.5;  // days since 11 Nov 1971
+        double D = d / 365.25;      // years
+
+        double a = 1.56828e-4 * KM_PER_AU;
+        double n = 285.161888;
+        double e = 0.0004;
+        double gamma = degToRad(1.79);
+        double theta = degToRad(240.38 - 0.01801 * d);
+
+        double h = degToRad(196.55 - 0.01801 * d);
+        double L = degToRad(28.96 + n * d - 0.27 * sin(h));
+        double P = degToRad(111.7 + 0.01798 * d);
+
+        // N and J give the orientation of the Laplacian plane with respect
+        // to the Earth's equator and equinox.
+        //    N - longitude of ascending node
+        //    J - inclination
+        double N = degToRad(46.37 - 0.0014 * D);
+        double J = degToRad(36.62 + 0.0008 * D);
+
+        // Compute the mean anomaly
+        double M = L - P;
+        
+        // Solve Kepler's equation--for a low eccentricity orbit, just a few
+        // iterations is enough.
+        double E = M;
+        for (int i = 0; i < 3; i++)
+            E = M + e * sin(E);
+
+        // Compute the position in the orbital plane (y = 0)
+        double x = a * (cos(E) - e);
+        double z = a * sqrt(1 - square(e)) * -sin(E);
+
+        // Orientation of the orbital plane with respect to the Laplacian plane
+        Mat3d Rorbit     = (Mat3d::yrotation(theta) *
+                            Mat3d::xrotation(gamma) *
+                            Mat3d::yrotation(P - theta));
+
+        Mat3d RLaplacian = (Mat3d::yrotation( N) *
+                            Mat3d::xrotation(-J) *
+                            Mat3d::yrotation(-N));
+
+        double marspole_RA  = 317.681;
+        double marspole_Dec =  52.886;
+        double Nm = degToRad(marspole_RA + 90);
+        double Jm = degToRad(90 - marspole_Dec);
+        Mat3d RMars_eq   = (Mat3d::yrotation( Nm) *
+                            Mat3d::xrotation( Jm) *
+                            Mat3d::yrotation(-Nm));
+
+        // Celestia wants the position of a satellite with respect to the
+        // equatorial plane of the planet it orbits.
+
+        // to Laplacian...
+        Point3d p = Rorbit * Point3d(x, 0.0, z);
+
+        // to Earth equatorial...
+        p = RLaplacian * p;
+
+        // to Mars equatorial...
+        return RMars_eq * p;
+    }
+#endif
+
+    double getPeriod() const
+    {
+        return 1.262441;
+    }
+
+    double getBoundingRadius() const
+    {
+        return 23462 * BoundingRadiusSlack;
+    }
+};
+
+
 // static const double JupAscendingNode = degToRad(20.453422);
 static const double JupAscendingNode = degToRad(22.203);
 
@@ -1533,49 +1775,92 @@ class CallistoOrbit : public CachingOrbit
                             gamma, phi, psi, G, Gp);
 
     //Calculate periodic terms for longitude
-    sigma = 0.84287*sin(l4 - p4) + 0.03431*sin(p4 - p3)
-          - 0.03305*sin(2*(psi - LPEJ)) - 0.03211*sin(G)
-          - 0.01862*sin(l4 - p3) + 0.01186*sin(psi - w4)
-          + 6.23e-3*sin(l4 + p4 - 2*G - 2*LPEJ) + 3.87e-3*sin(2*(l4 - p4))
-          - 2.84e-3*sin(5*Gp - 2*G + 0.9115) - 2.34e-3*sin(2*(psi - p4))
-          - 2.23e-3*sin(l3 - l4) - 2.08e-3*sin(l4 - LPEJ)
-          + 1.78e-3*sin(psi + w4 - 2*p4) + 1.34e-3*sin(p4 - LPEJ)
-          + 1.25e-3*sin(2*(l4 - G - LPEJ)) - 1.17e-3*sin(2*G)
-          - 1.12e-3*sin(2*(l3 - l4)) + 1.07e-3*sin(3*l3 - 7*l4 + 4*p4)
-          + 1.02e-3*sin(l4 - G - LPEJ) + 9.6e-4*sin(2*l4 - psi - w4)
-          + 8.7e-4*sin(2*(psi - w4)) - 8.5e-4*sin(3*l3 - 7*l4 + p3 + 3*p4)
-          + 8.5e-4*sin(l3 - 2*l4 + p4) - 8.1e-4*sin(2*(l4 - psi))
-          + 7.1e-4*sin(l4 + p4 - 2*LPEJ - 3*G) + 6.1e-4*sin(l1 - l4)
-          - 5.6e-4*sin(psi - w3) - 5.4e-4*sin(l3 - 2*l4 + p3)
-          + 5.1e-4*sin(l2 - l4) + 4.2e-4*sin(2*(psi - G - LPEJ))
-          + 3.9e-4*sin(2*(p4 - w4)) + 3.6e-4*sin(psi + LPEJ - p4 - w4)
-          + 3.5e-4*sin(2*Gp - G + 3.2877) - 3.5e-4*sin(l4 - p4 + 2*LPEJ - 2*psi)
-          - 3.2e-4*sin(l4 + p4 - 2*LPEJ - G) + 3.0e-4*sin(2*Gp - 2*G + 2.6032)
-          + 2.9e-4*sin(3*l3 - 7*l4 + 2*p3 + 2*p4) + 2.8e-4*sin(l4 - p4 + 2*psi - 2*LPEJ)
-          - 2.8e-4*sin(2*(l4 - w4)) - 2.7e-4*sin(p3 - p4 + w3 - w4)
-          - 2.6e-4*sin(5*Gp - 3*G + 3.2877) + 2.5e-4*sin(w4 - w3)
-          - 2.5e-4*sin(l2 - 3*l3 + 2*l4) - 2.3e-4*sin(3*(l3 - l4))
-          + 2.1e-4*sin(2*l4 - 2*LPEJ - 3*G) - 2.1e-4*sin(2*l3 - 3*l4 + p4)
-          + 1.9e-4*sin(l4 - p4 - G) - 1.9e-4*sin(2*l4 - p3 - p4)
-          - 1.8e-4*sin(l4 - p4 + G) - 1.6e-4*sin(l4 + p3 - 2*LPEJ - 2*G);
+    sigma =
+        0.84287*sin(l4 - p4)
+        + 0.03431*sin(p4 - p3)
+        - 0.03305*sin(2*(psi - LPEJ))
+        - 0.03211*sin(G)
+        - 0.01862*sin(l4 - p3)
+        + 0.01186*sin(psi - w4)
+        + 6.23e-3*sin(l4 + p4 - 2*G - 2*LPEJ)
+        + 3.87e-3*sin(2*(l4 - p4))
+        - 2.84e-3*sin(5*Gp - 2*G + 0.9115)
+        - 2.34e-3*sin(2*(psi - p4))
+        - 2.23e-3*sin(l3 - l4)
+        - 2.08e-3*sin(l4 - LPEJ)
+        + 1.78e-3*sin(psi + w4 - 2*p4)
+        + 1.34e-3*sin(p4 - LPEJ)
+        + 1.25e-3*sin(2*(l4 - G - LPEJ))
+        - 1.17e-3*sin(2*G)
+        - 1.12e-3*sin(2*(l3 - l4))
+        + 1.07e-3*sin(3*l3 - 7*l4 + 4*p4)
+        + 1.02e-3*sin(l4 - G - LPEJ)
+        + 9.6e-4*sin(2*l4 - psi - w4)
+        + 8.7e-4*sin(2*(psi - w4))
+        - 8.5e-4*sin(3*l3 - 7*l4 + p3 + 3*p4)
+        + 8.5e-4*sin(l3 - 2*l4 + p4)
+        - 8.1e-4*sin(2*(l4 - psi))
+        + 7.1e-4*sin(l4 + p4 - 2*LPEJ - 3*G)
+        + 6.1e-4*sin(l1 - l4)
+        - 5.6e-4*sin(psi - w3)
+        - 5.4e-4*sin(l3 - 2*l4 + p3)
+        + 5.1e-4*sin(l2 - l4)
+        + 4.2e-4*sin(2*(psi - G - LPEJ))
+        + 3.9e-4*sin(2*(p4 - w4))
+        + 3.6e-4*sin(psi + LPEJ - p4 - w4)
+        + 3.5e-4*sin(2*Gp - G + 3.2877)
+        - 3.5e-4*sin(l4 - p4 + 2*LPEJ - 2*psi)
+        - 3.2e-4*sin(l4 + p4 - 2*LPEJ - G)
+        + 3.0e-4*sin(2*Gp - 2*G + 2.6032)
+        + 2.9e-4*sin(3*l3 - 7*l4 + 2*p3 + 2*p4)
+        + 2.8e-4*sin(l4 - p4 + 2*psi - 2*LPEJ)
+        - 2.8e-4*sin(2*(l4 - w4))
+        - 2.7e-4*sin(p3 - p4 + w3 - w4)
+        - 2.6e-4*sin(5*Gp - 3*G + 3.2877)
+        + 2.5e-4*sin(w4 - w3)
+        - 2.5e-4*sin(l2 - 3*l3 + 2*l4)
+        - 2.3e-4*sin(3*(l3 - l4))
+        + 2.1e-4*sin(2*l4 - 2*LPEJ - 3*G)
+        - 2.1e-4*sin(2*l3 - 3*l4 + p4)
+        + 1.9e-4*sin(l4 - p4 - G)
+        - 1.9e-4*sin(2*l4 - p3 - p4)
+        - 1.8e-4*sin(l4 - p4 + G)
+        - 1.6e-4*sin(l4 + p3 - 2*LPEJ - 2*G);
     sigma = pfmod(sigma, 360.0);
     sigma = degToRad(sigma);
     L = l4 + sigma;
 
     //Calculate periodic terms for the tangent of the latitude
-    B = -7.6579e-3*sin(L - psi) + 4.4134e-3*sin(L - w4)
-      - 5.112e-4*sin(L - w3) + 7.73e-5*sin(L + psi - 2*LPEJ - 2*G)
-      + 1.04e-5*sin(L - psi + G) - 1.02e-5*sin(L - psi - G)
-      + 8.8e-6*sin(L + psi - 2*LPEJ - 3*G) - 3.8e-6*sin(L + psi - 2*LPEJ - G);
+    B =
+        - 7.6579e-3 * sin(L - psi)
+        + 4.4134e-3 * sin(L - w4)
+        - 5.112e-4  * sin(L - w3)
+        + 7.73e-5   * sin(L + psi - 2*LPEJ - 2*G)
+        + 1.04e-5   * sin(L - psi + G)
+        - 1.02e-5   * sin(L - psi - G)
+        + 8.8e-6    * sin(L + psi - 2*LPEJ - 3*G)
+        - 3.8e-6    * sin(L + psi - 2*LPEJ - G);
     B = atan(B);
 
     //Calculate the periodic terms for distance
-    R = -7.3546e-3*cos(l4 - p4) + 1.621e-4*cos(l4 - p3)
-      + 9.74e-3*cos(l3 - l4) - 5.43e-3*cos(l4 + p4 - 2*LPEJ - 2*G)
-      - 2.71e-3*cos(2*(l4 - p4)) + 1.82e-3*cos(l4 - LPEJ)
-      + 1.77e-3*cos(2*(l3 - l4)) - 1.67e-3*cos(2*l4 - psi - w4)
-      + 1.67e-3*cos(psi - w4) - 1.55e-3*cos(2*(l4 - LPEJ - G))
-      + 1.42e-3*cos(2*(l4 - psi));
+    R =
+        - 7.3546e-3 * cos(l4 - p4)
+        + 1.621e-4  * cos(l4 - p3)
+        + 9.74e-5   * cos(l3 - l4)
+        - 5.43e-5   * cos(l4 + p4 - 2*LPEJ - 2*G)
+        - 2.71e-5   * cos(2*(l4 - p4))
+        + 1.82e-5   * cos(l4 - LPEJ)
+        + 1.77e-5   * cos(2*(l3 - l4))
+        - 1.67e-5   * cos(2*l4 - psi - w4)
+        + 1.67e-5   * cos(psi - w4)
+        - 1.55e-5   * cos(2*(l4 - LPEJ - G))
+        + 1.42e-5   * cos(2*(l4 - psi))
+        + 1.05e-5   * cos(l1 - l4)
+        + 9.2e-6    * cos(l2 - l4)
+        - 8.9e-6    * cos(l4 - LPEJ -G)
+        - 6.2e-6    * cos(l4 + p4 - 2*LPEJ - 3*G)
+        + 4.8e-6    * cos(2*(l4 - w4));
+    
     R = 26.36273 * JupiterRadius * (1 + R);
 
     T = (jd - 2433282.423) / 36525.0;
@@ -1605,17 +1890,6 @@ class CallistoOrbit : public CachingOrbit
 };
 
 
-static double sinD(double theta)
-{
-    return sin(degToRad(theta));
-}
-
-static double cosD(double theta)
-{
-    return cos(degToRad(theta));
-}
-
-
 static const double SatAscendingNode = 168.8112;
 static const double SatTilt = 28.0817;
 // static const double SatAscendingNode = 169.530;
@@ -2359,6 +2633,128 @@ static UranianSatelliteOrbit* CreateUranianSatelliteOrbit(int n)
 };
 
 
+class TritonOrbit : public CachingOrbit
+{
+    Point3d computePosition(double jd) const
+    {
+        // Pole of the fixed reference plane
+        //double t = jd - 2433282.5;
+        double epoch = 2433282.5;
+        double t = jd - epoch;
+        double T = (jd - 2451545.0) / 36525;
+        double N = degToRad(359.28 + 54.308 * T);
+        double refplane_RA  = degToRad(298.72 +
+                                       2.58 * sin(N) - 0.04 * sin(2 * N));
+        double refplane_Dec = degToRad(42.63 -
+                                       1.90 * cos(N) - 0.01 * cos(2 * N));
+        double nepPole_RA = degToRad(299.36);
+        double nepPole_Dec = degToRad(43.46);
+
+        double a = 354800;
+        double e = 0.0000;
+        double n = degToRad(61.2588532);
+        double L0 = degToRad(200.913);
+        double L  = L0 + n * t;
+        double gamma = degToRad(158.996);
+        double theta = degToRad(151.401 + 0.57806 * t / 365.25);
+
+        double E = L - theta;
+        Point3d p(a * cos(E), 0.0, a * -sin(E));
+
+        // Orientation of the orbital plane with respect to the Laplacian plane
+        Mat3d Rorbit     = (Mat3d::yrotation(theta) *
+                            Mat3d::xrotation(gamma));
+
+        // Rotate to the Earth's equatorial plane
+        double Nr = degToRad(refplane_RA);
+        double Jr = degToRad(90 - refplane_Dec);
+        Mat3d Rrefplane = (Mat3d::yrotation( Nr) *
+                           Mat3d::xrotation( Jr) *
+                           Mat3d::yrotation(-Nr));
+
+        // Rotate to the Neptunian equatorial plane
+        double Nn = degToRad(nepPole_RA);
+        double Jn = degToRad(90 - nepPole_Dec);
+        Mat3d RNept_eq   = (Mat3d::yrotation( Nn) *
+                            Mat3d::xrotation(-Jn) *
+                            Mat3d::yrotation(-Nn));
+
+        return RNept_eq * (Rrefplane * (Rorbit * p));
+    }
+    
+    double getPeriod() const
+    {
+        return 5.877;
+    }
+
+    double getBoundingRadius() const
+    {
+        return 354800 * BoundingRadiusSlack;
+    }
+};
+
+
+class JPLEphOrbit : public CachingOrbit
+{
+public:
+    JPLEphOrbit(const JPLEphemeris& e, JPLEphemItem item,
+                double _period, double _boundingRadius) :
+        ephem(e),
+        body(item),
+        period(_period),
+        boundingRadius(_boundingRadius)
+    {
+    };
+
+    double getPeriod() const
+    {
+        return period;
+    }
+
+    double getBoundingRadius() const
+    {
+        return boundingRadius;
+    }
+
+    Point3d computePosition(double jd) const
+    {
+        Point3d baryPos = ephem.getPlanetPosition(body, jd);
+
+        // Convert from solar system barycentric position to heliocentric
+        // position.
+        Point3d sunPos = ephem.getPlanetPosition(JPLEph_Sun, jd);
+        Point3d heliocentricPos = Point3d(0.0, 0.0, 0.0) + (baryPos - sunPos);
+
+        // Rotate from the J2000 mean equator to the ecliptic
+        // 23 deg 26' 21".448
+        double J2000_equator = degToRad(23.4392911);
+        heliocentricPos = heliocentricPos * Mat3d::xrotation(J2000_equator);
+
+        return Point3d(heliocentricPos.x,
+                       heliocentricPos.z,
+                       -heliocentricPos.y);
+    }
+
+private:
+    const JPLEphemeris& ephem;
+    JPLEphemItem body;
+    double period;
+    double boundingRadius;
+};
+
+
+static Orbit* CreateJPLEphOrbit(JPLEphemItem body,
+                                double period,
+                                double boundingRadius)
+{
+    Orbit* o = new JPLEphOrbit(*jpleph, body, period, boundingRadius);
+    return new MixedOrbit(o,
+                          jpleph->getStartDate(),
+                          jpleph->getEndDate(),
+                          astro::SolarMass);
+}
+
+
 static double yearToJD(int year)
 {
     return (double) astro::Date(year, 1, 1);
@@ -2367,6 +2763,23 @@ static double yearToJD(int year)
 
 Orbit* GetCustomOrbit(const string& name)
 {
+    // Attempt to load JPL ephemeris data if we haven't tried already
+    if (!jplephInitialized)
+    {
+        jplephInitialized = true;
+        ifstream in("data/jpleph.dat", ios::in | ios::binary);
+        if (in.good())
+            jpleph = JPLEphemeris::load(in);
+        if (jpleph != NULL)
+        {
+            clog << "Loaded DE" << jpleph->getDENumber() <<
+                " ephemeris. Valid from JD" <<
+                setprecision(8) <<
+                jpleph->getStartDate() << " to JD" <<
+                jpleph->getEndDate() << '\n';
+        }
+    }
+
     if (name == "mercury")
         return new MixedOrbit(new MercuryOrbit(), yearToJD(-4000), yearToJD(4000), astro::SolarMass);
     if (name == "venus")
@@ -2387,6 +2800,30 @@ Orbit* GetCustomOrbit(const string& name)
         return new MixedOrbit(new NeptuneOrbit(), yearToJD(-4000), yearToJD(4000), astro::SolarMass);
     if (name == "pluto")
         return new MixedOrbit(new PlutoOrbit(), yearToJD(-4000), yearToJD(4000), astro::SolarMass);
+    if (name == "mercury-jpl")
+        return CreateJPLEphOrbit(JPLEph_Mercury,  0.2408  * 365.25, 6.0e7);
+    if (name == "venus-jpl")
+        return CreateJPLEphOrbit(JPLEph_Venus,    0.6152  * 365.25, 1.0e8);
+    if (name == "earth-jpl")
+        return CreateJPLEphOrbit(JPLEph_EarthMoonBary,      365.25, 1.6e8);
+    if (name == "mars-jpl")
+        return CreateJPLEphOrbit(JPLEph_Mars,      1.8809 * 365.25, 2.4e8);
+    if (name == "jupiter-jpl")
+        return CreateJPLEphOrbit(JPLEph_Jupiter,  11.86   * 365.25, 8.0e8);
+    if (name == "saturn-jpl")
+        return CreateJPLEphOrbit(JPLEph_Saturn,   29.4577 * 365.25, 1.5e9);
+    if (name == "uranus-jpl")
+        return CreateJPLEphOrbit(JPLEph_Uranus,   84.0139 * 365.25, 3.0e9);
+    if (name == "neptune-jpl")
+        return CreateJPLEphOrbit(JPLEph_Neptune, 164.793  * 365.25, 4.7e9);
+    if (name == "pluto-jpl")
+        return CreateJPLEphOrbit(JPLEph_Pluto,   248.54   * 365.25, 6.0e9);
+    if (name == "moon-jpl")
+        return CreateJPLEphOrbit(JPLEph_Moon,     27.321661,        5.0e5);
+    if (name == "phobos")
+        return new PhobosOrbit();
+    if (name == "deimos")
+        return new DeimosOrbit();
     if (name == "io")
         return new IoOrbit();
     if (name == "europa")
@@ -2423,6 +2860,8 @@ Orbit* GetCustomOrbit(const string& name)
         return CreateUranianSatelliteOrbit(4);
     if (name == "oberon")
         return CreateUranianSatelliteOrbit(5);
+    if (name == "triton")
+        return new TritonOrbit();
     else
         return CreateVSOP87Orbit(name);
 }
