@@ -174,10 +174,10 @@ void Galaxy::render(const Vec3f& offset,
     {
         galaxyTex = CreateProceduralTexture(128, 128, GL_RGBA,
                                             GalaxyTextureEval);
-        galaxyTex->bindName();
     }
     assert(galaxyTex != NULL);
 
+    glEnable(GL_TEXTURE_2D);
     galaxyTex->bind();
 
     Mat3f viewMat = viewerOrientation.toMatrix3();
