@@ -19,6 +19,23 @@ using namespace std;
 #define VELOCITY_CHANGE_TIME      0.25f
 
 
+static Vec3d slerp(double t, const Vec3d& v0, const Vec3d& v1)
+{
+    double r0 = v0.length();
+    double r1 = v1.length();
+    Vec3d u = v0 / r0;
+    Vec3d n = u ^ (v1 / r1);
+    n.normalize();
+    Vec3d v = n ^ u;
+    if (v * v1 < 0.0)
+        v = -v;
+
+    double cosTheta = u * (v1 / r1);
+    double theta = acos(cosTheta);
+
+    return (cos(theta * t) * u + sin(theta * t) * v) * Mathd::lerp(t, r0, r1);
+}
+
 
 Observer::Observer() :
     simTime(0.0),
@@ -31,7 +48,9 @@ Observer::Observer() :
     beginAccelTime(0.0),
     observerMode(Free),
     trackingOrientation(1.0f, 0.0f, 0.0f, 0.0f),
-    fov((float) (PI / 4.0))
+    fov((float) (PI / 4.0)),
+    reverseFlag(false),
+    locationFilter(~0)
 {
 }
 
@@ -133,6 +152,7 @@ RigidTransform Observer::getSituation() const
 {
     return frame.toUniversal(situation, getTime());
 }
+                        
 
 
 void Observer::setSituation(const RigidTransform& xform)
@@ -154,19 +174,20 @@ Vec3d toUniversal(const Vec3d& v,
             Quatf q = observer.getOrientation();
             Quatd qd(q.w, q.x, q.y, q.z);
             return v * qd.toMatrix3();
-        }
+        }                        
+
         
     case astro::Geographic:
-        if (sel.body == NULL)
+        if (sel.getType() != Selection::Type_Body)
             return v;
         else
-            return v * sel.body->getGeographicToHeliocentric(t);
+            return v * sel.body()->getGeographicToHeliocentric(t);
         
     case astro::Equatorial:
-        if (sel.body == NULL)
+        if (sel.getType() != Selection::Type_Body)
             return v;
         else
-            return v * sel.body->getLocalToHeliocentric(t);
+            return v * sel.body()->getLocalToHeliocentric(t);
 
     case astro::Ecliptical:
         // TODO: Multiply this by the planetary system's ecliptic orientation,
@@ -235,29 +256,108 @@ void Observer::update(double dt, double timeScale)
                     (journey.expFactor * (u - journey.accelTime) + 1) - 1;
             }
 
-            Vec3d v = jv;
-            if (v.length() == 0.0)
+            if (journey.traj == Linear)
             {
-                p = journey.from;
-            }
-            else
-            {
-                v.normalize();
-                if (t < 0.5)
-                    p = journey.from + v * astro::kilometersToMicroLightYears(x);
+                Vec3d v = jv;
+                if (v.length() == 0.0)
+                {
+                    p = journey.from;
+                }
                 else
-                    p = journey.to - v * astro::kilometersToMicroLightYears(x);
+                {
+                    v.normalize();
+                    if (t < 0.5)
+                        p = journey.from + v * astro::kilometersToMicroLightYears(x);
+                    else
+                        p = journey.to - v * astro::kilometersToMicroLightYears(x);
+                }
+            }
+            else if (journey.traj == GreatCircle)
+            {
+                Selection centerObj = frame.refObject;
+                if (centerObj.body() != NULL)
+                {
+                    Body* body = centerObj.body();
+                    if (body->getSystem())
+                    {
+                        if (body->getSystem()->getPrimaryBody() != NULL)
+                            centerObj = Selection(body->getSystem()->getPrimaryBody());
+                        else
+                            centerObj = Selection(body->getSystem()->getStar());
+                    }
+                }
+
+                UniversalCoord ufrom  = frame.toUniversal(RigidTransform(journey.from, Quatf(1.0f)), simTime).translation;
+                UniversalCoord uto    = frame.toUniversal(RigidTransform(journey.to, Quatf(1.0f)), simTime).translation;
+                UniversalCoord origin = centerObj.getPosition(simTime);
+                Vec3d v0 = ufrom - origin;
+                Vec3d v1 = uto - origin;
+
+                if (jv.length() == 0.0)
+                {
+                    p = journey.from;
+                }
+                else
+                {
+                    x = astro::kilometersToMicroLightYears(x / jv.length());
+                    Vec3d v;
+
+                    if (t < 0.5)
+                        v = slerp(x, v0, v1);
+                    else
+                        v = slerp(x, v1, v0);
+
+                    p = origin + v;
+                    p = frame.fromUniversal(RigidTransform(p, Quatf(1.0f)), simTime).translation;
+                }
+            }
+            else if (journey.traj == CircularOrbit)
+            {
+                Selection centerObj = frame.refObject;
+
+                UniversalCoord ufrom  = frame.toUniversal(RigidTransform(journey.from, Quatf(1.0f)), simTime).translation;
+                UniversalCoord uto    = frame.toUniversal(RigidTransform(journey.to, Quatf(1.0f)), simTime).translation;
+                UniversalCoord origin = centerObj.getPosition(simTime);
+                Vec3d v0 = ufrom - origin;
+                Vec3d v1 = uto - origin;
+
+                if (jv.length() == 0.0)
+                {
+                    p = journey.from;
+                }
+                else
+                {
+                    //x = astro::kilometersToMicroLightYears(x / jv.length());
+                    Quatd q0(1.0);
+                    Quatd q1(journey.rotation1.w, journey.rotation1.x,
+                             journey.rotation1.y, journey.rotation1.z);
+                    p = origin + v0 * Quatd::slerp(q0, q1, t).toMatrix3();
+                                              
+                    p = frame.fromUniversal(RigidTransform(p, Quatf(1.0f)),
+                                            simTime).translation;
+                }
             }
         }
 
         // Spherically interpolate the orientation over the first half
         // of the journey.
         Quatf orientation;
-        if (t < 0.5f)
+        if (t >= journey.startInterpolation && t < journey.endInterpolation )
         {
             // Smooth out the interpolation to avoid jarring changes in
             // orientation
-            double v = sin(t * PI);
+            double v;
+            if (journey.traj == CircularOrbit)
+            {
+                // In circular orbit mode, interpolation of orientation must
+                // match the interpolation of position.
+                v = t;
+            }
+            else
+            {
+                v = pow(sin((t - journey.startInterpolation) /
+                            (journey.endInterpolation - journey.startInterpolation) * PI / 2), 2);
+            }
 
             // Be careful to choose the shortest path when interpolating
             if (norm(journey.initialOrientation - journey.finalOrientation) <
@@ -272,7 +372,11 @@ void Observer::update(double dt, double timeScale)
                                            -journey.finalOrientation, v);
             }
         }
-        else
+        else if (t < journey.startInterpolation)
+        {
+            orientation = journey.initialOrientation;
+        }
+        else // t >= endInterpolation
         {
             orientation = journey.finalOrientation;
         }
@@ -282,7 +386,8 @@ void Observer::update(double dt, double timeScale)
         // If the journey's complete, reset to manual control
         if (t == 1.0f)
         {
-            situation = RigidTransform(journey.to, journey.finalOrientation);
+            if (journey.traj != CircularOrbit)
+                situation = RigidTransform(journey.to, journey.finalOrientation);
             observerMode = Free;
             setVelocity(Vec3d(0, 0, 0));
 //            targetVelocity = Vec3d(0, 0, 0);
@@ -330,11 +435,36 @@ void Observer::setTrackedObject(const Selection& sel)
 }
 
 
+const string& Observer::getDisplayedSurface() const
+{
+    return displayedSurface;
+}
+
+
+void Observer::setDisplayedSurface(const string& surf)
+{
+    displayedSurface = surf;
+}
+
+
+uint32 Observer::getLocationFilter() const
+{
+    return locationFilter;
+}
+
+
+void Observer::setLocationFilter(uint32 _locationFilter)
+{
+    locationFilter = _locationFilter;
+}
+
+
 void Observer::reverseOrientation()
 {
     Quatf q = getOrientation();
     q.yrotate((float) PI);
     setOrientation(q);
+    reverseFlag = !reverseFlag;
 }
 
 
@@ -356,23 +486,33 @@ struct TravelExpFunc : public unary_function<double, double>
 void Observer::computeGotoParameters(const Selection& destination,
                                      JourneyParams& jparams,
                                      double gotoTime,
+                                     double startInter,
+                                     double endInter,
                                      Vec3d offset,
                                      astro::CoordinateSystem offsetFrame,
                                      Vec3f up,
                                      astro::CoordinateSystem upFrame)
 {
+    if (frame.coordSys == astro::PhaseLock)
+    {
+        setFrame(FrameOfReference(astro::Ecliptical, destination));
+    }
+    else
+    {
+        setFrame(FrameOfReference(frame.coordSys, destination));
+    }
+
     UniversalCoord targetPosition = destination.getPosition(getTime());
     Vec3d v = targetPosition - getPosition();
     v.normalize();
 
+    jparams.traj = Linear;
     jparams.duration = gotoTime;
     jparams.startTime = realTime;
 
     // Right where we are now . . .
     jparams.from = getPosition();
 
-    // The destination position lies along the line between the current
-    // position and the star
     offset = toUniversal(offset, *this, destination, getTime(), offsetFrame);
     jparams.to = targetPosition + offset;
 
@@ -384,6 +524,8 @@ void Observer::computeGotoParameters(const Selection& destination,
     Vec3d vn = targetPosition - jparams.to;
     Point3f focus((float) vn.x, (float) vn.y, (float) vn.z);
     jparams.finalOrientation = lookAt(Point3f(0, 0, 0), focus, upf);
+    jparams.startInterpolation = min(startInter, endInter);
+    jparams.endInterpolation   = max(startInter, endInter);
 
     jparams.accelTime = 0.5;
     double distance = astro::microLightYearsToKilometers(jparams.from.distanceTo(jparams.to)) / 2.0;
@@ -392,8 +534,71 @@ void Observer::computeGotoParameters(const Selection& destination,
                                                1e-10);
     jparams.expFactor = sol.first;
 
+    // Convert to frame coordinates
+    RigidTransform from(jparams.from, jparams.initialOrientation);
+    from = frame.fromUniversal(from, getTime());
+    jparams.from = from.translation;
+    jparams.initialOrientation= Quatf((float) from.rotation.w,
+                                      (float) from.rotation.x,
+                                      (float) from.rotation.y,
+                                      (float) from.rotation.z);
+    RigidTransform to(jparams.to, jparams.finalOrientation);
+    to = frame.fromUniversal(to, getTime());
+    jparams.to = to.translation;
+    jparams.finalOrientation= Quatf((float) to.rotation.w,
+                                    (float) to.rotation.x,
+                                    (float) to.rotation.y,
+                                    (float) to.rotation.z);
+}
+
+
+void Observer::computeGotoParametersGC(const Selection& destination,
+                                       JourneyParams& jparams,
+                                       double gotoTime,
+                                       double startInter,
+                                       double endInter,
+                                       Vec3d offset,
+                                       astro::CoordinateSystem offsetFrame,
+                                       Vec3f up,
+                                       astro::CoordinateSystem upFrame,
+                                       const Selection& centerObj)
+{
     setFrame(FrameOfReference(frame.coordSys, destination));
 
+    UniversalCoord targetPosition = destination.getPosition(getTime());
+    Vec3d v = targetPosition - getPosition();
+    v.normalize();
+
+    jparams.traj = GreatCircle;
+    jparams.duration = gotoTime;
+    jparams.startTime = realTime;
+
+    jparams.centerObject = centerObj;
+
+    // Right where we are now . . .
+    jparams.from = getPosition();
+
+    offset = toUniversal(offset, *this, destination, getTime(), offsetFrame);
+    jparams.to = targetPosition + offset;
+
+    Vec3d upd(up.x, up.y, up.z);
+    upd = toUniversal(upd, *this, destination, getTime(), upFrame);
+    Vec3f upf = Vec3f((float) upd.x, (float) upd.y, (float) upd.z);
+
+    jparams.initialOrientation = getOrientation();
+    Vec3d vn = targetPosition - jparams.to;
+    Point3f focus((float) vn.x, (float) vn.y, (float) vn.z);
+    jparams.finalOrientation = lookAt(Point3f(0, 0, 0), focus, upf);
+    jparams.startInterpolation = min(startInter, endInter);
+    jparams.endInterpolation   = max(startInter, endInter);
+
+    jparams.accelTime = 0.5;
+    double distance = astro::microLightYearsToKilometers(jparams.from.distanceTo(jparams.to)) / 2.0;
+    pair<double, double> sol = solve_bisection(TravelExpFunc(distance, jparams.accelTime),
+                                               0.0001, 100.0,
+                                               1e-10);
+    jparams.expFactor = sol.first;
+
     // Convert to frame coordinates
     RigidTransform from(jparams.from, jparams.initialOrientation);
     from = frame.fromUniversal(from, getTime());
@@ -420,6 +625,7 @@ void Observer::computeCenterParameters(const Selection& destination,
 
     jparams.duration = centerTime;
     jparams.startTime = realTime;
+    jparams.traj = Linear;
 
     // Don't move through space, just rotate the camera
     jparams.from = getPosition();
@@ -431,6 +637,9 @@ void Observer::computeCenterParameters(const Selection& destination,
     Vec3d vn = targetPosition - jparams.to;
     Point3f focus((float) vn.x, (float) vn.y, (float) vn.z);
     jparams.finalOrientation = lookAt(Point3f(0, 0, 0), focus, up);
+    jparams.startInterpolation = 0;
+    jparams.endInterpolation   = 1;
+
 
     jparams.accelTime = 0.5;
     jparams.expFactor = 0;
@@ -454,11 +663,102 @@ void Observer::computeCenterParameters(const Selection& destination,
 }
 
 
+void Observer::computeCenterCOParameters(const Selection& destination,
+                                         JourneyParams& jparams,
+                                         double centerTime)
+{
+    jparams.duration = centerTime;
+    jparams.startTime = realTime;
+    jparams.traj = CircularOrbit;
+
+    jparams.centerObject = frame.refObject;
+    jparams.expFactor = 0.5;
+
+    Vec3d v = destination.getPosition(getTime()) - getPosition();
+    Vec3f wf = Vec3f(0.0, 0.0, -1.0) * getOrientation().toMatrix3();
+    Vec3d w(wf.x, wf.y, wf.z);
+    v.normalize();
+
+    Vec3d n = w ^ v;
+    double nl = n.length();
+    double angle = 0.0;
+    if (nl > 0.0)
+    {
+        double cosAngle = w * v;
+        if (cosAngle < 1.0 - 1e-8)
+        {
+            if (cosAngle > 1e-8 - 1.0)
+                angle = acos(cosAngle);
+            else
+                angle = PI;
+        }
+        else
+        {
+            angle = 0.0;
+        }
+
+        n = n / nl;
+    }
+    else
+    {
+        n = Vec3d(1.0, 0.0, 0.0);
+    }
+
+    Selection centerObj = frame.refObject;
+    UniversalCoord centerPos = centerObj.getPosition(getTime());
+    UniversalCoord targetPosition = destination.getPosition(getTime());
+
+    Quatd qd;
+    qd.setAxisAngle(n, -angle);
+    Quatf q((float) qd.w, (float) qd.x, (float) qd.y, (float) qd.z);
+
+    jparams.from = getPosition();
+    jparams.to = centerPos + ((getPosition() - centerPos) * qd.toMatrix3());
+    jparams.initialOrientation = getOrientation();
+    jparams.finalOrientation = getOrientation() * q;
+    
+    jparams.startInterpolation = 0.0;
+    jparams.endInterpolation = 1.0;
+
+    jparams.rotation1 = q;
+
+    // Convert to frame coordinates
+    RigidTransform from(jparams.from, jparams.initialOrientation);
+    from = frame.fromUniversal(from, getTime());
+    jparams.from = from.translation;
+    jparams.initialOrientation= Quatf((float) from.rotation.w,
+                                      (float) from.rotation.x,
+                                      (float) from.rotation.y,
+                                      (float) from.rotation.z);
+
+    RigidTransform to(jparams.to, jparams.finalOrientation);
+    to = frame.fromUniversal(to, getTime());
+    jparams.to = to.translation;
+    jparams.finalOrientation= Quatf((float) to.rotation.w,
+                                    (float) to.rotation.x,
+                                    (float) to.rotation.y,
+                                    (float) to.rotation.z);
+}
+
+
 Observer::ObserverMode Observer::getMode() const
 {
     return observerMode;
 }
 
+
+// Center the selection by moving on a circular orbit arround
+// the primary body (refObject).
+void Observer::centerSelectionCO(const Selection& selection, double centerTime)
+{
+    if (!selection.empty() && !frame.refObject.empty())
+    {
+        computeCenterCOParameters(selection, journey, centerTime);
+        observerMode = Travelling;
+    }
+}
+
+
 void Observer::setMode(Observer::ObserverMode mode)
 {
     observerMode = mode;
@@ -468,8 +768,32 @@ void Observer::setMode(Observer::ObserverMode mode)
 void Observer::setFrame(const FrameOfReference& _frame)
 {
     RigidTransform transform = frame.toUniversal(situation, getTime());
-    frame = _frame;
-    situation = frame.fromUniversal(transform, getTime());
+    if (observerMode == Travelling)
+    {
+        RigidTransform from = frame.toUniversal(RigidTransform(journey.from, journey.initialOrientation), getTime());
+        RigidTransform to = frame.toUniversal(RigidTransform(journey.to, journey.finalOrientation), getTime());
+
+        frame = _frame;
+        situation = frame.fromUniversal(transform, getTime());
+
+        from = frame.fromUniversal(from, getTime());
+        journey.from = from.translation;
+        journey.initialOrientation = Quatf((float) from.rotation.w,
+                                           (float) from.rotation.x,
+                                           (float) from.rotation.y,
+                                           (float) from.rotation.z);
+        to = frame.fromUniversal(to, getTime());
+        journey.to = to.translation;
+        journey.finalOrientation = Quatf((float) to.rotation.w,
+                                         (float) to.rotation.x,
+                                         (float) to.rotation.y,
+                                         (float) to.rotation.z);
+    }
+    else
+    {
+        frame = _frame;
+        situation = frame.fromUniversal(transform, getTime());
+    }
 }
 
 
@@ -582,7 +906,8 @@ void Observer::setTargetSpeed(float s)
 {
     targetSpeed = s;
     Vec3f v;
-
+    if (reverseFlag)
+        s = -s;
     if (trackObject.empty())
     {
         trackingOrientation = getOrientation();
@@ -608,8 +933,70 @@ float Observer::getTargetSpeed()
 }
 
 
+void Observer::gotoJourney(const JourneyParams& params)
+{
+    journey = params;
+    double distance = astro::microLightYearsToKilometers(journey.from.distanceTo(journey.to)) / 2.0;
+    pair<double, double> sol = solve_bisection(TravelExpFunc(distance, journey.accelTime),
+                                               0.0001, 100.0,
+                                               1e-10);
+    journey.expFactor = sol.first;
+    journey.startTime = realTime;
+    observerMode = Travelling;
+}
+
 void Observer::gotoSelection(const Selection& selection,
-                             double gotoTime, 
+                             double gotoTime,
+                             Vec3f up,
+                             astro::CoordinateSystem upFrame)
+{
+    gotoSelection(selection, gotoTime, 0.0, 0.5, up, upFrame);
+}
+
+
+// Return the preferred distance for viewing an object
+static double getPreferredDistance(const Selection& selection)
+{
+    switch (selection.getType())
+    {
+    case Selection::Type_Body:
+        return 5.0 * selection.radius();
+    case Selection::Type_DeepSky:
+        return 5.0 * selection.radius();
+    case Selection::Type_Star:
+        return 100.0 * selection.radius();
+    case Selection::Type_Location:
+        {
+            double maxDist = getPreferredDistance(selection.location()->getParentBody());
+            return max(min(selection.location()->getSize() * 50.0, maxDist),
+                       1.0);
+        }
+    default:
+        return 1.0;
+    }
+}
+
+
+// Given an object and its current distance from the camera, determine how
+// close we should go on the next goto.
+static double getOrbitDistance(const Selection& selection,
+                               double currentDistance)
+{
+    // If further than 10 times the preferrred distance, goto the
+    // preferred distance.  If closer, zoom in 10 times closer or to the
+    // minimum distance.
+    double maxDist = astro::kilometersToMicroLightYears(getPreferredDistance(selection));
+    double minDist = astro::kilometersToMicroLightYears(1.01 * selection.radius());
+    double dist = (currentDistance > maxDist * 10.0) ? maxDist : currentDistance * 0.1;
+
+    return max(dist, minDist);
+}
+
+
+void Observer::gotoSelection(const Selection& selection,
+                             double gotoTime,
+                             double startInter,
+                             double endInter,
                              Vec3f up,
                              astro::CoordinateSystem upFrame)
 {
@@ -619,31 +1006,64 @@ void Observer::gotoSelection(const Selection& selection,
         Vec3d v = pos - getPosition();
         double distance = v.length();
 
-        double maxOrbitDistance;
-        if (selection.body != NULL)
-            maxOrbitDistance = astro::kilometersToMicroLightYears(5.0f * selection.body->getRadius());
-        else if (selection.deepsky != NULL)
-            maxOrbitDistance = 5.0f * selection.deepsky->getRadius() * 1e6f;
-        else if (selection.star != NULL)
-            maxOrbitDistance = astro::kilometersToMicroLightYears(100.0f * selection.star->getRadius());
-        else
-            maxOrbitDistance = 0.5f;
-
-        double radius = selection.radius();
-        double minOrbitDistance = astro::kilometersToMicroLightYears(1.01 * radius);
-
-        double orbitDistance = (distance > maxOrbitDistance * 10.0f) ? maxOrbitDistance : distance * 0.1f;
-        if (orbitDistance < minOrbitDistance)
-            orbitDistance = minOrbitDistance;
+        double orbitDistance = getOrbitDistance(selection, distance);
 
         computeGotoParameters(selection, journey, gotoTime,
-                              v * -(orbitDistance / distance), astro::Universal,
+                              startInter, endInter,
+                              v * -(orbitDistance / distance),
+                              astro::Universal,
                               up, upFrame);
         observerMode = Travelling;
     }
 }
 
 
+// Like normal goto, except we'll follow a great circle trajectory.  Useful
+// for travelling between surface locations, where we'd rather not go straight
+// through the middle of a planet.
+void Observer::gotoSelectionGC(const Selection& selection,
+                               double gotoTime,
+                               double startInter,
+                               double endInter,
+                               Vec3f up,
+                               astro::CoordinateSystem upFrame)
+{
+    if (!selection.empty())
+    {
+        Selection centerObj = selection.parent();
+
+        UniversalCoord pos = selection.getPosition(getTime());
+        Vec3d v = pos - centerObj.getPosition(getTime());
+        double distanceToCenter = v.length();
+        Vec3d viewVec = pos - getPosition();
+        double orbitDistance = getOrbitDistance(selection,
+                                                viewVec.length());
+        if (selection.location() != NULL)
+        {
+            Selection parent = selection.parent();
+            double maintainDist = astro::kilometersToMicroLightYears(getPreferredDistance(parent));
+            Vec3d parentPos = parent.getPosition(getTime()) - getPosition();
+            double parentDist = parentPos.length() -
+                astro::kilometersToMicroLightYears(parent.radius());
+
+            if (parentDist <= maintainDist && parentDist > orbitDistance)
+            {
+                orbitDistance = parentDist;
+            }
+        }
+
+        computeGotoParametersGC(selection, journey, gotoTime,
+                                //startInter, endInter,
+                                0.25, 0.75,
+                                v * (orbitDistance / distanceToCenter),
+                                astro::Universal,
+                                up, upFrame,
+                                centerObj);
+        observerMode = Travelling;
+    }
+}
+
+
 void Observer::gotoSelection(const Selection& selection,
                              double gotoTime,
                              double distance,
@@ -653,10 +1073,12 @@ void Observer::gotoSelection(const Selection& selection,
     if (!selection.empty())
     {
         UniversalCoord pos = selection.getPosition(getTime());
+        // The destination position lies along the line between the current
+        // position and the star
         Vec3d v = pos - getPosition();
         v.normalize();
 
-        computeGotoParameters(selection, journey, gotoTime,
+        computeGotoParameters(selection, journey, gotoTime, 0.25, 0.75,
                               v * -distance * 1e6, astro::Universal,
                               up, upFrame);
         observerMode = Travelling;
@@ -664,6 +1086,31 @@ void Observer::gotoSelection(const Selection& selection,
 }
 
 
+void Observer::gotoSelectionGC(const Selection& selection,
+                               double gotoTime,
+                               double distance,
+                               Vec3f up,
+                               astro::CoordinateSystem upFrame)
+{
+    if (!selection.empty())
+    {
+        Selection centerObj = selection.parent();
+
+        UniversalCoord pos = selection.getPosition(getTime());
+        Vec3d v = pos - centerObj.getPosition(getTime());
+        v.normalize();
+
+        // The destination position lies along a line extended from the center
+        // object to the target object
+        computeGotoParametersGC(selection, journey, gotoTime, 0.25, 0.75,
+                                v * -distance * 1e6, astro::Universal,
+                                up, upFrame,
+                                centerObj);
+        observerMode = Travelling;
+    }
+}
+
+
 void Observer::gotoSelectionLongLat(const Selection& selection,
                                     double gotoTime,
                                     double distance,
@@ -678,7 +1125,7 @@ void Observer::gotoSelectionLongLat(const Selection& selection,
         double x = cos(theta) * sin(phi);
         double y = cos(phi);
         double z = -sin(theta) * sin(phi);
-        computeGotoParameters(selection, journey, gotoTime,
+        computeGotoParameters(selection, journey, gotoTime, 0.25, 0.75,
                               Vec3d(x, y, z) * distance * 1e6, astro::Geographic,
                               up, astro::Geographic);
         observerMode = Travelling;
@@ -703,6 +1150,9 @@ void Observer::gotoLocation(const RigidTransform& transform,
                                      (float) transform.rotation.x,
                                      (float) transform.rotation.y,
                                      (float) transform.rotation.z);
+    journey.startInterpolation = 0.25f;
+    journey.endInterpolation   = 0.75f;
+
 
     journey.accelTime = 0.5;
     double distance = astro::microLightYearsToKilometers(journey.from.distanceTo(journey.to)) / 2.0;
@@ -724,7 +1174,7 @@ void Observer::getSelectionLongLat(const Selection& selection,
     // respect to currently selected object.
     if (!selection.empty())
     {
-        FrameOfReference refFrame(astro::Geographic, selection.body);
+        FrameOfReference refFrame(astro::Geographic, selection);
         RigidTransform xform = refFrame.fromUniversal(RigidTransform(getPosition(), getOrientation()),
                                                       getTime());
 
@@ -797,20 +1247,21 @@ void Observer::follow(const Selection& selection)
 
 void Observer::geosynchronousFollow(const Selection& selection)
 {
-    if (selection.body != NULL)
+    if (selection.body() != NULL ||
+        selection.location() != NULL)
     {
-        setFrame(FrameOfReference(astro::Geographic, selection.body));
+        setFrame(FrameOfReference(astro::Geographic, selection));
     }
 }
 
 void Observer::phaseLock(const Selection& selection)
 {
-    if (frame.refObject.body != NULL)
+    if (frame.refObject.body() != NULL)
     {
         if (selection == frame.refObject)
         {
             setFrame(FrameOfReference(astro::PhaseLock, selection,
-                                      Selection(selection.body->getSystem()->getStar())));
+                                      Selection(selection.body()->getSystem()->getStar())));
         }
         else
         {
@@ -821,9 +1272,9 @@ void Observer::phaseLock(const Selection& selection)
 
 void Observer::chase(const Selection& selection)
 {
-    if (selection.body != NULL)
+    if (selection.body() != NULL)
     {
-        setFrame(FrameOfReference(astro::Chase, selection.body));
+        setFrame(FrameOfReference(astro::Chase, selection));
     }
 }
 
