@@ -32,6 +32,7 @@ struct RenderListEntry
     float discSizeInPixels;
     float appMag;
     bool isCometTail;
+    int depthBucket;
 };
 
 class Renderer
@@ -39,8 +40,17 @@ class Renderer
  public:
     Renderer();
     ~Renderer();
-    
-    bool init(GLContext*, int, int);
+
+    struct DetailOptions
+    {
+        DetailOptions();
+        unsigned int ringSystemSections;
+        unsigned int orbitPathSamplePoints;
+        unsigned int shadowTextureSize;
+        unsigned int eclipseTextureSize;
+    };
+
+    bool init(GLContext*, int, int, DetailOptions&);
     void shutdown() {};
     void resize(int, int);
 
@@ -56,15 +66,17 @@ class Renderer
                 const Selection& sel);
     
     enum {
-        NoLabels            = 0x00,
-        StarLabels          = 0x01,
-        PlanetLabels        = 0x02,
-        MoonLabels          = 0x04,
-        ConstellationLabels = 0x08,
-        GalaxyLabels        = 0x10,
-        AsteroidLabels      = 0x20,
-        SpacecraftLabels    = 0x40,
-        BodyLabelMask       = (PlanetLabels | MoonLabels | AsteroidLabels | SpacecraftLabels),
+        NoLabels            = 0x000,
+        StarLabels          = 0x001,
+        PlanetLabels        = 0x002,
+        MoonLabels          = 0x004,
+        ConstellationLabels = 0x008,
+        GalaxyLabels        = 0x010,
+        AsteroidLabels      = 0x020,
+        SpacecraftLabels    = 0x040,
+        LocationLabels      = 0x080,
+        CometLabels         = 0x100,
+        BodyLabelMask       = (PlanetLabels | MoonLabels | AsteroidLabels | SpacecraftLabels | CometLabels),
     };
 
     enum {
@@ -86,19 +98,33 @@ class Renderer
         ShowAutoMag         = 0x4000,
         ShowCometTails      = 0x8000,
         ShowMarkers         = 0x10000,
+        ShowPartialTrajectories = 0x20000,
+    };
+
+    enum StarStyle 
+    {
+        FuzzyPointStars  = 0,
+        PointStars       = 1,
+        ScaledDiscStars  = 2,
+        StarStyleCount   = 3,
     };
 
     int getRenderFlags() const;
     void setRenderFlags(int);
     int getLabelMode() const;
     void setLabelMode(int);
-    void addLabelledStar(Star*);
+    void addLabelledStar(Star*, const std::string&);
     void clearLabelledStars();
     float getAmbientLightLevel() const;
     void setAmbientLightLevel(float);
+    float getMinimumOrbitSize() const;
     void setMinimumOrbitSize(float);
+    float getMinimumFeatureSize() const;
+    void setMinimumFeatureSize(float);
     float getDistanceLimit() const;
     void setDistanceLimit(float);
+    int getOrbitMask() const;
+    void setOrbitMask(int);
 
     bool getFragmentShaderEnabled() const;
     void setFragmentShaderEnabled(bool);
@@ -113,6 +139,8 @@ class Renderer
     void setSaturationMagnitude(float);
     float getBrightnessBias() const;
     void setBrightnessBias(float);
+    void setStarStyle(StarStyle);
+    StarStyle getStarStyle() const;
     void setResolution(unsigned int resolution);
     unsigned int getResolution();
 
@@ -124,7 +152,8 @@ class Renderer
         Point3f position;
     } Label;
 
-    void addLabel(std::string, Color, Point3f, float depth = -1);
+    void addLabel(std::string, Color, const Point3f&, float depth = -1);
+    void addSortedLabel(std::string, Color, const Point3f&);
     void clearLabels();
 
     void setFont(TextureFont*);
@@ -158,9 +187,10 @@ class Renderer
             rings(NULL),
             radius(1.0f),
             oblateness(0.0f),
-            mesh(InvalidResource),
+            model(InvalidResource),
             orientation(1.0f),
-            eclipseShadows(NULL)
+            eclipseShadows(NULL),
+            locations(NULL)
         {};
 
         Surface* surface;
@@ -168,9 +198,10 @@ class Renderer
         RingSystem* rings;
         float radius;
         float oblateness;
-        ResourceHandle mesh;
+        ResourceHandle model;
         Quatf orientation;
         std::vector<EclipseShadow>* eclipseShadows;
+        std::vector<Location*>* locations;
     };
 
     class StarVertexBuffer
@@ -194,6 +225,39 @@ class Renderer
         bool usePoints;
     };
 
+ private:
+    struct SkyVertex
+    {
+        float x, y, z;
+        unsigned char color[4];
+    };
+
+    struct SkyContourPoint
+    {
+        Vec3f v;
+        Vec3f eyeDir;
+        float centerDist;
+        float eyeDist;
+        float cosSkyCapAltitude;
+    };
+
+    struct StarLabel
+    {
+        Star* star;
+        string label;
+
+        StarLabel() : star(NULL), label("") {};
+        StarLabel(Star* _star, const std::string& _label) :
+            star(_star), label(_label) {};
+        StarLabel(const StarLabel& sl) : star(sl.star), label(sl.label) {};
+        StarLabel& operator=(const StarLabel& sl)
+        {
+            star = sl.star;
+            label = sl.label;
+            return *this;
+        };
+    };
+
  private:
     void setFieldOfView(float);
     void renderStars(const StarDatabase& starDB,
@@ -218,7 +282,7 @@ class Renderer
                       Color sunColor,
                       RenderProperties& obj);
 
-    void renderPlanet(const Body& body,
+    void renderPlanet(Body& body,
                       Point3f pos,
                       Vec3f sunDirection,
                       float distance,
@@ -253,11 +317,26 @@ class Renderer
                               float renderDistance,
                               bool useHaloes);
 
+    void renderEllipsoidAtmosphere(const Atmosphere& atmosphere,
+                                   Point3f center,
+                                   const Quatf& orientation,
+                                   Vec3f semiAxes,
+                                   const Vec3f& sunDirection,
+                                   Color ambientColor,
+                                   float fade,
+                                   bool lit);
+
+    void renderLocations(const vector<Location*>& locations,
+                         const Quatf& cameraOrientation,
+                         const Point3f& position,
+                         const Quatf& orientation,
+                         float scale);
+
     bool testEclipse(const Body&, const Body&, double now);
 
     void labelGalaxies(const DeepSkyCatalog& catalog,
                        const Observer& observer);
-    void labelStars(const std::vector<Star*>& stars,
+    void labelStars(const std::vector<StarLabel>& stars,
                     const StarDatabase& starDB,
                     const Observer& observer);
     void labelConstellations(const AsterismList& asterisms,
@@ -265,6 +344,8 @@ class Renderer
     void renderParticles(const std::vector<Particle>& particles,
                          Quatf orientation);
     void renderLabels();
+    std::vector<Label>::iterator renderSortedLabels(std::vector<Label>::iterator,
+                                                    float depth);
     void renderMarkers(const MarkerList&,
                        const UniversalCoord& position,
                        const Quatf& orientation,
@@ -289,6 +370,7 @@ class Renderer
     int renderMode;
     int labelMode;
     int renderFlags;
+    int orbitMask;
     float ambientLightLevel;
     bool fragmentShaderEnabled;
     bool vertexShaderEnabled;
@@ -299,16 +381,19 @@ class Renderer
     float faintestPlanetMag;
     float saturationMagNight;
     float saturationMag;
+    StarStyle starStyle;
 
     Color ambientColor;
+    std::string displayedSurface;
 
     StarVertexBuffer* starVertexBuffer;
     std::vector<RenderListEntry> renderList;
     std::vector<Particle> glareParticles;
     std::vector<Label> labels;
+    std::vector<Label> depthSortedLabels;
     std::vector<EclipseShadow> eclipseShadows;
 
-    std::vector<Star*> labelledStars;
+    std::vector<StarLabel> labelledStars;
 
     double modelMatrix[16];
     double projMatrix[16];
@@ -320,16 +405,32 @@ class Renderer
     bool useClampToBorder;
     unsigned int textureResolution;
 
+    DetailOptions detailOptions;
+
+ public:
+    struct OrbitSample 
+    {
+        double t;
+        Point3f pos;
+    };
+
+ private:
     struct CachedOrbit
     {
         Body* body;
-        std::vector<Point3f> trajectory;
+        std::vector<OrbitSample> trajectory;
         bool keep;
     };
     std::vector<CachedOrbit*> orbitCache;
 
     float minOrbitSize;
     float distanceLimit;
+    float minFeatureSize;
+    uint32 locationFilter;
+
+    SkyVertex* skyVertices;
+    uint32* skyIndices;
+    SkyContourPoint* skyContour;
 };
 
 #endif // _RENDER_H_
