@@ -59,6 +59,7 @@ class Simulation
     void selectPlanet(int);
     Selection findObject(std::string s);
     Selection findObjectFromPath(std::string s);
+    std::vector<std::string> getObjectCompletion(std::string s, bool withLocations = false);
     void gotoSelection(double gotoTime,
                        Vec3f up, astro::CoordinateSystem upFrame);
     void gotoSelection(double gotoTime, double distance,
@@ -73,6 +74,7 @@ class Simulation
                              double& latitude);
     void gotoSurface(double duration);
     void centerSelection(double centerTime = 0.5);
+    void centerSelectionCO(double centerTime = 0.5);
     void follow();
     void geosynchronousFollow();
     void phaseLock();
