@@ -32,6 +32,59 @@
 
 using namespace std;
 
+enum Disposition
+{
+    AddObject,
+    ReplaceObject,
+    ModifyObject,
+};
+
+
+/*!
+  Solar system catalog (.ssc) files contain items of three different types:
+  bodies, locations, and alternate surfaces.  Bodies planets, moons, asteroids,
+  comets, and spacecraft.  Locations are points on the surfaces of bodies which
+  may be labelled but aren't rendered.  Alternate surfaces are additional
+  surface definitions for bodies.
+
+  An ssc file contains zero or more definitions of this form:
+
+  \code
+  [disposition] [item type] "name" "parent name"
+  {
+     ...object info fields...
+  }
+  \endcode
+
+  The disposition of the object determines what happens if an item with the
+  same parent and same name already exists.  It may be one of the following:
+  - Add - Default if none is specified.  Add the item even if one of the
+    same name already exists.
+  - Replace - Replace an existing item with the new one
+  - Modify - Modify the existing item, changing the fields that appear
+    in the new definition.
+
+  All dispositions are equivalent to add if no item of the same name
+  already exists.
+
+  The item type is one of Body, Location, or AltSurface, defaulting to
+  Body when no type is given.
+
+  The name and parent name are both mandatory.
+*/
+
+static void errorMessagePrelude(const Tokenizer& tok)
+{
+    cerr << "Error in .ssc file (line " << tok.getLineNumber() << "): ";
+}
+
+static void sscError(const Tokenizer& tok,
+                     const string& msg)
+{
+    errorMessagePrelude(tok);
+    cerr << msg << '\n';
+}
+
 
 bool getDate(Hash* hash, const string& name, double& jd)
 {
@@ -54,20 +107,48 @@ bool getDate(Hash* hash, const string& name, double& jd)
 }
 
 
-static Surface* CreateSurface(Hash* surfaceData,
-                              const std::string& path)
+static Location* CreateLocation(Hash* locationData,
+                                Body* body)
 {
-    Surface* surface = new Surface();
+    Location* location = new Location();
 
-    surface->color = Color(1.0f, 1.0f, 1.0f);
+    Vec3d longlat(0.0, 0.0, 0.0);
+    locationData->getVector("LongLat", longlat);
+
+    Vec3f position = body->planetocentricToCartesian((float) longlat.x,
+                                                     (float) longlat.y,
+                                                     (float) longlat.z);
+    location->setPosition(position);
+
+    double size = 1.0;
+    locationData->getNumber("Size", size);
+    location->setSize((float) size);
+
+    double importance = -1.0;
+    locationData->getNumber("Importance", importance);
+    location->setImportance((float) importance);
+
+    string featureTypeName;
+    if (locationData->getString("Type", featureTypeName))
+        location->setFeatureType(Location::parseFeatureType(featureTypeName));
+
+    return location;
+}
+
+
+static void FillinSurface(Hash* surfaceData,
+                          Surface* surface,
+                          const std::string& path)
+{
     surfaceData->getColor("Color", surface->color);
 
-    Color hazeColor;
-    float hazeDensity = 0.0f;
-    surfaceData->getColor("HazeColor", hazeColor);
-    surfaceData->getNumber("HazeDensity", hazeDensity);
-    surface->hazeColor = Color(hazeColor.red(), hazeColor.green(),
-                               hazeColor.blue(), hazeDensity);
+    Color hazeColor = surface->hazeColor;
+    float hazeDensity = hazeColor.alpha();
+    if (surfaceData->getColor("HazeColor", hazeColor) | surfaceData->getNumber("HazeDensity", hazeDensity))
+    {
+        surface->hazeColor = Color(hazeColor.red(), hazeColor.green(),
+                                   hazeColor.blue(), hazeDensity);
+    }
 
     surfaceData->getColor("SpecularColor", surface->specularColor);
     surfaceData->getNumber("SpecularPower", surface->specularPower);
@@ -77,17 +158,20 @@ static Surface* CreateSurface(Hash* surfaceData,
     string nightTexture;
     string specularTexture;
     string normalTexture;
+    string overlayTexture;
     bool applyBaseTexture = surfaceData->getString("Texture", baseTexture);
     bool applyBumpMap = surfaceData->getString("BumpMap", bumpTexture);
     bool applyNightMap = surfaceData->getString("NightTexture", nightTexture);
     bool separateSpecular = surfaceData->getString("SpecularTexture",
                                                    specularTexture);
     bool applyNormalMap = surfaceData->getString("NormalMap", normalTexture);
+    bool applyOverlay = surfaceData->getString("OverlayTexture",
+                                               overlayTexture);
 
-    unsigned int baseFlags = Texture::WrapTexture | Texture::AllowSplitting;
-    unsigned int bumpFlags = Texture::WrapTexture | Texture::AllowSplitting;
-    unsigned int nightFlags = Texture::WrapTexture | Texture::AllowSplitting;
-    unsigned int specularFlags = Texture::WrapTexture | Texture::AllowSplitting;
+    unsigned int baseFlags = TextureInfo::WrapTexture | TextureInfo::AllowSplitting;
+    unsigned int bumpFlags = TextureInfo::WrapTexture | TextureInfo::AllowSplitting;
+    unsigned int nightFlags = TextureInfo::WrapTexture | TextureInfo::AllowSplitting;
+    unsigned int specularFlags = TextureInfo::WrapTexture | TextureInfo::AllowSplitting;
     
     float bumpHeight = 2.5f;
     surfaceData->getNumber("BumpHeight", bumpHeight);
@@ -101,7 +185,7 @@ static Surface* CreateSurface(Hash* surfaceData,
     bool compressTexture = false;
     surfaceData->getBoolean("CompressTexture", compressTexture);
     if (compressTexture)
-        baseFlags |= Texture::CompressTexture;
+        baseFlags |= TextureInfo::CompressTexture;
 
     if (blendTexture)
         surface->appearanceFlags |= Surface::BlendTexture;
@@ -115,6 +199,8 @@ static Surface* CreateSurface(Hash* surfaceData,
         surface->appearanceFlags |= Surface::ApplyNightMap;
     if (separateSpecular)
         surface->appearanceFlags |= Surface::SeparateSpecularMap;
+    if (applyOverlay)
+        surface->appearanceFlags |= Surface::ApplyOverlay;
     if (surface->specularColor != Color(0.0f, 0.0f, 0.0f))
         surface->appearanceFlags |= Surface::SpecularReflection;
 
@@ -131,7 +217,8 @@ static Surface* CreateSurface(Hash* surfaceData,
     else if (applyBumpMap)
         surface->bumpTexture.setTexture(bumpTexture, path, bumpHeight, bumpFlags);
 
-    return surface;
+    if (applyOverlay)
+        surface->overlayTexture.setTexture(overlayTexture, path, baseFlags);
 }
 
 
@@ -213,108 +300,147 @@ static EllipticalOrbit* CreateEllipticalOrbit(Hash* orbitData,
 }
 
 
-static RotationElements CreateRotationElements(Hash* rotationData,
-                                               float orbitalPeriod)
+static void FillinRotationElements(Hash* rotationData,
+                                   RotationElements& re,
+                                   float orbitalPeriod)
 {
-    RotationElements re;
-
     // The default is synchronous rotation (rotation period == orbital period)
-    float period = orbitalPeriod * 24.0f;
-    rotationData->getNumber("RotationPeriod", period);
-    re.period = period / 24.0f;
+    float period = orbitalPeriod;
+    if (rotationData->getNumber("RotationPeriod", period))
+        re.period = period / 24.0f;
 
     float offset = 0.0f;
-    rotationData->getNumber("RotationOffset", offset);
-    re.offset = degToRad(offset);
+    if (rotationData->getNumber("RotationOffset", offset))
+        re.offset = degToRad(offset);
 
     rotationData->getNumber("RotationEpoch", re.epoch);
 
     float obliquity = 0.0f;
-    rotationData->getNumber("Obliquity", obliquity);
-    re.obliquity = degToRad(obliquity);
+    if (rotationData->getNumber("Obliquity", obliquity))
+        re.obliquity = degToRad(obliquity);
 
     float ascendingNode = 0.0f;
-    rotationData->getNumber("EquatorAscendingNode", ascendingNode);
-    re.ascendingNode = degToRad(ascendingNode);
+    if (rotationData->getNumber("EquatorAscendingNode", ascendingNode))
+        re.ascendingNode = degToRad(ascendingNode);
 
     float precessionRate = 0.0f;
-    rotationData->getNumber("PrecessionRate", precessionRate);
-    re.precessionRate = degToRad(precessionRate);
-
-    return re;
+    if (rotationData->getNumber("PrecessionRate", precessionRate))
+        re.precessionRate = degToRad(precessionRate);
 }
 
 
+static Orbit* CreateOrbit(PlanetarySystem* system,
+                          Hash* planetData,
+                          const string& path,
+                          bool usePlanetUnits)
+{
+    Orbit* orbit = NULL;
+
+    string customOrbitName;
+    if (planetData->getString("CustomOrbit", customOrbitName))
+    {
+        orbit = GetCustomOrbit(customOrbitName);
+        if (orbit != NULL)
+        {
+            return orbit;
+        }
+        DPRINTF(0, "Could not find custom orbit named '%s'\n",
+                customOrbitName.c_str());
+    }
+
+    string sampOrbitFile;
+    if (planetData->getString("SampledOrbit", sampOrbitFile))
+    {
+        DPRINTF(1, "Attempting to load sampled orbit file '%s'\n",
+                sampOrbitFile.c_str());
+        ResourceHandle orbitHandle =
+            GetTrajectoryManager()->getHandle(TrajectoryInfo(sampOrbitFile, path));
+        orbit = GetTrajectoryManager()->find(orbitHandle);
+        if (orbit != NULL)
+        {
+            return orbit;
+        }
+        DPRINTF(0, "Could not load sampled orbit file '%s'\n",
+                sampOrbitFile.c_str());
+    }
+
+    Value* orbitDataValue = planetData->getValue("EllipticalOrbit");
+    if (orbitDataValue != NULL)
+    {
+        if (orbitDataValue->getType() != Value::HashType)
+        {
+            DPRINTF(0, "Object has incorrect elliptical orbit syntax.\n");
+            return NULL;
+        }
+        else
+        {
+            return CreateEllipticalOrbit(orbitDataValue->getHash(),
+                                         usePlanetUnits);
+        }
+    }
+
+    Vec3d longlat(0.0, 0.0, 0.0);
+    if (planetData->getVector("LongLat", longlat))
+    {
+        Body* parent = system->getPrimaryBody();
+        if (parent != NULL)
+        {
+            Vec3f pos = parent->planetocentricToCartesian((float) longlat.x, (float) longlat.y, (float) longlat.z);
+            Point3d posd(pos.x, pos.y, pos.z);
+            return new SynchronousOrbit(*parent, posd);
+        }
+        else
+        {
+            // TODO: Allow fixing objects to the surface of stars.
+        }
+        return NULL;
+    }
+
+    return NULL;
+}
+
 // Create a body (planet or moon) using the values from a hash
 // The usePlanetsUnits flags specifies whether period and semi-major axis
 // are in years and AU rather than days and kilometers
 static Body* CreatePlanet(PlanetarySystem* system,
+                          Body* existingBody,
                           Hash* planetData,
                           const string& path,
+                          Disposition disposition,
                           bool usePlanetUnits = true)
 {
-    Body* body = new Body(system);
-
-    Orbit* orbit = NULL;
-    string customOrbitName;
-
-    if (planetData->getString("CustomOrbit", customOrbitName))
+    Body* body = NULL;
+  
+    if (disposition == ModifyObject)
     {
-        orbit = GetCustomOrbit(customOrbitName);
-        if (orbit == NULL)
-            DPRINTF(0, "Could not find custom orbit named '%s'\n",
-                    customOrbitName.c_str());
+        body = existingBody;
     }
 
-    if (orbit == NULL)
+    if (body == NULL)
     {
-        string sampOrbitFile;
-        if (planetData->getString("SampledOrbit", sampOrbitFile))
-        {
-            DPRINTF(1, "Attempting to load sampled orbit file '%s'\n",
-                    sampOrbitFile.c_str());
-            ResourceHandle orbitHandle =
-                GetTrajectoryManager()->getHandle(TrajectoryInfo(sampOrbitFile, path));
-            orbit = GetTrajectoryManager()->find(orbitHandle);
-            if (orbit == NULL)
-            {
-                DPRINTF(0, "Could not load sampled orbit file '%s'\n",
-                        sampOrbitFile.c_str());
-            }
-        }
+        body = new Body(system);
     }
 
-    if (orbit == NULL)
+    Orbit* orbit = CreateOrbit(system, planetData, path, usePlanetUnits);
+
+    if (orbit != NULL)
     {
-        Value* orbitDataValue = planetData->getValue("EllipticalOrbit");
-        if (orbitDataValue != NULL)
-        {
-            if (orbitDataValue->getType() != Value::HashType)
-            {
-                DPRINTF(0, "Object '%s' has incorrect elliptical orbit syntax.\n",
-                        body->getName().c_str());
-            }
-            else
-            {
-                orbit = CreateEllipticalOrbit(orbitDataValue->getHash(),
-                                              usePlanetUnits);
-            }
-        }
+        body->setOrbit(orbit);
     }
-    if (orbit == NULL)
+
+    if (body->getOrbit() == NULL)
     {
         DPRINTF(0, "No valid orbit specified for object '%s'; skipping . . .\n",
                 body->getName().c_str());
         delete body;
         return NULL;
     }
-    body->setOrbit(orbit);
 
-    double radius = 10000.0;
+    double radius = (double)body->getRadius();
     planetData->getNumber("Radius", radius);
-    body->setRadius(radius);
+    body->setRadius((float) radius);
 
-    int classification = Body::Unknown;
+    int classification = body->getClassification();
     string classificationName;
     if (planetData->getString("Class", classificationName))
     {
@@ -330,8 +456,6 @@ static Body* CreatePlanet(PlanetarySystem* system,
             classification = Body::Spacecraft;
         else if (compareIgnoringCase(classificationName, "invisible") == 0)
             classification = Body::Invisible;
-        else
-            classification = Body::Unknown;
     }
 
     if (classification == Body::Unknown)
@@ -359,6 +483,7 @@ static Body* CreatePlanet(PlanetarySystem* system,
     // double ending      =  numeric_limits<double>::infinity();
     double beginning   = -1.0e+50;
     double ending      =  1.0e+50;
+    body->getLifespan(beginning, ending);
     getDate(planetData, "Beginning", beginning);
     getDate(planetData, "Ending", ending);
     body->setLifespan(beginning, ending);
@@ -368,29 +493,54 @@ static Body* CreatePlanet(PlanetarySystem* system,
         body->setInfoURL(infoURL);
     
     double albedo = 0.5;
-    planetData->getNumber("Albedo", albedo);
-    body->setAlbedo(albedo);
+    if (planetData->getNumber("Albedo", albedo))
+        body->setAlbedo((float) albedo);
 
     double oblateness = 0.0;
-    planetData->getNumber("Oblateness", oblateness);
-    body->setOblateness(oblateness);
+    if (planetData->getNumber("Oblateness", oblateness))
+        body->setOblateness((float) oblateness);
+    
+    double mass = 0.0;
+    if (planetData->getNumber("Mass", mass))
+        body->setMass((float) mass);
 
     Quatf orientation;
     if (planetData->getRotation("Orientation", orientation))
         body->setOrientation(orientation);
 
-    body->setRotationElements(CreateRotationElements(planetData, orbit->getPeriod()));
+    RotationElements re = body->getRotationElements();
+    if (disposition != ModifyObject)
+        re.period = (float) body->getOrbit()->getPeriod();
+    FillinRotationElements(planetData, re, (float) body->getOrbit()->getPeriod());
+    body->setRotationElements(re);
 
-    Surface* surface = CreateSurface(planetData, path);
-    body->setSurface(*surface);
-    delete surface;
+    Surface surface;
+    if (disposition == ModifyObject)
+    {
+        surface = body->getSurface();
+    }
+    else
+    {
+        surface.color = Color(1.0f, 1.0f, 1.0f);
+        surface.hazeColor = Color(0.0f, 0.0f, 0.0f, 0.0f);
+    }
+    FillinSurface(planetData, &surface, path);
+    body->setSurface(surface);
 
     {
-        string mesh("");
-        if (planetData->getString("Mesh", mesh))
+        string model("");
+        if (planetData->getString("Mesh", model))
         {
-            ResourceHandle meshHandle = GetMeshManager()->getHandle(MeshInfo(mesh, path));
-            body->setMesh(meshHandle);
+            Vec3f modelCenter(0.0f, 0.0f, 0.0f);
+            if (planetData->getVector("MeshCenter", modelCenter))
+            {
+                // TODO: Adjust bounding radius if model center isn't
+                // (0.0f, 0.0f, 0.0f)
+            }
+
+            ResourceHandle modelHandle = GetModelManager()->getHandle(ModelInfo(model, path, modelCenter));
+            body->setModel(modelHandle);
+
         }
     }
 
@@ -408,26 +558,44 @@ static Body* CreatePlanet(PlanetarySystem* system,
                 Hash* atmosData = atmosDataValue->getHash();
                 assert(atmosData != NULL);
                 
-                Atmosphere* atmosphere = new Atmosphere();
+                Atmosphere* atmosphere = NULL;
+                if (disposition == ModifyObject)
+                {
+                    atmosphere = body->getAtmosphere();
+                    if (atmosphere == NULL)
+                    {
+                        Atmosphere atm;
+                        body->setAtmosphere(atm);
+                        atmosphere = body->getAtmosphere();
+                    }
+                }
+                else
+                {
+                    atmosphere = new Atmosphere();
+                }
                 atmosData->getNumber("Height", atmosphere->height);
                 atmosData->getColor("Lower", atmosphere->lowerColor);
                 atmosData->getColor("Upper", atmosphere->upperColor);
                 atmosData->getColor("Sky", atmosphere->skyColor);
+                atmosData->getColor("Sunset", atmosphere->sunsetColor);
                 atmosData->getNumber("CloudHeight", atmosphere->cloudHeight);
-                atmosData->getNumber("CloudSpeed", atmosphere->cloudSpeed);
-                atmosphere->cloudSpeed = degToRad(atmosphere->cloudSpeed);
+                if (atmosData->getNumber("CloudSpeed", atmosphere->cloudSpeed))
+                    atmosphere->cloudSpeed = degToRad(atmosphere->cloudSpeed);
 
                 string cloudTexture;
                 if (atmosData->getString("CloudMap", cloudTexture))
                 {
                     atmosphere->cloudTexture.setTexture(cloudTexture,
                                                         path,
-                                                        Texture::WrapTexture);
+                                                        TextureInfo::WrapTexture);
                 }
 
                 body->setAtmosphere(*atmosphere);
-                delete atmosphere;
+                if (disposition != ModifyObject)
+                    delete atmosphere;
             }
+
+            delete atmosDataValue;
         }
     }
 
@@ -445,20 +613,28 @@ static Body* CreatePlanet(PlanetarySystem* system,
                 Hash* ringsData = ringsDataValue->getHash();
                 // ASSERT(ringsData != NULL);
 
+                RingSystem rings(0.0f, 0.0f);
+                if (body->getRings() != NULL)
+                    rings = *body->getRings();
+
                 double inner = 0.0, outer = 0.0;
-                ringsData->getNumber("Inner", inner);
-                ringsData->getNumber("Outer", outer);
+                if (ringsData->getNumber("Inner", inner))
+                    rings.innerRadius = inner;
+                if (ringsData->getNumber("Outer", outer))
+                    rings.outerRadius = outer;
 
                 Color color(1.0f, 1.0f, 1.0f);
-                ringsData->getColor("Color", color);
+                if (ringsData->getColor("Color", color))
+                    rings.color = color;
 
                 string textureName;
-                ringsData->getString("Texture", textureName);
-                MultiResTexture ringTex(textureName, path);
+                if (ringsData->getString("Texture", textureName))
+                    rings.texture = MultiResTexture(textureName, path);
 
-                body->setRings(RingSystem((float) inner, (float) outer,
-                                          color, ringTex));
+                body->setRings(rings);
             }
+
+            delete ringsDataValue;
         }
     }
 
@@ -475,16 +651,45 @@ bool LoadSolarSystemObjects(istream& in,
 
     while (tokenizer.nextToken() != Tokenizer::TokenEnd)
     {
+        // Read the disposition; if none is specified, the default is Add.
+        Disposition disposition = AddObject;
+        if (tokenizer.getTokenType() == Tokenizer::TokenName)
+        {
+            if (tokenizer.getNameValue() == "Add")
+            {
+                disposition = AddObject;
+                tokenizer.nextToken();
+            }
+            else if (tokenizer.getNameValue() == "Replace")
+            {
+                disposition = ReplaceObject;
+                tokenizer.nextToken();
+            }
+            else if (tokenizer.getNameValue() == "Modify")
+            {
+                disposition = ModifyObject;
+                tokenizer.nextToken();
+            }
+        }
+
+        // Read the item type; if none is specified the default is Body
+        string itemType("Body");
+        if (tokenizer.getTokenType() == Tokenizer::TokenName)
+        {
+            itemType = tokenizer.getNameValue();
+            tokenizer.nextToken();
+        }
+
         if (tokenizer.getTokenType() != Tokenizer::TokenString)
         {
-            cout << "Error parsing solar system file.\n";
+            sscError(tokenizer, "object name expected");
             return false;
         }
         string name = tokenizer.getStringValue();
 
         if (tokenizer.nextToken() != Tokenizer::TokenString)
         {
-            cout << "Error parsing solar system file; invalid parent name.\n";
+            sscError(tokenizer, "bad parent object name");
             return false;
         }
         string parentName = tokenizer.getStringValue();
@@ -492,64 +697,109 @@ bool LoadSolarSystemObjects(istream& in,
         Value* objectDataValue = parser.readValue();
         if (objectDataValue == NULL)
         {
-            cout << "Error reading solar system object.\n";
+            sscError(tokenizer, "bad object definition");
             return false;
         }
 
         if (objectDataValue->getType() != Value::HashType)
         {
-            cout << "Bad solar system object.\n";
+            sscError(tokenizer, "{ expected");
             return false;
         }
         Hash* objectData = objectDataValue->getHash();
 
-        DPRINTF(2, "Reading planet %s %s\n", parentName.c_str(), name.c_str());
-
         Selection parent = universe.findPath(parentName, NULL, 0);
         PlanetarySystem* parentSystem = NULL;
-        bool orbitsPlanet = false;
-        if (parent.star != NULL)
-        {
-            SolarSystem* solarSystem = universe.getSolarSystem(parent.star);
-            if (solarSystem == NULL)
-            {
-                // No solar system defined for this star yet, so we need
-                // to create it.
-                solarSystem = universe.createSolarSystem(parent.star);
-            }
-            parentSystem = solarSystem->getPlanets();
-        }
-        else if (parent.body != NULL)
-        {
-            // Parent is a planet or moon
-            parentSystem = parent.body->getSatellites();
-            if (parentSystem == NULL)
-            {
-                // If the planet doesn't already have any satellites, we
-                // have to create a new planetary system for it.
-                parentSystem = new PlanetarySystem(parent.body);
-                parent.body->setSatellites(parentSystem);
-            }
-            orbitsPlanet = true;
-        }
-        else
-        {
-            cout << "Parent body '" << parentName << "' of '" << name << "' not found.\n";
-        }
 
-        if (parentSystem != NULL)
+        if (itemType == "Body")
         {
-            if (parentSystem->find(name))
+            bool orbitsPlanet = false;
+            if (parent.star() != NULL)
             {
-                DPRINTF(0, "Warning duplicate definition of %s %s!\n",
-                        parentName.c_str(), name.c_str());
+                SolarSystem* solarSystem = universe.getSolarSystem(parent.star());
+                if (solarSystem == NULL)
+                {
+                    // No solar system defined for this star yet, so we need
+                    // to create it.
+                    solarSystem = universe.createSolarSystem(parent.star());
+                }
+                parentSystem = solarSystem->getPlanets();
+            }
+            else if (parent.body() != NULL)
+            {
+                // Parent is a planet or moon
+                parentSystem = parent.body()->getSatellites();
+                if (parentSystem == NULL)
+                {
+                    // If the planet doesn't already have any satellites, we
+                    // have to create a new planetary system for it.
+                    parentSystem = new PlanetarySystem(parent.body());
+                    parent.body()->setSatellites(parentSystem);
+                }
+                orbitsPlanet = true;
+            }
+            else
+            {
+                errorMessagePrelude(tokenizer);
+                cerr << "parent body '" << parentName << "' of '" << name << "' not found.\n";
             }
 
-            Body* body = CreatePlanet(parentSystem, objectData, directory, !orbitsPlanet);
-            if (body != NULL)
+            if (parentSystem != NULL)
             {
-                body->setName(name);
-                parentSystem->addBody(body);
+                Body* existingBody = parentSystem->find(name);
+                if (existingBody && disposition == AddObject)
+                {
+                    errorMessagePrelude(tokenizer);
+                    cerr << "warning duplicate definition of " <<
+                        parentName << " " <<  name << '\n';
+                }
+                
+                Body* body = CreatePlanet(parentSystem, existingBody, objectData, directory, disposition, !orbitsPlanet);
+                if (body != NULL)
+                {
+                    body->setName(name);
+                    if (disposition == ReplaceObject)
+                    {
+                        parentSystem->replaceBody(existingBody, body);
+                        delete existingBody;
+                    } 
+                    else if (disposition == AddObject)
+                    {
+                        parentSystem->addBody(body);
+                    }
+                }
+            }
+        }
+        else if (itemType == "AltSurface")
+        {
+            Surface* surface = new Surface();
+            surface->color = Color(1.0f, 1.0f, 1.0f);
+            surface->hazeColor = Color(0.0f, 0.0f, 0.0f, 0.0f);
+            FillinSurface(objectData, surface, directory);
+            if (surface != NULL && parent.body() != NULL)
+                parent.body()->addAlternateSurface(name, surface);
+            else
+                sscError(tokenizer, "bad alternate surface");
+        }
+        else if (itemType == "Location")
+        {
+            if (parent.body() != NULL)
+            {
+                Location* location = CreateLocation(objectData, parent.body());
+                if (location != NULL)
+                {
+                    location->setName(name);
+                    parent.body()->addLocation(location);
+                }
+                else
+                {
+                    sscError(tokenizer, "bad location");
+                }
+            }
+            else
+            {
+                errorMessagePrelude(tokenizer);
+                cerr << "parent body '" << parentName << "' of '" << name << "' not found.\n";
             }
         }
     }
@@ -565,7 +815,7 @@ SolarSystem::SolarSystem(Star* _star) : star(_star)
 }
 
 
-const Star* SolarSystem::getStar() const
+Star* SolarSystem::getStar() const
 {
     return star;
 }
