@@ -100,7 +100,7 @@ Star* StarDatabase::find(const string& name) const
         // Search by the default catalog number
         uint32 catalogNumber = (uint32) atoi(string(name, 1, string::npos).c_str());
         return find(catalogNumber, 0);
-        
+
     }
     else if (compareIgnoringCase(name, HDCatalogPrefix, HDCatalogPrefix.length()) == 0)
     {
@@ -152,6 +152,17 @@ Star* StarDatabase::find(const string& name) const
     }
 }
 
+std::vector<std::string> StarDatabase::getCompletion(const string& name) const
+{
+    std::vector<std::string> completion;
+
+    // only named stars are supported by completion.
+    if (!name.empty() && names != NULL)
+        return names->getCompletion(name);
+    else
+        return completion;
+}
+
 
 // Return the name for the star with specified catalog number.  The returned
 // string will be:
@@ -341,7 +352,8 @@ bool StarDatabase::loadBinary(istream& in)
 	    DPRINTF(0, "Warning, distance of star # %ld of %12.2f ly seems excessive (parallax: %2.5f)!\n", catNo, distance, parallax);
 	}
 #endif // DEBUG
-        star->setPosition(astro::equatorialToCelestialCart(RA, dec, (float) distance));
+        Point3d pos = astro::equatorialToCelestialCart((double) RA, (double) dec, distance);
+        star->setPosition(Point3f((float) pos.x, (float) pos.y, (float) pos.z));
 
 	// Use apparent magnitude and distance to determine the absolute
 	// magnitude of the star.
@@ -490,7 +502,9 @@ static Star* CreateStar(uint32 catalogNumber, Hash* starData)
 
     // Convert ra from degrees to hours for eqToCelCart function
     ra /= 15.0;
-    star->setPosition(astro::equatorialToCelestialCart((float) ra, (float) dec, (float) distance));
+
+    Point3d pos = astro::equatorialToCelestialCart(ra, dec, distance);
+    star->setPosition(Point3f((float) pos.x, (float) pos.y, (float) pos.z));
 
     return star;
 }
@@ -510,6 +524,17 @@ bool StarDatabase::load(istream& in)
         }
         uint32 catalogNumber = (uint32) tokenizer.getNumberValue();
 
+        string name;
+        if (tokenizer.nextToken() == Tokenizer::TokenString)
+        {
+            // A star name (or names) is present
+            name = tokenizer.getStringValue();
+        }
+        else
+        {
+            tokenizer.pushBack();
+        }
+
         Value* starDataValue = parser.readValue();
         if (starDataValue == NULL)
         {
@@ -530,7 +555,16 @@ bool StarDatabase::load(istream& in)
             // Ensure that the star array is large enough
             if (nStars == capacity)
             {
+                // Grow the array by 5%--this may be too little, but the
+                // assumption here is that there will be small numbers of
+                // stars in text files added to a big collection loaded from
+                // a binary file.
                 capacity = (int) (capacity * 1.05);
+
+                // 100 stars seems like a reasonable minimum
+                if (capacity < 100)
+                    capacity = 100;
+
                 Star* newStars = new Star[capacity];
                 if (newStars == NULL)
                 {
@@ -538,11 +572,34 @@ bool StarDatabase::load(istream& in)
                     return false;
                 }
 
-                copy(stars, stars + nStars, newStars);
-                delete[] stars;
+                if (stars != NULL)
+                {
+                    copy(stars, stars + nStars, newStars);
+                    delete[] stars;
+                }
                 stars = newStars;
             }
             stars[nStars++] = *star;
+
+            if (names != NULL && !name.empty())
+            {
+                // Iterate through the string for names delimited
+                // by ':', and insert them into the star database.
+                // Note that db->add() will skip empty names.
+                string::size_type startPos = 0; 
+                while (startPos != string::npos)
+                {
+                    string::size_type next = name.find(':', startPos);
+                    string::size_type length = string::npos;
+                    if (next != string::npos)
+                    {
+                        length = next - startPos;
+                        next++;
+                    }
+                    names->add(catalogNumber, name.substr(startPos, length));
+                    startPos = next;
+                }
+            }
         }
         else
         {
