@@ -17,51 +17,13 @@
 
 #include <celutil/util.h>
 #include <celutil/debug.h>
+#include <celutil/utf8.h>
 #include "celestia.h"
 #include "star.h"
 #include "starname.h"
 
 using namespace std;
 
-
-static const char *greekAlphabet[] =
-{
-    "Alpha",
-    "Beta",
-    "Gamma",
-    "Delta",
-    "Epsilon",
-    "Zeta",
-    "Eta",
-    "Theta",
-    "Iota",
-    "Kappa",
-    "Lambda",
-    "Mu",
-    "Nu",
-    "Xi",
-    "Omicron",
-    "Pi",
-    "Rho",
-    "Sigma",
-    "Tau",
-    "Upsilon",
-    "Phi",
-    "Chi",
-    "Psi",
-    "Omega"
-};
-
-static const char* canonicalAbbrevs[] =
-{
-    "ALF", "BET", "GAM", "DEL", "EPS", "ZET", "ETA", "TET",
-    "IOT", "KAP", "LAM", "MU" , "NU" , "XI" , "OMI", "PI" ,
-    "RHO", "SIG", "TAU", "UPS", "PHI", "CHI", "PSI", "OME",
-};
-
-static string noAbbrev("");
-
-
 StarNameDatabase::StarNameDatabase()
 {
 }
@@ -91,6 +53,20 @@ uint32 StarNameDatabase::findCatalogNumber(const string& name) const
         return iter->second;
 }
 
+std::vector<std::string> StarNameDatabase::getCompletion(const std::string& name) const
+{
+    std::vector<std::string> completion;
+    for (NameIndex::const_iterator iter = nameIndex.begin();
+         iter != nameIndex.end() ; iter++)
+    {
+        if (!UTF8StringCompare(iter->first, name, name.length()))
+        {
+            completion.push_back(iter->first);
+        }
+    }
+    return completion;
+}
+
 
 // Return the first name matching the catalog number or finalName()
 // if there are no matching names.  The first name *should* be the
@@ -251,46 +227,3 @@ uint32 StarNameDatabase::findName(string name) const
 }
 
 
-// Greek alphabet crud . . . should probably moved to it's own module.
-
-Greek* Greek::instance = NULL;
-
-Greek::Greek()
-{
-    nLetters = sizeof(greekAlphabet) / sizeof(greekAlphabet[0]);
-    names = new string[nLetters];
-    abbrevs = new string[nLetters];
-
-    for (int i = 0; i < nLetters; i++)
-    {
-        names[i] = string(greekAlphabet[i]);
-        abbrevs[i] = string(canonicalAbbrevs[i]);
-    }
-}
-
-Greek::~Greek()
-{
-    delete[] names;
-    delete[] abbrevs;
-}
-
-const string& Greek::canonicalAbbreviation(const std::string& letter)
-{
-    if (instance == NULL)
-        instance = new Greek();
-
-    int i;
-    for (i = 0; i < Greek::instance->nLetters; i++)
-    {
-        if (compareIgnoringCase(letter, instance->names[i]) == 0)
-            return instance->abbrevs[i];
-    }
-
-    for (i = 0; i < Greek::instance->nLetters; i++)
-    {
-        if (compareIgnoringCase(letter, instance->abbrevs[i]) == 0)
-            return instance->abbrevs[i];
-    }
-
-    return noAbbrev;
-}
