@@ -43,7 +43,11 @@ Color StellarClass::getApparentColor(StellarClass::SpectralClass sc) const
     case StellarClass::Spectral_R:
     case StellarClass::Spectral_S:
     case StellarClass::Spectral_N:
+    case StellarClass::Spectral_C:
         return Color(1.0f, 0.4f, 0.4f);
+    case StellarClass::Spectral_L:
+    case StellarClass::Spectral_T:
+        return Color(0.75f, 0.2f, 0.2f);
     default:
         // TODO: Figure out reasonable colors for Wolf-Rayet stars,
         // white dwarfs, and other oddities
@@ -82,7 +86,7 @@ char* StellarClass::str(char* buf, unsigned int buflen) const
     }
     else if (st == StellarClass::NormalStar)
     {
-	s0[0] = "OBAFGKMRSNWW?"[(unsigned int) getSpectralClass()];
+	s0[0] = "OBAFGKMRSNWW?LTC"[(unsigned int) getSpectralClass()];
         s0[1] = '\0';
 	s1[0] = "0123456789"[getSpectralSubclass()];
         s1[1] = '\0';
@@ -205,6 +209,9 @@ StellarClass StellarClass::parse(const std::string& s)
     case 'S':
         specClass = StellarClass::Spectral_N;
         break;
+    case 'C':
+        specClass = StellarClass::Spectral_C;
+        break;
     case 'W':
         i++;
         if (starType[i] == 'C')
@@ -214,7 +221,12 @@ StellarClass StellarClass::parse(const std::string& s)
         else
             i--;
         break;
-
+    case 'L':
+        specClass = StellarClass::Spectral_L;
+        break;
+    case 'T':
+        specClass = StellarClass::Spectral_T;
+        break;
     case 'D':
         type = StellarClass::WhiteDwarf;
         return StellarClass(type, specClass, 0, lum);
