@@ -10,6 +10,7 @@
 #include <cctype>
 #include <cmath>
 #include <iomanip>
+#include <celutil/utf8.h>
 #include "tokenizer.h"
 
 
@@ -25,7 +26,8 @@ Tokenizer::Tokenizer(istream* _in) :
     haveValidNumber(false),
     haveValidName(false),
     haveValidString(false),
-    pushedBack(false)
+    pushedBack(false),
+    lineNum(1)
 {
 }
 
@@ -84,7 +86,7 @@ Tokenizer::TokenType Tokenizer::nextToken()
                 sign = -1;
                 integerValue = 0;
             }
-            else if (isalpha(nextChar))
+            else if (isalpha(nextChar) || nextChar == '_')
             {
                 state = NameState;
                 textToken += (char) nextChar;
@@ -139,7 +141,7 @@ Tokenizer::TokenType Tokenizer::nextToken()
             break;
 
         case NameState:
-            if (isalpha(nextChar) || isdigit(nextChar))
+            if (isalpha(nextChar) || isdigit(nextChar) || nextChar == '_')
             {
                 state = NameState;
                 textToken += (char) nextChar;
@@ -178,21 +180,30 @@ Tokenizer::TokenType Tokenizer::nextToken()
             if (nextChar == '\\')
             {
                 textToken += '\\';
+                state = StringState;
             }
             else if (nextChar == 'n')
             {
                 textToken += '\n';
+                state = StringState;
             }
             else if (nextChar == '"')
             {
                 textToken += '"';
+                state = StringState;
+            }
+            else if (nextChar == 'u')
+            {
+                unicodeValue = 0;
+                unicodeEscapeDigits = 0;
+                state = UnicodeEscapeState;
             }
             else
             {
                 newToken = TokenError;
                 syntaxError("Unknown escape code in string");
+                state = StringState;
             }
-            state = StringState;
             break;
 
         case NumberState:
@@ -295,6 +306,33 @@ Tokenizer::TokenType Tokenizer::nextToken()
                 syntaxError("'.' in stupid place");
             }
             break;
+
+        case UnicodeEscapeState:
+            if (isxdigit(nextChar))
+            {
+                unsigned int digitValue;
+                if (nextChar >= 'a' && nextChar <= 'f')
+                    digitValue = nextChar - 'a' + 10;
+                else if (nextChar >= 'A' && nextChar <= 'F')
+                    digitValue = nextChar - 'A' + 10;
+                else
+                    digitValue = nextChar - '0';
+                unicodeValue = (unicodeValue << 4) + digitValue;
+                unicodeEscapeDigits++;
+                if (unicodeEscapeDigits == 4)
+                {
+                    char utf8Encoded[7];
+                    UTF8Encode((wchar_t) unicodeValue, utf8Encoded);
+                    textToken += utf8Encoded;
+                    state = StringState;
+                }
+            }
+            else
+            {
+                state = ErrorState;
+                syntaxError("Bad Unicode escape in string");
+            }
+            break;
         }
 
         if (newToken == TokenBegin)
@@ -348,7 +386,11 @@ string Tokenizer::getStringValue()
 
 int Tokenizer::readChar()
 {
-    return (int) in->get();
+    int c = (int) in->get();
+    if (c == '\n')
+        lineNum++;
+
+    return c;
 }
 
 void Tokenizer::syntaxError(const char* message)
@@ -357,9 +399,9 @@ void Tokenizer::syntaxError(const char* message)
 }
 
 
-int Tokenizer::getLineNumber()
+int Tokenizer::getLineNumber() const
 {
-    return 0;
+    return lineNum;
 }
 
 #if 0
