@@ -12,6 +12,7 @@
 #include <celmath/mathlib.h>
 #include <celmath/vecmath.h>
 #include <celmath/intersect.h>
+#include <celutil/utf8.h>
 #include "astro.h"
 #include "3dsmesh.h"
 #include "meshmanager.h"
@@ -107,6 +108,38 @@ SolarSystem* Universe::getSolarSystem(const Star* star) const
 }
 
 
+// A more general version of the method above--return the solar system
+// that contains an object, or NULL if there is no solar sytstem.
+SolarSystem* Universe::getSolarSystem(const Selection& sel) const
+{
+    switch (sel.getType())
+    {
+    case Selection::Type_Star:
+        return getSolarSystem(sel.star());
+
+    case Selection::Type_Body:
+        {
+            PlanetarySystem* system = sel.body()->getSystem();
+            while (system != NULL)
+            {
+                Body* parent = system->getPrimaryBody();
+                if (parent != NULL)
+                    system = parent->getSystem();
+                else
+                    return getSolarSystem(Selection(system->getStar()));
+            }
+            return NULL;
+        }
+
+    case Selection::Type_Location:
+        return getSolarSystem(Selection(sel.location()->getParentBody()));
+
+    default:
+        return NULL;
+    }
+}
+
+
 // Create a new solar system for a star and return a pointer to it; if it
 // already has a solar system, just return a pointer to the existing one.
 SolarSystem* Universe::createSolarSystem(Star* star) const
@@ -180,6 +213,12 @@ void Universe::unmarkObject(const Selection& sel, int priority)
 }
 
 
+void Universe::unmarkAll()
+{
+    markers->erase(markers->begin(), markers->end());
+}
+
+
 bool Universe::isMarked(const Selection& sel, int priority) const
 {
     for (MarkerList::iterator iter = markers->begin();
@@ -236,8 +275,8 @@ static bool ApproxPlanetPickTraversal(Body* body, void* info)
 {
     PlanetPickInfo* pickInfo = (PlanetPickInfo*) info;
 
-    // Reject invisible bodies
-    if (body->getClassification() == Body::Invisible)
+    // Reject invisible bodies and bodies that don't exist at the current time
+    if (body->getClassification() == Body::Invisible || !body->extant(pickInfo->jd))
         return true;
 
     Point3d bpos = body->getHeliocentricPosition(pickInfo->jd);
@@ -280,9 +319,10 @@ static bool ExactPlanetPickTraversal(Body* body, void* info)
 
     // Test for intersection with the bounding sphere
     if (body->getClassification() != Body::Invisible &&
+        body->extant(pickInfo->jd) &&
         testIntersection(pickInfo->pickRay, Sphered(bpos, radius), distance))
     {
-        if (body->getMesh() == InvalidResource)
+        if (body->getModel() == InvalidResource)
         {
             // There's no mesh, so the object is an ellipsoid.  If it's
             // oblate, do a ray intersection test to see if the object was
@@ -322,10 +362,10 @@ static bool ExactPlanetPickTraversal(Body* body, void* info)
             r.origin.z *= s;
             r.direction *= s;
 
-            Mesh* mesh = GetMeshManager()->find(body->getMesh());
-            if (mesh != NULL)
+            Model* model = GetModelManager()->find(body->getModel());
+            if (model != NULL)
             {
-                if (!mesh->pick(r, distance))
+                if (!model->pick(r, distance))
                     distance = -1.0;
             }
         }
@@ -387,7 +427,7 @@ Selection Universe::pickPlanet(SolarSystem& solarSystem,
 	// sufficiently close to the pickRay 
 
 	solarSystem.getPlanets()->traverse(ApproxPlanetPickTraversal,
-					 (void*) &pickInfo);
+                                           (void*) &pickInfo);
         if (pickInfo.closestBody == closestBody)
 	  return  Selection(closestBody); 
         // Nothing else around, select the body and return
@@ -585,7 +625,7 @@ Selection Universe::pickStar(const UniversalCoord& origin,
     {
         Vec3f axis = direction ^ n;
         axis.normalize();
-        rotation.setAxisAngle(axis, (float) 2.0 * asin(sinAngle2));
+        rotation.setAxisAngle(axis, (float) (2.0 * asin(sinAngle2)));
     }
     StarPicker picker(o, direction, tolerance);
     starCatalog->findVisibleStars(picker,
@@ -664,6 +704,106 @@ Selection Universe::pick(const UniversalCoord& origin,
 }
 
 
+// Search by name for an immediate child of the specified object.
+Selection Universe::findChildObject(const Selection& sel,
+                                    const string& name) const
+{
+    switch (sel.getType())
+    {
+    case Selection::Type_Star:
+        {
+            SolarSystem* sys = getSolarSystem(sel.star());
+            if (sys != NULL)
+            {
+                PlanetarySystem* planets = sys->getPlanets();
+                if (planets != NULL)
+                    return Selection(planets->find(name, false));
+            }
+        }
+        break;
+
+    case Selection::Type_Body:
+        {
+            // First, search for a satellite
+            PlanetarySystem* sats = sel.body()->getSatellites();
+            if (sats != NULL)
+            {
+                Body* body = sats->find(name, false);
+                if (body != NULL)
+                    return Selection(body);
+            }
+
+            // If a satellite wasn't found, check this object's locations
+            Location* loc = sel.body()->findLocation(name);
+            if (loc != NULL)
+                return Selection(loc);
+        }
+        break;
+
+    case Selection::Type_Location:
+        // Locations have no children
+        break;
+
+    case Selection::Type_DeepSky:
+        // Deep sky objects have no children
+        break;
+
+    default:
+        break;
+    }
+
+    return Selection();
+}
+
+
+// Search for a name within an object's context.  For stars, planets (bodies),
+// and locations, the context includes all bodies in the associated solar
+// system.  For locations and planets, the context additionally includes
+// sibling or child locations, respectively.
+Selection Universe::findObjectInContext(const Selection& sel,
+                                        const string& name) const
+{
+    Body* contextBody = NULL;
+
+    switch (sel.getType())
+    {
+    case Selection::Type_Body:
+        contextBody = sel.body();
+        break;
+
+    case Selection::Type_Location:
+        contextBody = sel.location()->getParentBody();
+        break;
+
+    default:
+        break;
+    }
+
+    // First, search for bodies . . .
+    SolarSystem* sys = getSolarSystem(sel);
+    if (sys != NULL)
+    {
+        PlanetarySystem* planets = sys->getPlanets();
+        if (planets != NULL)
+        {
+            Body* body = planets->find(name, true);
+            if (body != NULL)
+                return Selection(body);
+        }
+    }
+
+    // . . . and then locations.
+    if (contextBody != NULL)
+    {
+        Location* loc = contextBody->findLocation(name);
+        if (loc != NULL)
+            return Selection(loc);
+    }
+
+    return Selection();
+}
+
+
 // Select an object by name, with the following priority:
 //   1. Try to look up the name in the star catalog
 //   2. Search the deep sky catalog for a matching name.
@@ -671,15 +811,15 @@ Selection Universe::pick(const UniversalCoord& origin,
 //      about which solar systems are relevant, and let the caller pass them
 //      to us to search.
 Selection Universe::find(const string& s,
-                         PlanetarySystem** solarSystems,
-                         int nSolarSystems)
+                         Selection* contexts,
+                         int nContexts)
 {
     if (deepSkyCatalog != NULL)
     {
         for (DeepSkyCatalog::const_iterator iter = deepSkyCatalog->begin();
              iter != deepSkyCatalog->end(); iter++)
         {
-            if (compareIgnoringCase((*iter)->getName(), s) == 0)
+            if (UTF8StringCompare((*iter)->getName(), s) == 0)
                 return Selection(*iter);
         }
     }
@@ -688,14 +828,11 @@ Selection Universe::find(const string& s,
     if (star != NULL)
         return Selection(star);
 
-    for (int i = 0; i < nSolarSystems; i++)
+    for (int i = 0; i < nContexts; i++)
     {
-        if (solarSystems[i] != NULL)
-        {
-            Body* body = solarSystems[i]->find(s, true);
-            if (body != NULL)
-                return Selection(body);
-        }
+        Selection sel = findObjectInContext(contexts[i], s);
+        if (!sel.empty())
+            return sel;
     }
 
     return Selection();
@@ -709,35 +846,20 @@ Selection Universe::find(const string& s,
 // variable in Unix and Windows.  Typically, the solar system will be one
 // in which the user is currently located.
 Selection Universe::findPath(const string& s,
-                             PlanetarySystem* solarSystems[],
-                             int nSolarSystems)
+                             Selection contexts[],
+                             int nContexts)
 {
     string::size_type pos = s.find('/', 0);
 
+    // No delimiter found--just do a normal find.
     if (pos == string::npos)
-        return find(s, solarSystems, nSolarSystems);
+        return find(s, contexts, nContexts);
 
+    // Find the base object
     string base(s, 0, pos);
-    Selection sel = find(base, solarSystems, nSolarSystems);
-    if (sel.empty())
-        return sel;
+    Selection sel = find(base, contexts, nContexts);
 
-    if (sel.deepsky != NULL)
-        return Selection();
-
-    const PlanetarySystem* worlds = NULL;
-    if (sel.body != NULL)
-    {
-        worlds = sel.body->getSatellites();
-    }
-    else if (sel.star != NULL)
-    {
-        SolarSystem* ssys = getSolarSystem(sel.star);
-        if (ssys != NULL)
-            worlds = ssys->getPlanets();
-    }
-
-    while (worlds != NULL)
+    while (!sel.empty() && pos != string::npos)
     {
         string::size_type nextPos = s.find('/', pos + 1);
         string::size_type len;
@@ -745,26 +867,129 @@ Selection Universe::findPath(const string& s,
             len = s.size() - pos - 1;
         else
             len = nextPos - pos - 1;
-
         string name = string(s, pos + 1, len);
-        
-        Body* body = worlds->find(name);
-        if (body == NULL)
+
+        sel = findChildObject(sel, name);
+
+        pos = nextPos;
+    }
+
+    return sel;
+}
+
+
+std::vector<std::string> Universe::getCompletion(const string& s,
+                                                 Selection* contexts,
+                                                 int nContexts,
+                                                 bool withLocations)
+{
+    std::vector<std::string> completion;
+
+    // Solar bodies first
+    for (int i = 0; i < nContexts; i++)
+    {
+        if (withLocations && contexts[i].getType() == Selection::Type_Body)
         {
-            return Selection();
+            std::vector<Location*>* locations = contexts[i].body()->getLocations();
+            if (locations != NULL)
+            {
+                for (vector<Location*>::const_iterator iter = locations->begin();
+                     iter != locations->end(); iter++)
+                {
+                    if (!UTF8StringCompare(s, (*iter)->getName(), s.length()))
+                        completion.push_back((*iter)->getName());
+                }
+            }
         }
-        else if (nextPos == string::npos)
+
+        SolarSystem* sys = getSolarSystem(contexts[i]);
+        if (sys != NULL)
         {
-            return Selection(body);
-        }
-        else
-        {
-            worlds = body->getSatellites();
-            pos = nextPos;
+            PlanetarySystem* planets = sys->getPlanets();
+            if (planets != NULL)
+            {
+                std::vector<std::string> bodies = planets->getCompletion(s);
+                completion.insert(completion.end(),
+                                  bodies.begin(), bodies.end());
+            }
         }
     }
 
-    return Selection();
+    // Deep sky object
+    if (deepSkyCatalog != NULL)
+    {
+        for (DeepSkyCatalog::const_iterator iter = deepSkyCatalog->begin();
+             iter != deepSkyCatalog->end(); iter++)
+        {
+            if (UTF8StringCompare((*iter)->getName(), s, s.length()) == 0)
+                completion.push_back(Selection(*iter).getName());
+        }
+    }
+
+    // finally stars;
+    std::vector<std::string> stars = starCatalog->getCompletion(s);
+    completion.insert(completion.end(), stars.begin(), stars.end());
+
+    return completion;
+}
+
+std::vector<std::string> Universe::getCompletionPath(const string& s,
+                                                     Selection* contexts,
+                                                     int nContexts,
+                                                     bool withLocations)
+{
+    std::vector<std::string> completion;
+    std::vector<std::string> locationCompletion;
+    
+    string::size_type pos = s.rfind('/', s.length());
+
+    if (pos == string::npos)
+        return getCompletion(s, contexts, nContexts, withLocations);
+
+    string base(s, 0, pos);
+    Selection sel = findPath(base, contexts, nContexts);
+
+    if (sel.empty())
+        return completion;
+
+    if (sel.getType() == Selection::Type_DeepSky)
+    {
+        completion.push_back(sel.deepsky()->getName());
+        return completion;
+    }
+
+    PlanetarySystem* worlds = NULL;
+    if (sel.getType() == Selection::Type_Body)
+    {
+        worlds = sel.body()->getSatellites();
+        std::vector<Location*>* locations = sel.body()->getLocations();
+        if (locations != NULL && withLocations)
+        {
+            string search = s.substr(pos + 1);
+            for (vector<Location*>::const_iterator iter = locations->begin();
+                 iter != locations->end(); iter++)
+            {
+                if (!UTF8StringCompare(search, (*iter)->getName(),
+                                       search.length()))
+                {
+                    locationCompletion.push_back((*iter)->getName());
+                }
+            }
+        }
+    }
+    else if (sel.getType() == Selection::Type_Star)
+    {
+        SolarSystem* ssys = getSolarSystem(sel.star());
+        if (ssys != NULL)
+            worlds = ssys->getPlanets();
+    }
+
+    if (worlds != NULL)
+        completion = worlds->getCompletion(s.substr(pos + 1), false);
+
+    completion.insert(completion.end(), locationCompletion.begin(), locationCompletion.end());
+
+    return completion;
 }
 
 
