@@ -50,14 +50,29 @@ class Universe
                                 const Vec3f&, float faintest,
                                 float tolerance = 0.0f);
     Selection find(const std::string& s,
-                   PlanetarySystem** solarSystems = NULL,
-                   int nSolarSystems = 0);
+                   Selection* contexts = NULL,
+                   int nContexts = 0);
     Selection findPath(const std::string& s,
-                       PlanetarySystem** solarSystems = NULL,
-                       int nSolarSystems = 0);
+                       Selection* contexts = NULL,
+                       int nContexts = 0);
+    Selection findChildObject(const Selection& sel,
+                              const string& name) const;
+    Selection findObjectInContext(const Selection& sel,
+                                  const string& name) const;
+
+    std::vector<std::string> getCompletion(const std::string& s,
+                                           Selection* contexts = NULL,
+                                           int nContexts = 0,
+                                           bool withLocations = false);
+    std::vector<std::string> getCompletionPath(const std::string& s,
+                                               Selection* contexts = NULL,
+                                               int nContexts = 0,
+                                               bool withLocations = false);
+
 
     SolarSystem* getNearestSolarSystem(const UniversalCoord& position) const;
     SolarSystem* getSolarSystem(const Star* star) const;
+    SolarSystem* getSolarSystem(const Selection&) const;
     SolarSystem* createSolarSystem(Star* star) const;
 
     void markObject(const Selection&,
@@ -66,6 +81,7 @@ class Universe
                     Marker::Symbol symbol,
                     int priority);
     void unmarkObject(const Selection&, int priority);
+    void unmarkAll();
     bool isMarked(const Selection&, int priority) const;
     MarkerList* getMarkers() const;
 
