@@ -23,6 +23,7 @@ unsigned int vp::diffuseHaze = 0;
 unsigned int vp::diffuseBump = 0;
 unsigned int vp::diffuseBumpHaze = 0;
 unsigned int vp::shadowTexture = 0;
+unsigned int vp::multiShadow = 0;
 unsigned int vp::everything = 0;
 unsigned int vp::diffuseTexOffset = 0;
 unsigned int vp::ringIllum = 0;
@@ -30,6 +31,8 @@ unsigned int vp::ringShadow = 0;
 unsigned int vp::cometTail = 0;
 unsigned int vp::nightLights = 0;
 unsigned int vp::glossMap = 0;
+unsigned int vp::perFragmentSpecular = 0;
+unsigned int vp::perFragmentSpecularAlpha = 0;
 
 
 class VertexProcessorNV : public VertexProcessor
@@ -127,7 +130,7 @@ static bool LoadNvVertexProgram(const string& filename, unsigned int& id)
 
 static int findLineNumber(const string& s, int index)
 {
-    if (index >= s.length())
+    if (index >= (int)s.length())
         return -1;
 
     int lineno = 1;
@@ -247,6 +250,19 @@ VertexProcessor* vp::initARB()
         return NULL;
     if (!LoadARBVertexProgram("shaders/glossmap_arb.vp", glossMap))
         return NULL;
+
+    // Load vertex programs that are only required with fragment programs
+    if (ExtensionSupported("GL_NV_fragment_program") ||
+        ExtensionSupported("GL_ARB_fragment_program"))
+    {
+        if (!LoadARBVertexProgram("shaders/multishadow_arb.vp", multiShadow))
+            return NULL;
+        if (!LoadARBVertexProgram("shaders/texphong_arb.vp", perFragmentSpecular))
+            return NULL;
+        if (!LoadARBVertexProgram("shaders/texphong_alpha_arb.vp", perFragmentSpecularAlpha))
+            return NULL;
+    }
+
     cout << "All ARB vertex programs loaded successfully.\n";
 
     return new VertexProcessorARB();
@@ -363,7 +379,13 @@ static unsigned int parameterMappings[] =
     90, // Constant0 - relevant for NV_vertex_program only
      0, // Unused
     41, // TexGen_S,
-    42, // TexGen_Y
+    42, // TexGen_T
+     0, // TexGen_S2
+     0, // TexGen_T2
+     0, // TexGen_S3
+     0, // TexGen_T3
+     0, // TexGen_S4
+     0, // TexGen_T4
 };
 
 VertexProcessorNV::VertexProcessorNV()
