@@ -27,7 +27,6 @@
 #include "favorites.h"
 #include "destination.h"
 #include "moviecapture.h"
-//#define CELX
 #ifdef CELX
 #include "celx.h"
 #endif
@@ -64,6 +63,8 @@ class View
     float height;
     int renderFlags;
     int labelMode;
+    float zoom;
+    float alternateZoom;
 
     void walkTreeResize(View*, int);
     bool walkTreeResizeDelta(View*, float, bool);
@@ -81,6 +82,26 @@ class CelestiaCore // : public Watchable<CelestiaCore>
         ControlKey   = 0x10,
     };
 
+    enum CursorShape {
+        ArrowCursor         = 0,
+        UpArrowCursor       = 1,
+        CrossCursor         = 2,
+        InvertedCrossCursor = 3,
+        WaitCursor          = 4,
+        BusyCursor          = 5,
+        IbeamCursor         = 6,
+        SizeVerCursor       = 7,
+        SizeHorCursor       = 8,
+        SizeBDiagCursor     = 9,
+        SizeFDiagCursor     = 10,
+        SizeAllCursor       = 11,
+        SplitVCursor        = 12,
+        SplitHCursor        = 13,
+        PointingHandCursor  = 14,
+        ForbiddenCursor     = 15,
+        WhatsThisCursor     = 16,
+    };
+    
     enum {
         Joy_XAxis           = 0,
         Joy_YAxis           = 1,
@@ -133,6 +154,7 @@ class CelestiaCore // : public Watchable<CelestiaCore>
         Key_NumPad7         = 31,
         Key_NumPad8         = 32,
         Key_NumPad9         = 33,
+        Key_BackTab         = 127,
         KeyCount            = 128,
     };
 
@@ -145,8 +167,16 @@ class CelestiaCore // : public Watchable<CelestiaCore>
         AmbientLightChanged   = 16,
         FaintestChanged       = 32,
         HistoryChanged        = 64,
+        TextEnterModeChanged  = 128,
     };
 
+    enum
+    {
+        KbNormal         = 0,
+        KbAutoComplete   = 1,
+        KbPassToScript   = 2,
+    };        
+
     typedef void (*ContextMenuFunc)(float, float, Selection);
 
  public:
@@ -170,13 +200,16 @@ class CelestiaCore // : public Watchable<CelestiaCore>
     void setHistoryCurrent(std::vector<Url>::size_type curr);
 
     // event processing methods
-    void charEntered(char);
-    void keyDown(int);
-    void keyUp(int);
+    void charEntered(const char*, int modifiers = 0);
+    void charEntered(char, int modifiers = 0);
+    void keyDown(int key, int modifiers = 0);
+    void keyUp(int key, int modifiers = 0);
     void mouseWheel(float, int);
     void mouseButtonDown(float, float, int);
     void mouseButtonUp(float, float, int);
     void mouseMove(float, float, int);
+    void mouseMove(float, float);
+    void pickView(float, float);
     void joystickAxis(int axis, float amount);
     void joystickButton(int button, bool down);
     void resize(GLsizei w, GLsizei h);
@@ -203,6 +236,7 @@ class CelestiaCore // : public Watchable<CelestiaCore>
     void setTimeZoneBias(int);
     std::string getTimeZoneName() const;
     void setTimeZoneName(const std::string&);
+    void setTextEnterMode(int);
     int getTextEnterMode() const;
 
     void initMovieCapture(MovieCapture*);
@@ -227,16 +261,29 @@ class CelestiaCore // : public Watchable<CelestiaCore>
     void setFaintest(float);
     void setFaintestAutoMag();
 
-    void splitView(View::Type);
-    void singleView();
-    void deleteView();
+    void splitView(View::Type type, View* av = NULL, float splitPos = 0.5f);
+    void singleView(View* av = NULL);
+    void deleteView(View* v = NULL);
     bool getFramesVisible() const;
     void setFramesVisible(bool);
     bool getActiveFrameVisible() const;
     void setActiveFrameVisible(bool);
+    bool getLightDelayActive() const;
+    void setLightDelayActive(bool);
+    int getScreenDpi() const;
+    void setScreenDpi(int);
+    int getDistanceToScreen() const;
+    void setDistanceToScreen(int);
+
+    void setFOVFromZoom();
+    void setZoomFromFOV();
 
     void flash(const std::string&, double duration = 1.0);
 
+    CelestiaConfig* getConfig() const;
+
+    void notifyWatchers(int);
+
     class Alerter
     {
     public:
@@ -245,12 +292,23 @@ class CelestiaCore // : public Watchable<CelestiaCore>
     };
 
     void setAlerter(Alerter*);
+    Alerter* getAlerter() const;
 
+    class CursorHandler
+    {
+    public:
+        virtual ~CursorHandler() {};
+        virtual void setCursorShape(CursorShape) = 0;
+        virtual CursorShape getCursorShape() const = 0;
+    };
+    
+    void setCursorHandler(CursorHandler*);
+    CursorHandler* getCursorHandler() const;
+    
  private:
     bool readStars(const CelestiaConfig&);
     void renderOverlay();
     void fatalError(const std::string&);
-    void notifyWatchers(int);
 
  private:
     CelestiaConfig* config;
@@ -277,25 +335,24 @@ class CelestiaCore // : public Watchable<CelestiaCore>
     double messageStart;
     double messageDuration;
     std::string typedText;
-    bool textEnterMode;
+    std::vector<std::string> typedTextCompletion;
+    int typedTextCompletionIdx;
+    int textEnterMode;
     int hudDetail;
     bool wireframe;
     bool editMode;
     bool altAzimuthMode;
+    bool showConsole;
     bool lightTravelFlag;
     double flashFrameStart;
 
     Timer* timer;
 
-    CommandSequence* currentScript;
-    CommandSequence* initScript;
-    CommandSequence* demoScript;
     Execution* runningScript;
     ExecutionEnvironment* execEnv;
 
 #ifdef CELX
     LuaState* celxScript;
-    double scriptAwakenTime;
 #endif // CELX    
 
     int timeZoneBias;              // Diff in secs between local time and GMT
@@ -318,10 +375,12 @@ class CelestiaCore // : public Watchable<CelestiaCore>
     double currentTime;
     double timeScale;
     bool paused;
+    bool scriptPaused;
 
     Vec3f joystickRotation;
     bool joyButtonsPressed[JoyButtonCount];
     bool keysPressed[KeyCount];
+    bool shiftKeysPressed[KeyCount];
     double KeyAccel;
 
     MovieCapture* movieCapture;
@@ -333,6 +392,8 @@ class CelestiaCore // : public Watchable<CelestiaCore>
 
     Alerter* alerter;
     std::vector<CelestiaWatcher*> watchers;
+    CursorHandler* cursorHandler;
+    CursorShape defaultCursorShape;
     
     std::vector<Url> history;
     std::vector<Url>::size_type historyCurrent;
@@ -343,6 +404,14 @@ class CelestiaCore // : public Watchable<CelestiaCore>
     bool showActiveViewFrame;
     bool showViewFrames;
     View *resizeSplit;
+
+    int screenDpi;
+    int distanceToScreen;
+
+#ifdef CELX
+    friend View* getViewByObserver(CelestiaCore*, Observer*);
+    friend void getObservers(CelestiaCore*, std::vector<Observer*>&);
+#endif
 };
 
 #endif // _CELESTIACORE_H_
