@@ -18,6 +18,18 @@
 using namespace std;
 
 
+static unsigned int getUint(Hash* params,
+                            const string& paramName,
+                            unsigned int defaultValue)
+{
+    double value = 0.0;
+    if (params->getNumber(paramName, value))
+        return (unsigned int) value;
+    else
+        return defaultValue;
+}
+
+
 CelestiaConfig* ReadCelestiaConfig(string filename)
 {
     ifstream configFile(filename.c_str());
@@ -70,11 +82,31 @@ CelestiaConfig* ReadCelestiaConfig(string filename)
     configParams->getString("LabelFont", config->labelFont);
     configParams->getString("TitleFont", config->titleFont);
     configParams->getString("LogoTexture", config->logoTextureFile);
+    configParams->getString("Cursor", config->cursor);
     // configParams->getNumber("LogoWidth", config->logoWidth);
     // configParams->getNumber("LogoHeight", config->logoHeight);
 
+    double aaSamples = 1;
+    configParams->getNumber("AntialiasingSamples", aaSamples);
+    config->aaSamples = (unsigned int) aaSamples;
+
+    config->hdr = false;
+    configParams->getBoolean("HighDynamicRange", config->hdr);
+
     config->rotateAcceleration = 120.0f;
     configParams->getNumber("RotateAcceleration", config->rotateAcceleration);
+    config->mouseRotationSensitivity = 1.0f;
+    configParams->getNumber("MouseRotationSensitivity", config->mouseRotationSensitivity);
+    config->scriptScreenshotCount = 0.0f;
+    configParams->getNumber("ScriptScreenshotCount", config->scriptScreenshotCount);    
+    configParams->getString("ScriptScreenshotDirectory", config->scriptScreenshotDirectory);
+    config->scriptSystemAccessPolicy = "ask";
+    configParams->getString("ScriptSystemAccessPolicy", config->scriptSystemAccessPolicy);
+
+    config->ringSystemSections = getUint(configParams, "RingSystemSections", 100);
+    config->orbitPathSamplePoints = getUint(configParams, "OrbitPathSamplePoints", 100);
+    config->shadowTextureSize = getUint(configParams, "ShadowTextureSize", 256);
+    config->eclipseTextureSize = getUint(configParams, "EclipseTextureSize", 128);
 
     Value* solarSystemsVal = configParams->getValue("SolarSystemCatalogs");
     if (solarSystemsVal != NULL)
