@@ -33,8 +33,23 @@ struct CelestiaConfig
     std::string labelFont;
     std::string titleFont;
     std::string logoTextureFile;
+    std::string cursor;
     std::vector<std::string> ignoreGLExtensions;
     float rotateAcceleration;
+    float mouseRotationSensitivity;
+    float scriptScreenshotCount;
+    std::string scriptScreenshotDirectory;
+    std::string scriptSystemAccessPolicy;
+
+    // Renderer detail options
+    unsigned int shadowTextureSize;
+    unsigned int eclipseTextureSize;
+    unsigned int ringSystemSections;
+    unsigned int orbitPathSamplePoints;
+
+    unsigned int aaSamples;
+
+    bool hdr;
 };
 
 
