@@ -15,7 +15,6 @@
  *                                                                         *
  ***************************************************************************/
 
-#include <time.h>
 #include <qvbox.h>
 #include <kmainwindow.h>
 #include <kpushbutton.h>
@@ -32,12 +31,23 @@
 #include <kkeydialog.h>
 #include <qspinbox.h>
 #include <qlabel.h>
+#include <qlayout.h>
+
 
 #include "kdeapp.h"
 #include "kdepreferencesdialog.h"
 #include "celengine/render.h"
 #include "celengine/glcontext.h"
 
+static uint32 FilterOtherLocations = ~(Location::City |
+                    Location::Observatory |
+                    Location::LandingSite |
+                    Location::Crater |
+                    Location::Mons |
+                    Location::Terra |
+                    Location::Vallis |
+                    Location::Mare);
+
 KdePreferencesDialog::KdePreferencesDialog(QWidget* parent, CelestiaCore* core) :
     KDialogBase (KDialogBase::IconList, "",
     KDialogBase::Ok | KDialogBase::Apply | KDialogBase::Cancel, KDialogBase::Ok,parent)
@@ -107,10 +117,6 @@ KdePreferencesDialog::KdePreferencesDialog(QWidget* parent, CelestiaCore* core)
     actionColl->action("showEclipseShadows")->connect(showEclipseShadowsCheck, SIGNAL(clicked()), SLOT(activate()));
     showEclipseShadowsCheck->setChecked(renderFlags & Renderer::ShowEclipseShadows);
 
-    QCheckBox* showStarsAsPointsCheck = new QCheckBox(i18n("Stars As Points"), showGroup);
-    actionColl->action("showStarsAsPoints")->connect(showStarsAsPointsCheck, SIGNAL(clicked()), SLOT(activate()));
-    showStarsAsPointsCheck->setChecked(renderFlags & Renderer::ShowStarsAsPoints);
-
     QCheckBox* showRingShadowsCheck = new QCheckBox(i18n("Ring Shadows"), showGroup);
     actionColl->action("showRingShadows")->connect(showRingShadowsCheck, SIGNAL(clicked()), SLOT(activate()));
     showRingShadowsCheck->setChecked(renderFlags & Renderer::ShowRingShadows);
@@ -131,35 +137,86 @@ KdePreferencesDialog::KdePreferencesDialog(QWidget* parent, CelestiaCore* core)
     QVBox* vbox1 = new QVBox(renderFrame);
     int labelMode = appCore->getRenderer()->getLabelMode();
     savedLabelMode = labelMode;
+    int orbitMask = appCore->getRenderer()->getOrbitMask();
+    savedOrbitMask = orbitMask;
+
+    QGroupBox* labelGroup = new QGroupBox(0, Qt::Horizontal, i18n("Orbits / Labels"), vbox1);
+    QGridLayout* labelGroupLayout = new QGridLayout( labelGroup->layout() );
+    labelGroupLayout->setAlignment( Qt::AlignTop );
+
+    QLabel* orbitsLabel = new QLabel(i18n("Orbits"), labelGroup);
+    labelGroupLayout->addWidget(orbitsLabel, 0, 0);
+    QLabel* labelsLabel = new QLabel(i18n("Labels"), labelGroup);
+    labelGroupLayout->addWidget(labelsLabel, 0, 1);
 
-    QGroupBox* labelGroup = new QGroupBox(1, Qt::Horizontal, i18n("Labels"), vbox1);
     QCheckBox* showStarLabelsCheck = new QCheckBox(i18n("Stars"), labelGroup);
     actionColl->action("showStarLabels")->connect(showStarLabelsCheck, SIGNAL(clicked()), SLOT(activate()));
     showStarLabelsCheck->setChecked(labelMode & Renderer::StarLabels);
+    labelGroupLayout->addWidget(showStarLabelsCheck, 1, 1);
 
+
+    QCheckBox* showPlanetOrbitsCheck = new QCheckBox("", labelGroup);
+    actionColl->action("showPlanetOrbits")->connect(showPlanetOrbitsCheck, SIGNAL(clicked()), SLOT(activate()));
+    showPlanetOrbitsCheck->setChecked(orbitMask & Body::Planet);
+    labelGroupLayout->addWidget(showPlanetOrbitsCheck, 3, 0, Qt::AlignHCenter);
     QCheckBox* showPlanetLabelsCheck = new QCheckBox(i18n("Planets"), labelGroup);
     actionColl->action("showPlanetLabels")->connect(showPlanetLabelsCheck, SIGNAL(clicked()), SLOT(activate()));
     showPlanetLabelsCheck->setChecked(labelMode & Renderer::PlanetLabels);
+    labelGroupLayout->addWidget(showPlanetLabelsCheck, 3, 1);
 
+    QCheckBox* showMoonOrbitsCheck = new QCheckBox("", labelGroup);
+    actionColl->action("showMoonOrbits")->connect(showMoonOrbitsCheck, SIGNAL(clicked()), SLOT(activate()));
+    showMoonOrbitsCheck->setChecked(orbitMask & Body::Moon);
+    labelGroupLayout->addWidget(showMoonOrbitsCheck, 4, 0, Qt::AlignHCenter);
     QCheckBox* showMoonLabelsCheck = new QCheckBox(i18n("Moons"), labelGroup);
     actionColl->action("showMoonLabels")->connect(showMoonLabelsCheck, SIGNAL(clicked()), SLOT(activate()));
     showMoonLabelsCheck->setChecked(labelMode & Renderer::MoonLabels);
+    labelGroupLayout->addWidget(showMoonLabelsCheck, 4, 1);
+
+    QCheckBox* showCometOrbitsCheck = new QCheckBox("", labelGroup);
+    actionColl->action("showCometOrbits")->connect(showCometOrbitsCheck, SIGNAL(clicked()), SLOT(activate()));
+    showCometOrbitsCheck->setChecked(orbitMask & Body::Comet);
+    labelGroupLayout->addWidget(showCometOrbitsCheck, 5, 0, Qt::AlignHCenter);
+    QCheckBox* showCometLabelsCheck = new QCheckBox(i18n("Comets"), labelGroup);
+    actionColl->action("showCometLabels")->connect(showCometLabelsCheck, SIGNAL(clicked()), SLOT(activate()));
+    showCometLabelsCheck->setChecked(labelMode & Renderer::CometLabels);
+    labelGroupLayout->addWidget(showCometLabelsCheck, 5, 1);
 
     QCheckBox* showConstellationLabelsCheck = new QCheckBox(i18n("Constellations"), labelGroup);
     actionColl->action("showConstellationLabels")->connect(showConstellationLabelsCheck, SIGNAL(clicked()), SLOT(activate()));
     showConstellationLabelsCheck->setChecked(labelMode & Renderer::ConstellationLabels);
+    labelGroupLayout->addWidget(showConstellationLabelsCheck, 6, 1);
 
     QCheckBox* showGalaxyLabelsCheck = new QCheckBox(i18n("Galaxies"), labelGroup);
     actionColl->action("showGalaxyLabels")->connect(showGalaxyLabelsCheck, SIGNAL(clicked()), SLOT(activate()));
     showGalaxyLabelsCheck->setChecked(labelMode & Renderer::GalaxyLabels);
+    labelGroupLayout->addWidget(showGalaxyLabelsCheck, 7, 1);
 
+    QCheckBox* showAsteroidOrbitsCheck = new QCheckBox("", labelGroup);
+    actionColl->action("showAsteroidOrbits")->connect(showAsteroidOrbitsCheck, SIGNAL(clicked()), SLOT(activate()));
+    showAsteroidOrbitsCheck->setChecked(orbitMask & Body::Asteroid);
+    labelGroupLayout->addWidget(showAsteroidOrbitsCheck, 8, 0, Qt::AlignHCenter);
     QCheckBox* showAsteroidLabelsCheck = new QCheckBox(i18n("Asteroids"), labelGroup);
     actionColl->action("showAsteroidLabels")->connect(showAsteroidLabelsCheck, SIGNAL(clicked()), SLOT(activate()));
     showAsteroidLabelsCheck->setChecked(labelMode & Renderer::AsteroidLabels);
+    labelGroupLayout->addWidget(showAsteroidLabelsCheck, 8, 1);
 
+    QCheckBox* showSpacecraftOrbitsCheck = new QCheckBox("", labelGroup);
+    actionColl->action("showSpacecraftOrbits")->connect(showSpacecraftOrbitsCheck, SIGNAL(clicked()), SLOT(activate()));
+    showSpacecraftOrbitsCheck->setChecked(orbitMask & Body::Spacecraft);
+    labelGroupLayout->addWidget(showSpacecraftOrbitsCheck, 9, 0, Qt::AlignHCenter);
     QCheckBox* showSpacecraftLabelsCheck = new QCheckBox(i18n("Spacecrafts"), labelGroup);
     actionColl->action("showSpacecraftLabels")->connect(showSpacecraftLabelsCheck, SIGNAL(clicked()), SLOT(activate()));
     showSpacecraftLabelsCheck->setChecked(labelMode & Renderer::SpacecraftLabels);
+    labelGroupLayout->addWidget(showSpacecraftLabelsCheck, 9, 1);
+
+    QCheckBox* showLocationLabelsCheck = new QCheckBox(i18n("Locations"), labelGroup);
+    actionColl->action("showLocationLabels")->connect(showLocationLabelsCheck, SIGNAL(clicked()), SLOT(activate()));
+    showLocationLabelsCheck->setChecked(labelMode & Renderer::LocationLabels);
+    labelGroupLayout->addWidget(showLocationLabelsCheck, 10, 1);
+
+    QSpacerItem* spacer = new QSpacerItem( 151, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );
+    labelGroupLayout->addItem( spacer, 0, 2 );
 
     savedAmbientLightLevel = int(appCore->getRenderer()->getAmbientLightLevel() * 100);
     QGroupBox* ambientLightGroup = new QGroupBox(1, Qt::Vertical, i18n("Ambient Light"), vbox1);
@@ -192,10 +249,77 @@ KdePreferencesDialog::KdePreferencesDialog(QWidget* parent, CelestiaCore* core)
     infoTextCombo->setCurrentItem(savedHudDetail);
     ((KdeApp*)parent)->connect(infoTextCombo, SIGNAL(activated(int)), SLOT(slotHudDetail(int)));
 
+    QGroupBox* fovGroup = new QGroupBox(2, Qt::Horizontal, i18n("Automatic FOV"), vbox1);
+    new QLabel(i18n("Screen DPI: "), fovGroup);
+    new QLabel(QString::number(appCore->getScreenDpi(), 10), fovGroup);
+    new QLabel(i18n("Viewing Distance (cm): "), fovGroup);
+    dtsSpin = new QSpinBox(10, 300, 1, fovGroup);
+    savedDistanceToScreen = appCore->getDistanceToScreen();
+    dtsSpin->setValue(savedDistanceToScreen / 10);
+    connect(dtsSpin, SIGNAL(valueChanged(int)), SLOT(slotDistanceToScreen(int)));
+
+    // Locations page
+    Observer* obs = appCore->getSimulation()->getActiveObserver();
+    savedLocationFilter = obs->getLocationFilter();
+
+    QFrame* locationsFrame = addPage(i18n("Locations"), i18n("Locations"),
+       KGlobal::iconLoader()->loadIcon("package_network", KIcon::NoGroup));
+    QVBoxLayout* locationsLayout = new QVBoxLayout( locationsFrame );
+    locationsLayout->setAutoAdd(TRUE);
+    locationsLayout->setAlignment(Qt::AlignTop);
+
+    QCheckBox* showCityLocationsCheck = new QCheckBox(i18n("Cities"), locationsFrame);
+    actionColl->action("showCityLocations")->connect(showCityLocationsCheck, SIGNAL(clicked()), SLOT(activate()));
+    showCityLocationsCheck->setChecked(savedLocationFilter & Location::City);
+
+    QCheckBox* showObservatoryLocationsCheck = new QCheckBox(i18n("Observatories"), locationsFrame);
+    actionColl->action("showObservatoryLocations")->connect(showObservatoryLocationsCheck, SIGNAL(clicked()), SLOT(activate()));
+    showObservatoryLocationsCheck->setChecked(savedLocationFilter & Location::Observatory);
+
+    QCheckBox* showLandingSiteLocationsCheck = new QCheckBox(i18n("Landing Sites"), locationsFrame);
+    actionColl->action("showLandingSiteLocations")->connect(showLandingSiteLocationsCheck, SIGNAL(clicked()), SLOT(activate()));
+    showLandingSiteLocationsCheck->setChecked(savedLocationFilter & Location::LandingSite);
+
+    QCheckBox* showCraterLocationsCheck = new QCheckBox(i18n("Craters"), locationsFrame);
+    actionColl->action("showCraterLocations")->connect(showCraterLocationsCheck, SIGNAL(clicked()), SLOT(activate()));
+    showCraterLocationsCheck->setChecked(savedLocationFilter & Location::Crater);
+
+    QCheckBox* showMonsLocationsCheck = new QCheckBox(i18n("Mons"), locationsFrame);
+    actionColl->action("showMonsLocations")->connect(showMonsLocationsCheck, SIGNAL(clicked()), SLOT(activate()));
+    showMonsLocationsCheck->setChecked(savedLocationFilter & Location::Mons);
+
+    QCheckBox* showTerraLocationsCheck = new QCheckBox(i18n("Terra"), locationsFrame);
+    actionColl->action("showTerraLocations")->connect(showTerraLocationsCheck, SIGNAL(clicked()), SLOT(activate()));
+    showTerraLocationsCheck->setChecked(savedLocationFilter & Location::Terra);
+
+    QCheckBox* showVallisLocationsCheck = new QCheckBox(i18n("Vallis"), locationsFrame);
+    actionColl->action("showVallisLocations")->connect(showVallisLocationsCheck, SIGNAL(clicked()), SLOT(activate()));
+    showVallisLocationsCheck->setChecked(savedLocationFilter & Location::Vallis);
+
+    QCheckBox* showMareLocationsCheck = new QCheckBox(i18n("Mare"), locationsFrame);
+    actionColl->action("showMareLocations")->connect(showMareLocationsCheck, SIGNAL(clicked()), SLOT(activate()));
+    showMareLocationsCheck->setChecked(savedLocationFilter & Location::Mare);
+
+    QCheckBox* showOtherLocationsCheck = new QCheckBox(i18n("Other"), locationsFrame);
+    actionColl->action("showOtherLocations")->connect(showOtherLocationsCheck, SIGNAL(clicked()), SLOT(activate()));
+    showOtherLocationsCheck->setChecked(savedLocationFilter & FilterOtherLocations);
+
+    QGroupBox* minFeatureSizeGroup = new QGroupBox(1, Qt::Vertical, i18n("Minimum Feature Size"), locationsFrame);
+    minFeatureSizeGroup->setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Minimum));
+    savedMinFeatureSize = (int)appCore->getRenderer()->getMinimumFeatureSize();
+    QSlider* minFeatureSizeSlider = new QSlider(1, 1000, 1, savedMinFeatureSize, Qt::Horizontal, minFeatureSizeGroup);
+    connect(minFeatureSizeSlider, SIGNAL(valueChanged(int)), SLOT(slotMinFeatureSize(int)));
+    minFeatureSizeLabel = new QLabel(minFeatureSizeGroup);
+    minFeatureSizeLabel->setMinimumWidth(40);
+    minFeatureSizeLabel->setAlignment(Qt::AlignRight | Qt::AlignVCenter);
+    sprintf(buff, "%d", savedMinFeatureSize);
+    minFeatureSizeLabel->setText(buff);
+
+
     // Time page
     timeHasChanged = false;
     QVBox* timeFrame = addVBoxPage(i18n("Date/Time"), i18n("Date/Time"),
-        KGlobal::iconLoader()->loadIcon("kalarm", KIcon::NoGroup));
+        KGlobal::iconLoader()->loadIcon("clock", KIcon::NoGroup));
 
     savedDisplayLocalTime = appCore->getTimeZoneBias();
     QGroupBox* displayTimezoneGroup = new QGroupBox(1, Qt::Vertical, i18n("Display"), timeFrame);
@@ -208,7 +332,7 @@ KdePreferencesDialog::KdePreferencesDialog(QWidget* parent, CelestiaCore* core)
     displayTimezoneGroup->addSpace(0);
 
     QGroupBox* setTimezoneGroup = new QGroupBox(1, Qt::Horizontal, i18n("Set"), timeFrame);
-    new QLabel(i18n("Local Time is only supported for dates between 1902 and 2037."), setTimezoneGroup);   
+    new QLabel(i18n("Local Time is only supported for dates between 1902 and 2037.\n"), setTimezoneGroup);   
     QHBox *hbox2 = new QHBox(setTimezoneGroup);
     new QLabel(i18n("Timezone: "), hbox2);
     setTimezoneCombo = new QComboBox(hbox2);
@@ -268,10 +392,10 @@ KdePreferencesDialog::KdePreferencesDialog(QWidget* parent, CelestiaCore* core)
 
     Selection selection = appCore->getSimulation()->getSelection();
     std::string sel_name;
-    if (selection.body != 0) {
+    if (selection.body() != 0) {
         QHBox* ltBox = new QHBox(setTimezoneGroup);
         char time[50];
-        sel_name = selection.body->getName();
+        sel_name = selection.body()->getName();
         Vec3d v = selection.getPosition(appCore->getSimulation()->getTime()) - 
                   appCore->getSimulation()->getObserver().getPosition();
         double dist = astro::lightYearsToKilometers(v.length()*1e-6);
@@ -282,11 +406,17 @@ KdePreferencesDialog::KdePreferencesDialog(QWidget* parent, CelestiaCore* core)
         if (lt_h == 0) snprintf(time, 50, "%d min %02.1f s", lt_m, lt_s);
         else snprintf(time, 50, "%ld h %02d min %02.1f s", lt_h, lt_m, lt_s);
         
-        new QLabel(i18n("Selection: " + QString(sel_name.c_str()) 
+        new QLabel(i18n("\nSelection: " + QString(sel_name.c_str()) 
         + QString("\nLight Travel Time: %2").arg(time)), ltBox);
         
         KPushButton *ltButton = new KPushButton(ltBox);
-        ltButton->setText(i18n("Subtract"));
+        ltButton->setToggleButton(true);        
+
+        if (!appCore->getLightDelayActive())
+            ltButton->setText(i18n("Include Light Travel Time"));
+        else
+            ltButton->setText(i18n("Ignore Light Travel Time "));
+
         ltButton->setSizePolicy(nowButtonSizePolicy);
         connect(ltButton, SIGNAL(clicked()), SLOT(ltSubstract()));
     }
@@ -373,16 +503,24 @@ void KdePreferencesDialog::ltSubstract() {
     
     Selection selection = appCore->getSimulation()->getSelection();
     std::string sel_name;
-    if (selection.body != 0) {
-        sel_name = selection.body->getName();
-        Vec3d v = selection.getPosition(d) - 
+
+    // LT-delay only for solar bodies && target-speed < 0.99 c
+
+    if (selection.body() != 0 &&
+        (appCore->getSimulation()->getTargetSpeed() < 0.99 *
+            astro::kilometersToMicroLightYears(astro::speedOfLight))) {
+        sel_name = selection.body()->getName();
+        Vec3d v = selection.getPosition(d) -
                   appCore->getSimulation()->getObserver().getPosition();
+        appCore->setLightDelayActive(!appCore->getLightDelayActive());
         double dist = astro::lightYearsToKilometers(v.length()*1e-6);
         double lt = dist / astro::speedOfLight;
+        if (appCore->getLightDelayActive())
         d -= lt / 86400.;
-    }    
-    
+        else
+            d += lt / 86400.;
     setTime(d);
+    }
 }
 
 void KdePreferencesDialog::setTime(double d) {
@@ -441,11 +579,14 @@ void KdePreferencesDialog::slotOk() {
 void KdePreferencesDialog::slotCancel() {
     appCore->getRenderer()->setRenderFlags(savedRendererFlags);
     appCore->getRenderer()->setLabelMode(savedLabelMode);
+    appCore->getRenderer()->setOrbitMask(savedOrbitMask);
     appCore->getRenderer()->setAmbientLightLevel(savedAmbientLightLevel/100.);
     appCore->getSimulation()->setFaintestVisible(savedFaintestVisible/100.);
     appCore->setHudDetail(savedHudDetail);
     appCore->getRenderer()->getGLContext()->setRenderPath((GLContext::GLRenderPath)savedRenderPath);
-
+    appCore->setDistanceToScreen(savedDistanceToScreen);
+    appCore->getSimulation()->getActiveObserver()->setLocationFilter(savedLocationFilter);
+    appCore->getRenderer()->setMinimumFeatureSize(savedMinFeatureSize);
     reject();
 }
 
@@ -457,6 +598,9 @@ void KdePreferencesDialog::slotApply() {
     savedHudDetail = appCore->getHudDetail();
     savedDisplayLocalTime = appCore->getTimeZoneBias();
     savedRenderPath = (int)appCore->getRenderer()->getGLContext()->getRenderPath();
+    savedDistanceToScreen = appCore->getDistanceToScreen();
+    savedLocationFilter = appCore->getSimulation()->getActiveObserver()->getLocationFilter();
+    savedMinFeatureSize = appCore->getRenderer()->getMinimumFeatureSize();
 
     keyChooser->commitChanges();
 
@@ -501,6 +645,14 @@ void KdePreferencesDialog::slotFaintestVisible(int m) {
     faintestLabel->setText(buff);
 }
 
+void KdePreferencesDialog::slotMinFeatureSize(int s) {
+    char buff[20];
+
+    parent->slotMinFeatureSize(s);
+    sprintf(buff, "%d", s);
+    minFeatureSizeLabel->setText(buff);
+}
+
 void KdePreferencesDialog::slotAmbientLightLevel(int l) {
     char buff[20];
     
@@ -522,6 +674,10 @@ void KdePreferencesDialog::slotRenderPath(int pathIdx) {
     setRenderPathLabel();
 }
 
+void KdePreferencesDialog::slotDistanceToScreen(int dts) {
+    appCore->setDistanceToScreen(dts * 10);
+}
+
 void KdePreferencesDialog::setRenderPathLabel() {
     switch(appCore->getRenderer()->getGLContext()->getRenderPath()) {
     case GLContext::GLPath_Basic:
