@@ -139,7 +139,7 @@ BOOL APIENTRY SolarSystemBrowserProc(HWND hDlg,
         }
         else if (LOWORD(wParam) == IDC_BUTTON_CENTER)
         {
-            browser->appCore->charEntered('C');
+            browser->appCore->charEntered('c');
         }
         else if (LOWORD(wParam) == IDC_BUTTON_GOTO)
         {
@@ -159,6 +159,14 @@ BOOL APIENTRY SolarSystemBrowserProc(HWND hDlg,
                 {
                     browser->appCore->getSimulation()->setSelection(Selection(body));
                 }
+                else
+                {
+                    // If the body is NULL, assume that this is the tree item for
+                    // the sun.
+                    const SolarSystem* solarSys = browser->appCore->getSimulation()->getNearestSolarSystem();
+                    if (solarSys != NULL && solarSys->getStar() != NULL)
+                        browser->appCore->getSimulation()->setSelection(Selection(const_cast<Star*>(solarSys->getStar())));
+                }
             }
         }
     }
@@ -179,3 +187,9 @@ SolarSystemBrowser::SolarSystemBrowser(HINSTANCE appInstance,
                              SolarSystemBrowserProc,
                              reinterpret_cast<LONG>(this));
 }
+
+
+SolarSystemBrowser::~SolarSystemBrowser()
+{
+    SetWindowLong(hwnd, DWL_USER, 0);
+}
