@@ -52,65 +52,104 @@ static BOOL APIENTRY ViewOptionsProc(HWND hDlg,
 
     case WM_COMMAND:
     {
-        //Take out a Renderer* for readability sake.
         Renderer* renderer = Dlg->appCore->getRenderer();
+        uint32 renderFlags = renderer->getRenderFlags();
+        uint32 labelMode = renderer->getLabelMode();
+        uint32 orbitMask = renderer->getOrbitMask();
 
-        if(LOWORD(wParam) == IDC_SHOWATMOSPHERES)
-            renderer->setRenderFlags(renderer->getRenderFlags() ^ renderer->ShowAtmospheres);
-        else if(LOWORD(wParam) == IDC_SHOWCELESTIALGRID)
-            renderer->setRenderFlags(renderer->getRenderFlags() ^ renderer->ShowCelestialSphere);
-        else if(LOWORD(wParam) == IDC_SHOWCLOUDS)
-            renderer->setRenderFlags(renderer->getRenderFlags() ^ renderer->ShowCloudMaps);
-        else if(LOWORD(wParam) == IDC_SHOWCONSTELLATIONS)
-            renderer->setRenderFlags(renderer->getRenderFlags() ^ renderer->ShowDiagrams);
-        else if(LOWORD(wParam) == IDC_SHOWECLIPSESHADOWS)
-            renderer->setRenderFlags(renderer->getRenderFlags() ^ renderer->ShowEclipseShadows);
-        else if(LOWORD(wParam) == IDC_SHOWGALAXIES)
-            renderer->setRenderFlags(renderer->getRenderFlags() ^ renderer->ShowGalaxies);
-        else if(LOWORD(wParam) == IDC_SHOWNIGHTSIDELIGHTS)
-            renderer->setRenderFlags(renderer->getRenderFlags() ^ renderer->ShowNightMaps);
-        else if(LOWORD(wParam) == IDC_SHOWORBITS)
-            renderer->setRenderFlags(renderer->getRenderFlags() ^ renderer->ShowOrbits);
-        else if(LOWORD(wParam) == IDC_SHOWPLANETS)
-            renderer->setRenderFlags(renderer->getRenderFlags() ^ renderer->ShowPlanets);
-        else if(LOWORD(wParam) == IDC_SHOWSTARS)
+        switch (LOWORD(wParam))
         {
-            renderer->setRenderFlags(renderer->getRenderFlags() ^ renderer->ShowStars);
-
-            HWND hAsPoints = GetDlgItem(hDlg, IDC_SHOWSTARSASPOINTS);
-            if(hAsPoints)
-                EnableWindow(hAsPoints, (BOOL)(renderer->getRenderFlags() & renderer->ShowStars));
-        }
-        else if(LOWORD(wParam) == IDC_SHOWSTARSASPOINTS)
-            renderer->setRenderFlags(renderer->getRenderFlags() ^ renderer->ShowStarsAsPoints);
-        else if(LOWORD(wParam) == IDC_SHOWCONSTELLATIONBORDERS)
-            renderer->setRenderFlags(renderer->getRenderFlags() ^ renderer->ShowBoundaries);
-        else if(LOWORD(wParam) == IDC_SHOWRINGSHADOWS)
-            renderer->setRenderFlags(renderer->getRenderFlags() ^ renderer->ShowRingShadows);
-        else if(LOWORD(wParam) == IDC_SHOWCOMETTAILS)
-            renderer->setRenderFlags(renderer->getRenderFlags() ^ renderer->ShowCometTails);
-        else if(LOWORD(wParam) == IDC_LABELCONSTELLATIONS)
-            renderer->setLabelMode(renderer->getLabelMode() ^ renderer->ConstellationLabels);
-        else if(LOWORD(wParam) == IDC_LABELGALAXIES)
-            renderer->setLabelMode(renderer->getLabelMode() ^ renderer->GalaxyLabels);
-        else if(LOWORD(wParam) == IDC_LABELPLANETS)
-            renderer->setLabelMode(renderer->getLabelMode() ^ renderer->PlanetLabels);
-        else if(LOWORD(wParam) == IDC_LABELMOONS)
-            renderer->setLabelMode(renderer->getLabelMode() ^ renderer->MoonLabels);
-        else if(LOWORD(wParam) == IDC_LABELSTARS)
-            renderer->setLabelMode(renderer->getLabelMode() ^ renderer->StarLabels);
-        else if(LOWORD(wParam) == IDC_LABELASTEROIDS)
-            renderer->setLabelMode(renderer->getLabelMode() ^ renderer->AsteroidLabels);
-        else if(LOWORD(wParam) == IDC_LABELSPACECRAFT)
-            renderer->setLabelMode(renderer->getLabelMode() ^ renderer->SpacecraftLabels);
-        else if(LOWORD(wParam) == IDC_INFOTEXT0)
+        case IDC_SHOWATMOSPHERES:
+            renderer->setRenderFlags(renderFlags ^ Renderer::ShowAtmospheres);
+            break;
+        case IDC_SHOWCELESTIALGRID:
+            renderer->setRenderFlags(renderFlags ^ Renderer::ShowCelestialSphere);
+            break;
+        case IDC_SHOWCLOUDS:
+            renderer->setRenderFlags(renderFlags ^ Renderer::ShowCloudMaps);
+            break;
+        case IDC_SHOWCONSTELLATIONS:
+            renderer->setRenderFlags(renderFlags ^ Renderer::ShowDiagrams);
+            break;
+        case IDC_SHOWECLIPSESHADOWS:
+            renderer->setRenderFlags(renderFlags ^ Renderer::ShowEclipseShadows);
+            break;
+        case IDC_SHOWGALAXIES:
+            renderer->setRenderFlags(renderFlags ^ Renderer::ShowGalaxies);
+            break;
+        case IDC_SHOWNIGHTSIDELIGHTS:
+            renderer->setRenderFlags(renderFlags ^ Renderer::ShowNightMaps);
+            break;
+        case IDC_SHOWORBITS:
+            renderer->setRenderFlags(renderFlags ^ Renderer::ShowOrbits);
+            break;
+        case IDC_PLANETORBITS:
+            renderer->setOrbitMask(orbitMask ^ Body::Planet);
+            break;
+        case IDC_MOONORBITS:
+            renderer->setOrbitMask(orbitMask ^ Body::Moon);
+            break;
+        case IDC_ASTEROIDORBITS:
+            renderer->setOrbitMask(orbitMask ^ Body::Asteroid);
+            break;
+        case IDC_COMETORBITS:
+            renderer->setOrbitMask(orbitMask ^ Body::Comet);
+            break;
+        case IDC_SPACECRAFTORBITS:
+            renderer->setOrbitMask(orbitMask ^ Body::Spacecraft);
+            break;
+        case IDC_SHOWPLANETS:
+            renderer->setRenderFlags(renderFlags ^ Renderer::ShowPlanets);
+            break;
+        case IDC_SHOWSTARS:
+            renderer->setRenderFlags(renderFlags ^ Renderer::ShowStars);
+            break;
+        case IDC_SHOWCONSTELLATIONBORDERS:
+            renderer->setRenderFlags(renderFlags ^ Renderer::ShowBoundaries);
+            break;
+        case IDC_SHOWRINGSHADOWS:
+            renderer->setRenderFlags(renderFlags ^ Renderer::ShowRingShadows);
+            break;
+        case IDC_SHOWCOMETTAILS:
+            renderer->setRenderFlags(renderFlags ^ Renderer::ShowCometTails);
+            break;
+        case IDC_SHOWMARKERS:
+            renderer->setRenderFlags(renderFlags ^ Renderer::ShowMarkers);
+            break;
+        case IDC_LABELCONSTELLATIONS:
+            renderer->setLabelMode(labelMode ^ Renderer::ConstellationLabels);
+            break;
+        case IDC_LABELGALAXIES:
+            renderer->setLabelMode(labelMode ^ Renderer::GalaxyLabels);
+            break;
+        case IDC_LABELPLANETS:
+            renderer->setLabelMode(labelMode ^ Renderer::PlanetLabels);
+            break;
+        case IDC_LABELMOONS:
+            renderer->setLabelMode(labelMode ^ Renderer::MoonLabels);
+            break;
+        case IDC_LABELSTARS:
+            renderer->setLabelMode(labelMode ^ Renderer::StarLabels);
+            break;
+        case IDC_LABELASTEROIDS:
+            renderer->setLabelMode(labelMode ^ Renderer::AsteroidLabels);
+            break;
+        case IDC_LABELCOMETS:
+            renderer->setLabelMode(labelMode ^ Renderer::CometLabels);
+            break;
+        case IDC_LABELSPACECRAFT:
+            renderer->setLabelMode(labelMode ^ Renderer::SpacecraftLabels);
+            break;
+        case IDC_INFOTEXT0:
             Dlg->appCore->setHudDetail(0);
-        else if(LOWORD(wParam) == IDC_INFOTEXT1)
+            break;
+        case IDC_INFOTEXT1:
             Dlg->appCore->setHudDetail(1);
-        else if(LOWORD(wParam) == IDC_INFOTEXT2)
+            break;
+        case IDC_INFOTEXT2:
             Dlg->appCore->setHudDetail(2);
-        else if (LOWORD(wParam) == IDOK)
-        {
+            break;
+        case IDOK:
             if (Dlg != NULL && Dlg->parent != NULL)
             {
                 SendMessage(Dlg->parent, WM_COMMAND, IDCLOSE,
@@ -118,14 +157,12 @@ static BOOL APIENTRY ViewOptionsProc(HWND hDlg,
             }
             EndDialog(hDlg, 0);
             return TRUE;
-        }
-        else if (LOWORD(wParam) == IDCANCEL)
-        {
+        case IDCANCEL:
             if (Dlg != NULL && Dlg->parent != NULL)
             {
-                // Reset render flags, label mode, and hud detail to initial values
+                // Reset render flags, label mode, and hud detail to
+                // initial values
                 Dlg->RestoreSettings(hDlg);
-
                 SendMessage(Dlg->parent, WM_COMMAND, IDCLOSE,
                             reinterpret_cast<LPARAM>(Dlg));
             }
@@ -196,56 +233,62 @@ ViewOptionsDialog::ViewOptionsDialog(HINSTANCE appInstance,
                              reinterpret_cast<LONG>(this));
 }
 
+
+static void dlgCheck(HWND hDlg, WORD item, uint32 flags, uint32 f)
+{
+    SendDlgItemMessage(hDlg, item, BM_SETCHECK,
+                       ((flags & f) != 0) ? BST_CHECKED : BST_UNCHECKED, 0);
+}
+
 void ViewOptionsDialog::SetControls(HWND hDlg)
 {
     int renderFlags = appCore->getRenderer()->getRenderFlags();
     int labelMode = appCore->getRenderer()->getLabelMode();
     int hudDetail = appCore->getHudDetail();
+    int orbitMask = appCore->getRenderer()->getOrbitMask();
 
     //Set checkboxes and radiobuttons
     SendDlgItemMessage(hDlg, IDC_SHOWATMOSPHERES, BM_SETCHECK,
-        ((renderFlags & appCore->getRenderer()->ShowAtmospheres) != 0)? BST_CHECKED:BST_UNCHECKED, 0);
+        (renderFlags & Renderer::ShowAtmospheres)? BST_CHECKED:BST_UNCHECKED, 0);
     SendDlgItemMessage(hDlg, IDC_SHOWCELESTIALGRID, BM_SETCHECK,
-        ((renderFlags & appCore->getRenderer()->ShowCelestialSphere) != 0)? BST_CHECKED:BST_UNCHECKED, 0);
+        (renderFlags & Renderer::ShowCelestialSphere)? BST_CHECKED:BST_UNCHECKED, 0);
     SendDlgItemMessage(hDlg, IDC_SHOWCLOUDS, BM_SETCHECK,
-        ((renderFlags & appCore->getRenderer()->ShowCloudMaps) != 0)? BST_CHECKED:BST_UNCHECKED, 0);
+        (renderFlags & Renderer::ShowCloudMaps)? BST_CHECKED:BST_UNCHECKED, 0);
     SendDlgItemMessage(hDlg, IDC_SHOWCONSTELLATIONS, BM_SETCHECK,
-        ((renderFlags & appCore->getRenderer()->ShowDiagrams) != 0)? BST_CHECKED:BST_UNCHECKED, 0);
+        (renderFlags & Renderer::ShowDiagrams)? BST_CHECKED:BST_UNCHECKED, 0);
     SendDlgItemMessage(hDlg, IDC_SHOWECLIPSESHADOWS, BM_SETCHECK,
-        ((renderFlags & appCore->getRenderer()->ShowEclipseShadows) != 0)? BST_CHECKED:BST_UNCHECKED, 0);
+        (renderFlags & Renderer::ShowEclipseShadows)? BST_CHECKED:BST_UNCHECKED, 0);
     SendDlgItemMessage(hDlg, IDC_SHOWGALAXIES, BM_SETCHECK,
-        ((renderFlags & appCore->getRenderer()->ShowGalaxies) != 0)? BST_CHECKED:BST_UNCHECKED, 0);
+        (renderFlags & Renderer::ShowGalaxies)? BST_CHECKED:BST_UNCHECKED, 0);
     SendDlgItemMessage(hDlg, IDC_SHOWNIGHTSIDELIGHTS, BM_SETCHECK,
-        ((renderFlags & appCore->getRenderer()->ShowNightMaps) != 0)? BST_CHECKED:BST_UNCHECKED, 0);
-    SendDlgItemMessage(hDlg, IDC_SHOWORBITS, BM_SETCHECK,
-        ((renderFlags & appCore->getRenderer()->ShowOrbits) != 0)? BST_CHECKED:BST_UNCHECKED, 0);
+        (renderFlags & Renderer::ShowNightMaps)? BST_CHECKED:BST_UNCHECKED, 0);
+    dlgCheck(hDlg, IDC_SHOWORBITS,       renderFlags, Renderer::ShowOrbits);
+    dlgCheck(hDlg, IDC_PLANETORBITS,     orbitMask,   Body::Planet);
+    dlgCheck(hDlg, IDC_MOONORBITS,       orbitMask,   Body::Moon);
+    dlgCheck(hDlg, IDC_ASTEROIDORBITS,   orbitMask,   Body::Asteroid);
+    dlgCheck(hDlg, IDC_COMETORBITS,      orbitMask,   Body::Comet);
+    dlgCheck(hDlg, IDC_SPACECRAFTORBITS, orbitMask,   Body::Spacecraft);
     SendDlgItemMessage(hDlg, IDC_SHOWPLANETS, BM_SETCHECK,
-        ((renderFlags & appCore->getRenderer()->ShowPlanets) != 0)? BST_CHECKED:BST_UNCHECKED, 0);
+        (renderFlags & Renderer::ShowPlanets)? BST_CHECKED:BST_UNCHECKED, 0);
     SendDlgItemMessage(hDlg, IDC_SHOWSTARS, BM_SETCHECK,
-        ((renderFlags & appCore->getRenderer()->ShowStars) != 0)? BST_CHECKED:BST_UNCHECKED, 0);
-    SendDlgItemMessage(hDlg, IDC_SHOWSTARSASPOINTS, BM_SETCHECK,
-        ((renderFlags & appCore->getRenderer()->ShowStarsAsPoints) != 0)? BST_CHECKED:BST_UNCHECKED, 0);
+        (renderFlags & Renderer::ShowStars)? BST_CHECKED:BST_UNCHECKED, 0);
     SendDlgItemMessage(hDlg, IDC_SHOWCONSTELLATIONBORDERS, BM_SETCHECK,
-        ((renderFlags & appCore->getRenderer()->ShowBoundaries) != 0)? BST_CHECKED:BST_UNCHECKED, 0);
+        (renderFlags & Renderer::ShowBoundaries)? BST_CHECKED:BST_UNCHECKED, 0);
     SendDlgItemMessage(hDlg, IDC_SHOWRINGSHADOWS, BM_SETCHECK,
-        ((renderFlags & appCore->getRenderer()->ShowRingShadows) != 0)? BST_CHECKED:BST_UNCHECKED, 0);
+        (renderFlags & Renderer::ShowRingShadows)? BST_CHECKED:BST_UNCHECKED, 0);
     SendDlgItemMessage(hDlg, IDC_SHOWCOMETTAILS, BM_SETCHECK,
-        ((renderFlags & appCore->getRenderer()->ShowCometTails) != 0)? BST_CHECKED:BST_UNCHECKED, 0);
+        (renderFlags & Renderer::ShowCometTails)? BST_CHECKED:BST_UNCHECKED, 0);
+    SendDlgItemMessage(hDlg, IDC_SHOWMARKERS, BM_SETCHECK,
+        (renderFlags & Renderer::ShowMarkers)? BST_CHECKED:BST_UNCHECKED, 0);
 
-    SendDlgItemMessage(hDlg, IDC_LABELCONSTELLATIONS, BM_SETCHECK,
-        ((labelMode & appCore->getRenderer()->ConstellationLabels) != 0)? BST_CHECKED:BST_UNCHECKED, 0);
-    SendDlgItemMessage(hDlg, IDC_LABELGALAXIES, BM_SETCHECK,
-        ((labelMode & appCore->getRenderer()->GalaxyLabels) != 0)? BST_CHECKED:BST_UNCHECKED, 0);
-    SendDlgItemMessage(hDlg, IDC_LABELPLANETS, BM_SETCHECK,
-        ((labelMode & appCore->getRenderer()->PlanetLabels) != 0)? BST_CHECKED:BST_UNCHECKED, 0);
-    SendDlgItemMessage(hDlg, IDC_LABELMOONS, BM_SETCHECK,
-        ((labelMode & appCore->getRenderer()->MoonLabels) != 0)? BST_CHECKED:BST_UNCHECKED, 0);
-    SendDlgItemMessage(hDlg, IDC_LABELSTARS, BM_SETCHECK,
-        ((labelMode & appCore->getRenderer()->StarLabels) != 0)? BST_CHECKED:BST_UNCHECKED, 0);
-    SendDlgItemMessage(hDlg, IDC_LABELASTEROIDS, BM_SETCHECK,
-        ((labelMode & appCore->getRenderer()->AsteroidLabels) != 0)? BST_CHECKED:BST_UNCHECKED, 0);
-    SendDlgItemMessage(hDlg, IDC_LABELSPACECRAFT, BM_SETCHECK,
-        ((labelMode & appCore->getRenderer()->SpacecraftLabels) != 0)? BST_CHECKED:BST_UNCHECKED, 0);
+    dlgCheck(hDlg, IDC_LABELCONSTELLATIONS, labelMode, Renderer::ConstellationLabels);
+    dlgCheck(hDlg, IDC_LABELGALAXIES,   labelMode, Renderer::GalaxyLabels);
+    dlgCheck(hDlg, IDC_LABELSTARS,      labelMode, Renderer::StarLabels);
+    dlgCheck(hDlg, IDC_LABELPLANETS,    labelMode, Renderer::PlanetLabels);
+    dlgCheck(hDlg, IDC_LABELMOONS,      labelMode, Renderer::MoonLabels);
+    dlgCheck(hDlg, IDC_LABELASTEROIDS,  labelMode, Renderer::AsteroidLabels);
+    dlgCheck(hDlg, IDC_LABELCOMETS,     labelMode, Renderer::CometLabels);
+    dlgCheck(hDlg, IDC_LABELSPACECRAFT, labelMode, Renderer::SpacecraftLabels);
 
     CheckRadioButton(hDlg, IDC_INFOTEXT0, IDC_INFOTEXT2, IDC_INFOTEXT0 + hudDetail);
 
